/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/


Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;
const ioService = Cc["@mozilla.org/network/io-service;1"]
                    .getService(Ci.nsIIOService);

function Protocol() {}

Protocol.prototype =
{
    classDescription : "Skin Protocol",
    classID : Components.ID("{7d88e2e0-28dd-11de-8c30-0800200c9a66}"),
    contractID : "@mozilla.org/network/protocol;1?name=skin",
    protocolFlags : Ci.nsIProtocolHandler.URI_NOAUTH | Ci.nsIProtocolHandler.URI_IS_UI_RESOURCE |
                    Ci.nsIProtocolHandler.URI_IS_LOCAL_RESOURCE | Ci.nsIProtocolHandler.URI_INHERITS_SECURITY_CONTEXT,
    scheme : "skin",
    defaultPort : -1,
    allowPort : function (port, scheme) { return false; },

    baseSkinRoot : "chrome://ttbasetheme/content/",
    activeSkinRoot : "chrome://tthome/skin/",

    QueryInterface: XPCOMUtils.generateQI([Ci.nsIProtocolHandler]),

    newURI : function(spec, charset, baseURI)
    {
        var url = Cc["@mozilla.org/network/standard-url;1"]
                    .createInstance(Ci.nsIStandardURL);
        url.init(Ci.nsIStandardURL.URLTYPE_NO_AUTHORITY, this.defaultPort, spec, charset, baseURI);
        return url;
    },

    newChannel : function(uri)
    {
        function chromeURLExists(url)
        {
            let scriptableStream = null;
            try {
                let ioService = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
                let input = ioService.newChannel(url, null, null).open();

                scriptableStream = Cc["@mozilla.org/scriptableinputstream;1"].getService(Ci.nsIScriptableInputStream);
                scriptableStream.init(input);
                scriptableStream.read(0);
                return true;
            } catch (e) {
                return false;
            } finally {
                try {
                    if (scriptableStream)
                        scriptableStream.close();
                } catch (e2) {}
            }
        }

        var path = uri.path.replace(/^\/+/, "").replace(/\?.*/, "");
        var result = null;

        let existsInActiveSkin = chromeURLExists(this.activeSkinRoot + path);
        if (/\.css$/.test(path) && existsInActiveSkin && chromeURLExists(this.baseSkinRoot + path))
        {
            let separator = Cc["@mozilla.org/io/string-input-stream;1"].createInstance(Ci.nsIStringInputStream);
            separator.setData("\n", -1);

            let stream = Cc["@mozilla.org/io/multiplex-input-stream;1"].createInstance(Ci.nsIMultiplexInputStream);
            stream.appendStream(ioService.newChannel(this.baseSkinRoot + path, null, null).open());
            stream.appendStream(separator);
            stream.appendStream(ioService.newChannel(this.activeSkinRoot + path, null, null).open());

            result = Cc["@mozilla.org/network/input-stream-channel;1"].createInstance(Ci.nsIInputStreamChannel);
            result.contentStream = stream;
            result.setURI(uri);
            result.QueryInterface(Ci.nsIChannel);
        }
        else if (existsInActiveSkin)
            result = ioService.newChannel(this.activeSkinRoot + path, null, null);
        else
            result = ioService.newChannel(this.baseSkinRoot + path, null, null);

        result.originalURI = uri;
        return result;
    }
};

var NSGetModule = XPCOMUtils.generateNSGetModule([Protocol]);
