/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/


function AddonManager()
{
}
AddonManager.prototype =
{
start : function()
    {
        var successCallback = makeCallback(this, this._saveAvailableUpdates);
        var errorCallback = function(e) {
            unimportantError(e);
            runAsyncDelayed(updateCheckFunction, 5 * 60 * 1000);
        }

        var me = this;
        var updateCheckFunction = function()
        {
            me.requestAddonUpdates(successCallback, errorCallback);
        }

        runAsyncDelayed(updateCheckFunction, 7000);
    },
requestAddonUpdates : function(successCallback, errorCallback)
    {
        var getNewestVersionsBody = <getnewestversions><contentidlist/></getnewestversions>;
        var extensions = ttExtension.prototype.importItemsFromExtensionManager();
        var themes = ttTheme.prototype.importItemsFromExtensionManager();
        for each (var item in extensions.concat(themes))
        {
            getNewestVersionsBody.contentidlist.appendChild(<content idurl={item.idurlWithVersion}/>);
        }

        var serverargs = {
            price: "free",
            includeMissing: false,
            operatingSystem : gApplication.settings.operatingSystemServerName,
        };

        var dev = gApplication.deviceManager.activeDevice;
        if (dev)
        {
            serverargs.deviceModel = dev.model.id;
            serverargs.deviceCapabilities = gApplication.deviceManager.activeDeviceCapabilitiesString;
            serverargs.deviceParameters = gApplication.deviceManager.activeDeviceParametersString;
            serverargs.deviceNavcoreVersion = dev.versionNavCore;
            serverargs.deviceNavcoreBuild = dev.buildNumberNavCore;
        }
        return rpc.call("shop", "getNewestVersions", serverargs,
                getNewestVersionsBody, RPC.METHOD_POST | RPC.USE_CACHE,
                function(result)
                {
                    var items = gApplication.itemFactory.createItemListFromServer(result);
                    items = items.filter(function(item) item instanceof ttAddon);
                    successCallback(items);
                }, errorCallback);
    },
getKnownAddonUpdates : function()
    {
        var extensionManager = Cc["@mozilla.org/extensions/manager;1"].getService(CI.nsIExtensionManager);
        var ds = extensionManager.datasource;
        var rdfService = Cc["@mozilla.org/rdf/rdf-service;1"].getService(CI.nsIRDFService);
        var link = rdfService.GetResource("http://www.tomtom.com/2007/em-rdf#updateXML");
        var containerUtils = Cc["@mozilla.org/rdf/container-utils;1"].getService(CI.nsIRDFContainerUtils);

        var result = [];
        var root = rdfService.GetResource("urn:mozilla:item:root");
        var rootContainer = containerUtils.MakeSeq(ds, root);
        var enumerator = rootContainer.GetElements();
        while (enumerator.hasMoreElements())
        {
            var addon = enumerator.getNext().QueryInterface(CI.nsIRDFResource);
            var target = ds.GetTarget(addon, link, true);
            if (target && target instanceof CI.nsIRDFLiteral)
                result.push(gApplication.itemFactory.createItemFromServer(new XML(target.Value)));
        }
        return result;
    },

    _saveAvailableUpdates : function(items)
    {
        if (items.length == 0)
            return;

        var hasNewUpdates = false;

        var extensionManager = Cc["@mozilla.org/extensions/manager;1"].getService(CI.nsIExtensionManager);
        var ds = extensionManager.datasource;
        var rdfService = Cc["@mozilla.org/rdf/rdf-service;1"].getService(CI.nsIRDFService);
        var link = rdfService.GetResource("http://www.tomtom.com/2007/em-rdf#updateXML");
        for each (var item in items)
        {
            var source = rdfService.GetResource("urn:mozilla:item:" + item.idstr);
            var newXML = item.sourceXML.toXMLString();

            var oldTarget = ds.GetTarget(source, link, true);
            if (oldTarget)
            {
                try {
                    var oldXML = oldTarget.QueryInterface(CI.nsIRDFLiteral).Value;
                    if (oldXML == newXML)
                        continue;

                    var oldUpdate = new ttAddon();
                    oldUpdate.importFromServer(new XML(oldXML));
                    if (item.newerThan(oldUpdate))
                        hasNewUpdates = true;
                } catch (e) { unimportantError(e); }

                ds.Unassert(source, link, oldTarget);
            }
            else
                hasNewUpdates = true;

            var target = rdfService.GetLiteral(item.sourceXML.toXMLString());
            ds.Assert(source, link, target, true);
        }
        ds.QueryInterface(CI.nsIRDFRemoteDataSource).Flush();

        if (hasNewUpdates)
            prefService().setBoolPref("extensions.update.notifyUser", true);
    },

  __end : null
}
