/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/


EXPORTED_SYMBOLS.push(
    "rpc",
    "RPC",
    "NetworkCall",
    "CombinedCall",
    "NoOpCall",
    "FailureAlreadyReportedToCallbackException"
);
function RPC()
{
    this._onlineObservers = new Array();
}
RPC.prototype =
{
init : function (successCallback, errorCallback)
    {
        assert(successCallback, "You need to wait for init to return");
        if (typeof successCallback != "function")
            throw new InvalidArgument("successCallback is not a function. Did you forget |new| before rpc.call?");
        if (errorCallback == null)
            errorCallback = error;

        if (typeof errorCallback != "function")
            throw new InvalidArgument("errorCallback is not a function.");
        if (this._inited)
        {
            try {
                successCallback();
            } catch (e) { errorCallback(e) };
            return new NoOpCall();
        }
        var me = this;
        return new rpcCall("redirect", "services",
                     { clientVersion: gApplication.settings.version },
                     null, RPC.METHOD_GET,
        function (result)
        {
            for each (var service in result.service)
            {
                var type = makeSafe.jsidentifer(service.@type);
                var url = makeSafe.url(service.@baseurl);
                me._serviceBaseURLs[type] = url;
            }
            me._inited = true;
            successCallback();
        },
        errorCallback);
    },
login : function (username, password, successCallback, errorCallback)
    {
        return this.call("account", "login",
            {username: username, password: password},
            null, RPC.METHOD_POST,
            makeCallback(this, function()
            {
                this.clearCache();
                successCallback();
            }), errorCallback);
    },
logout : function (successCallback, errorCallback)
    {
        this.clearCache();

        var callback = function ()
        {
            try
            {
                var cookieManager = Cc["@mozilla.org/cookiemanager;1"]
                                    .getService(CI.nsICookieManager);
                cookieManager.removeAll();
                successCallback();
            }
            catch (e)
            {
                errorCallback(e);
            }
        };
        return this.call("account", "logout", null, null, RPC.METHOD_POST,
              callback, callback);
    },
get isOnline()
    {
        return this._isOnline;
    },
    _setIsOnline : function(newVal)
    {
        newVal = newVal ? true : false;
        if (newVal == this._isOnline)
            return;{}
this._isOnline = newVal;
        for (var i = 0; i < this._onlineObservers.length; i++)
        {
            try {
                this._onlineObservers[i].changed();
            } catch (e) { error(e); }
        }
    },
    _isOnline : false,
    _onlineObservers : null,
    addOnlineObserver : function(observer)
    {
        this._onlineObservers.push(observer);
    },
    removeOnlineObserver : function(observer)
    {
        ArrayRemoveAllMatching(this._onlineObservers, observer);
    },
call : function (server, func, args, xmluploadbody, flags,
                     successCallback, errorCallback)
    {
        if (this._inited)
        {
            return new rpcCall(server, func, args, xmluploadbody, flags,
                                successCallback, errorCallback);
        }
        else
        {
            var calls = new CombinedCall();
            calls.addCall(this.init(
            function()
            {
                calls.addCall(new rpcCall(server, func, args, xmluploadbody, flags,
                                           successCallback, errorCallback));
            },
            errorCallback));
            return calls;
        }
    },
serviceBaseURL : function(serviceName)
    {
        try {
            var url = prefService().getCharPref("tomtom.server.serviceURL." + serviceName);
            return makeSafe.url(url);
        } catch (e if serviceName == "redirect") {
            throw this.stringBundle.GetStringFromName("serviceURLNotFound");
        } catch (e) {
        }
        return this._serviceBaseURLs[serviceName];
    },
contentBaseURI : function()
    {
        return this.serviceBaseURL("shop");
    },
clearCache : function()
    {
        this._cache = [];
    },
get stringBundle()
    {
        if (this._stringBundle)
            return this._stringBundle;
        this._stringBundle = getStringBundle("chrome://tthome/locale/logic/comm.properties");

        return this._stringBundle;
    },

_serviceBaseURLs : new Object(),
    _inited : false,
    _cache : new Array(),
    _stringBundle : null,
    __end: null
}

const rpc = new RPC();
RPC._METHOD_GET_INTERNAL = 1;
RPC.METHOD_POST = 2;
RPC.USE_CACHE = 4;
RPC.DRY_RUN = 8;
RPC.METHOD_GET = RPC._METHOD_GET_INTERNAL | RPC.USE_CACHE;
function rpcCall(server, func, args, xmluploadbody, flags,
                 successCallback, errorCallback)
{
    try
    {
        this.init(rpc, server, func, args, xmluploadbody, flags,
                successCallback, errorCallback);
        this.setup();
        this.send();
    } catch (e if e instanceof FailureAlreadyReportedToCallbackException) {}
};
rpcCall.prototype =
{
    _rpc : null,
    _mFinishedCallback : null,
    _reloginRequest : null,
    _reloginAttempts : 0,
    _isDone : true,
    errorCode : 0,
    errorStr : null,
    cancelException : null,
init : function (rpc, server, func, args, xmluploadbody, flags,
                     successCallback, errorCallback)
    {

        this._rpc = rpc;
        if (!server || !func)
            throw new InvalidArgument("need server and function ID");
        if (!args)
            args = {};
        this.server = makeSafe.alphanum(server);
        func = makeSafe.string(func);
        if (!func.match(/^[a-zA-Z0-9/]*$/))
            throw new Malformed("Bad character in function name");
        this.func = makeSafe.alphanum(func);

        flags = makeSafe.integer(flags);
        if (flags & RPC._METHOD_GET_INTERNAL)
            this.post = false;
        else if (flags & RPC.METHOD_POST)
            this.post = true;
        else
            throw new InvalidArgument("request method has to be specified");

        this.useCache = (flags & RPC.USE_CACHE);
        this.dryrun = (flags & RPC.DRY_RUN);{}
if (successCallback == null)
            successCallback = function() {};
        if (errorCallback == null)
            errorCallback = error;
			

        if (typeof successCallback != "function")
            throw new InvalidArgument("successCallback is not a function");
        if (typeof errorCallback != "function")
            throw new InvalidArgument("errorCallback is not a function");
        this.successCallback = successCallback;
        this.errorCallback = errorCallback;
        if (xmluploadbody && typeof(xmluploadbody) != "xml")
            throw new InvalidArgument("xmluploadbody is not E4X or null");
        this.xmluploadbody = xmluploadbody;
        this.args = args;

        if (!this.post && this.xmluploadbody)
            throw new InvalidArgument("Upload body cannot be used with method GET, server doesn't support this");

        if (!this._rpc.serviceBaseURL(this.server))
        {
            var me = this;
            runAsync(function()
            {
                me.success = false;
                me.errorStr = "Server type " + me.server + " not known";
                me.errorCode = -5;
                me._doneCallbacks();
            });
            throw new FailureAlreadyReportedToCallbackException();
        }
    },
setup : function()
    {
        this.url = makeSafe.url(this.func, this._rpc.serviceBaseURL(this.server));
        var me = this;
        if (getIOService().offline)
        {
            runAsync(function()
            {
                me.success = false;
                me.cancelException = NetUtil.exceptionForStatusCode(Components.results.NS_ERROR_OFFLINE, me.url.spec);
                me._doneCallbacks();
            });
            throw new FailureAlreadyReportedToCallbackException();
        }

        this.request = new XMLHttpRequest();
        this.request.open(this.post ? "POST" : "GET", this.url.spec);
        this.request.channel.loadGroup = null;
        this.request.channel.loadFlags |= CI.nsICachingChannel.LOAD_BYPASS_LOCAL_CACHE;
        this._setGenericArguments();
        for (var i in this.args)
            this.setArg(i, this.args[i]);

        this.request.onload = function() { me._done(true); }
        this.request.onerror = function() { me._done(false); }
        this.funcsuccess = false;
        this.errorCode = -1;
        this.errorStr = "";
        this.errorSeverity = Exception.SEVERITY_FATAL;
        this.resultXML = null;
    },
setArg : function(name, value)
    {
        name = makeSafe.urlqueryname(name);
        name = name.replace(/_/g, "-");
        value = makeSafe.string(value + "");
        value = this._encode(value);
        this.request.setRequestHeader("TT-Param-" + name, value);
    },
    send : function()
    {
        if (this._tryCache())
            return;

        this._reserveCache();

        var body = null;
        if (this.xmluploadbody)
        {
            body = this.xmluploadbody.toXMLString();
            this.request.setRequestHeader("Content-Type", "text/xml");
            this.request.setRequestHeader("Content-Length", body.length);
        }
if (body){}
this._starttime = new Date();

        this.request.send(body);
        this._isDone = false;
    },
    cancel : function(ex)
    {
        if (this._isDone)
            return;

        if (this._reloginRequest)
        {
            this._reloginRequest.cancel(ex);
            return;
        }

        this.request.abort();

        this.cancelException = ex ? ex : new UserCancelledException();
        this.success = false;

        this._putCache();
        this._doneCallbacks();
    },
    setFinishedCallback : function (aFinishedCallback)
    {
        this._mFinishedCallback = aFinishedCallback;
    },
    _done : function(success)
    {
if (success && this.request.status >= 200 && this.request.status < 300)
        {
            try
            {
                this.errorCode = makeSafe.integer(this.request.
                                              getResponseHeader("TT-Result-Code"));
                this.errorStr = makeSafe.label(this._decode(this.request.
                                              getResponseHeader("TT-Result-Text")));
                var errorSeverity = this.request.getResponseHeader("TT-Result-Severity");
                if (errorSeverity)
                    this.errorSeverity = makeSafe.integer(errorSeverity) + Exception.SEVERITY_BASE;
                this.success = this.errorCode == 0;

                if (this.errorCode == 403 && gApplication.login.isLoggedIn && this._reloginAttempts == 0)
                {
var login = gApplication.login;
                    var me = this;
                    var curUsername = login.currentUsername;
                    var curPassword = login.currentPassword;

                    this._reloginAttempts++;
                    this._reloginRequest = login.login(curUsername, curPassword, function()
                    {
me._reloginRequest = null;
                        try {
                            me.setup();
                            me.send();
                        } catch (e if e instanceof FailureAlreadyReportedToCallbackException) {}
                    }, function(e) {
                        this._isDone = true;
                        login.currentUsername = null;
                        me.errorCallback(e);

                        if (me._mFinishedCallback)
                            me._mFinishedCallback(me);
                    });
                    return;
                }
            }
            catch (e)
            {
                this.success = false;
                this.errorCode = -3;
                this.errorStr = this._rpc.stringBundle.GetStringFromName("severeServerError");
                unimportantError(e);
            }
        }
        else
        {
            try {
                this.success = false;
                this.errorCode = -this.request.status;
                this.errorStr = this.request.statusText;
            } catch(e) {
                this.cancelException = NetUtil.exceptionForStatusCode(this.request.channel.status, this.url.spec);
                unimportantError(e);
            }
        }
        if (this.success)
        {
            try
            {
                var mimetype = this.request.getResponseHeader("Content-Type");
                mimetype = mimetype.split(";")[0];
                if (mimetype == "text/xml" ||
                    mimetype == "application/xml" ||
                    mimetype == "text/rdf")
                {
                    try
                    {
                        var text = this.request.responseText;
                        text = text.replace(/<\?xml[^>]*\?>/, "");
                        this.resultXML = new XML(text);{}
}
                    catch (e)
                    {
throw e;
                    }
                }
                else
                    this.resultXML = null;
            }
            catch (e)
            {
                this.success = false;
                this.errorCode = -4;
                this.errorStr = this._rpc.stringBundle.GetStringFromName("badXMLResponse");
            }
        }
{}
this._putCache();

        this._doneCallbacks();

        rpc._setIsOnline(success);
    },

    _doneCallbacks : function()
    {

        this._isDone = true;
        if (this.success)
        {
            try {
                this.successCallback(this.resultXML);
            } catch (e) {
                this.errorCallback(e);
            }
        }
        else
        {
            try {
                var e;
                if (this.cancelException)
                    e = this.cancelException;
                else
                {
                    var args = {};
                    for (let key in this.args)
                        if (key != "password")
                            args[key] = this.args[key];

                    e = new ServerError(this.errorStr, this.errorCode, this.url ? this.url.spec : null,
                                        this.errorSeverity,
                                        {
                                            func: this.server + "/" + this.func,
                                            args: args,
                                            method: this.post ? "POST" : "GET",
                                            useCache: this.useCache ? "true" : "false",
                                            dryRun: this.dryRun ? "true" : "false",
                                            xmluploadbody: this.xmluploadbody
                                        });
                }
                try {
                    e.debugInfoFromServer = this.request.getResponseHeader("TT-Result-DebugText");
                } catch (ex) {}

                this.errorCallback(e);
            } catch (e) {
                error(e);
            }
        }

        if (this._mFinishedCallback)
            this._mFinishedCallback(this);
    },
_setGenericArguments : function()
    {
        if (this.dryrun)
            this.request.setRequestHeader("TT-Generic-DryRun", "true");
        this.request.setRequestHeader("TT-Generic-ClientVersion", gApplication.settings.version);
        this.request.setRequestHeader("TT-Generic-Language", gApplication.settings.networkLocale);
        this.request.setRequestHeader("TT-Generic-Country", gApplication.settings.country);
    },
    _encode : function(str)
    {
        str = encodeURIComponent(str);
        str = str.replace(/\%20/g, " ");
        return str;
    },
    _decode : function(str)
    {
        return decodeURIComponent(str);
    },
get _cacheKey()
    {
        var args = [];
        for (var key in this.args)
            args.push(encodeURIComponent(key) + "=" + encodeURIComponent(this.args[key]));
        args.sort();

        return memoize(this, "_cacheKey", encodeURIComponent(this.server) + "\n" +
            encodeURIComponent(this.func) + "\n" +
            (this.post ? "true" : "false") + "\n" +
            (this.dryrun ? "true" : "false") + "\n" +
            args.join("&") + "\n" +
            (typeof this.xmluploadbody == "xml" ?  encodeURIComponent(this.xmluploadbody.toXMLString()) : "") + "\n");
    },
_setResultFromCache : function(success, cancelException, errorCode, errorStr, resultXML)
    {
        this.success = success;
        this.cancelException = cancelException;
        this.errorCode = errorCode;
        this.errorStr = errorStr;
        this.resultXML = resultXML;{}
runAsync(makeCallback(this, this._doneCallbacks));
    },
_reserveCache : function()
    {
        if (!this.useCache)
            return;
        this._rpc._cache[this._cacheKey] =
        {
            done: false,
            additionalRequests: []
        };
    },
_putCache : function()
    {
        if (!this.useCache)
            return;

        var cache = this._rpc._cache;
        var match = cache[this._cacheKey];
        if (match && !match.done)
        {
            delete cache[this._cacheKey];
            for each (let request in match.additionalRequests)
                request._setResultFromCache(this.success, this.cancelException, this.errorCode, this.errorStr, this.resultXML);
        }
        if (this.cancelException || !(this.errorCode >= 0 && this.errorCode < 100))
            return;

        try {
            var expire = null;
            var header = this.request.getResponseHeader("TT-Expires");
            if (header)
            {
                expire = new Date(header);
            }
            else
            {
                expire = new Date();
                expire.setHours(expire.getHours() + 1);
            }
            if (expire < new Date())
                return;

            debug("Result of " + this.server + "/" + this.func + " call cached, expires in " + ((expire - new Date()) / 1000) + " seconds");
            cache[this._cacheKey] =
            {
                done: true,
                success: this.success,
                cancelException: this.cancelException,
                errorCode: this.errorCode,
                errorStr: this.errorStr,
                resultXML: this.resultXML,
                expire: expire
            };
        } catch (e) { unimportantError(e); }
    },
_tryCache : function()
    {
        if (!this.useCache)
            return false;

        var cache = this._rpc._cache;
        if (this._cacheKey in cache)
        {
            var match = cache[this._cacheKey];
            if (match.done)
            {
                if (match.expire > Date.now())
                {
                    this._setResultFromCache(match.success, match.cancelException, match.errorCode, match.errorStr, match.resultXML);
                    return true;
                }
                else
                    delete cache[this._cacheKey];
            }
            else
            {
                match.additionalRequests.push(this);
                return true;
            }
        }
        return false;
    },
    __end: null
}
function NetworkCall()
{
    throw new NotReached("purely abstract base object");
}
NetworkCall.prototype =
{
    cancel : function(ex)
    {
        throw new NotReached("override in inheriting class");
    },
setFinishedCallback : function(aFinishedCallback)
    {
        throw new NotReached("override in inheriting class");
    }
}

extend(rpcCall, NetworkCall);
function CombinedCall()
{
    this._calls = [];
}
CombinedCall.prototype =
{
    _calls : null,
    _mFinishedCallback : null,
_cancelException : null,
_hadCalls : false,
    cancel : function(ex)
    {
        this._cancelException = ex ? ex : new UserCancelledException();
        var calls = this._calls.slice();
        for each(var call in calls)
            call.cancel(this._cancelException);
    },
    setFinishedCallback : function(aFinishedCallback)
    {
        this._mFinishedCallback = aFinishedCallback;
        if (this._calls.length == 0 && this._hadCalls)
        {
            if (this._mFinishedCallback)
                this._mFinishedCallback(this);
        }
    },
addCall : function(aCall)
    {
        assert(aCall instanceof NetworkCall, "aCall must be an instance of NetworkCall");

        this._hadCalls = true;
        this._calls.push(aCall);
        aCall.setFinishedCallback(makeCallback(this, this._oneCallFinished));
        if (this._cancelException)
            aCall.cancel(this._cancelException);
    },

    _oneCallFinished : function(aCall)
    {
        assert(aCall, "aCall must not be null");
        var found = ArrayRemoveFirst(this._calls, aCall);
        assert(found, "finished call was not found in the pending-call queue");

        if (this._calls.length == 0)
        {
            try {
                if (this._mFinishedCallback)
                    this._mFinishedCallback(this);
            } catch (e) { error(e); }
        }
    },

    __end : null
}
extend(CombinedCall, NetworkCall);
function NoOpCall()
{
}
NoOpCall.prototype =
{
    cancel : function(ex)
    {
    },
    setFinishedCallback : function(cb)
    {
        cb(this);
    },
    __end : null
}
extend(NoOpCall, NetworkCall);
function FailureAlreadyReportedToCallbackException()
{
    Exception.call(this);
}
extend(FailureAlreadyReportedToCallbackException, Exception);
