/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/


function DeviceManager()
{
    this._activeDevice = null;
    this._activeInstallLocation = null;
    this._observers = new Array();
    this._deviceAccessObservers = new Array();
    this._deviceAccessPoller = Cc["@mozilla.org/timer;1"].createInstance(CI.nsITimer);
}
DeviceManager.prototype =
{

    _activeDevice : null,
    _activeInstallLocation : null,
    _deviceSelectionPaused : false,
    _observers : null,
    _activeDeviceType : null,
_deviceAccessObservers : null,
_deviceAccessPoller : null,
start : function()
    {
        this.repopulated();
        gApplication.homeServices.devices().addObserver(this);
    },
get activeDevice()
    {
        return this._activeDevice;
    },
get activeInstallLocation()
    {
        return this._activeInstallLocation;
    },
get lastDevice()
    {
        if (!prefService().prefHasUserValue("tomtom.lastConnectedDevice.id"))
        {
return null;
        }
        return new LastConnectedDevice();
    },
get hasMultipleInstallLocations()
    {
        return (this.getAllInstallLocations().length > 1);
    },
get activeDeviceParametersString()
    {
        return this.getDeviceParametersStringForDevice(this.activeDevice || this.lastDevice);
    },
get activeDeviceCapabilitiesString()
    {
        var activeDevice = this.activeDevice;
        if (!activeDevice)
        {
            var lastDevice = this.lastDevice;
            return lastDevice ? lastDevice.model.capabilitiesString : "";
        }

        return IteratorToArray(activeDevice.capabilities, CI.nsISupportsCString).join(" ");
    },
get activeDeviceMapShareJoined()
    {
        var activeDevice = this.activeDevice;
        if (!activeDevice)
        {
            var lastDevice = this.lastDevice;
            return lastDevice ? lastDevice.mapShareJoined : null;
        }

        return activeDevice.mapShare == null ? false : activeDevice.mapShare.mapPatchDownloadsEnabled;
    },
get isDeviceSelectionPaused()
    {
        return this._deviceSelectionPaused;
    },
    deviceSelection : function(device)
    {
        var me = this;
        var selectDevice = function (result)
        {
            if (result)
            {
                me._setTo(result.device, result.installLocation);
            }
            else if (device && me._activeDevice == null)
            {
                me.repopulated();
            }
        }
        openDialogAsWindow("chrome://tthome/content/ui/dialogs/device/deviceSelection.xul",
            { installLocOfDevice: device, resultCallback: selectDevice }
        )
    },

    addObserver : function(observer)
    {
        this._observers.push(observer);
    },
    removeObserver : function(observer)
    {
        ArrayRemoveAllMatching(this._observers, observer);
    },

    addDeviceAccessObserver : function(observer)
    {
        this._deviceAccessObservers.push(observer);
        if (this.activeDevice)
            this._startDeviceAccessPolling();
    },
    removeDeviceAccessObserver : function(observer)
    {
        ArrayRemoveAllMatching(this._deviceAccessObservers, observer);
        if (this._deviceAccessObservers.length == 0)
            this._stopDeviceAccessPolling();
    },

    added : function(dev)
    {
        try {
            assert(dev instanceof CI.ttIDevice || dev instanceof CI.ttIInstallLocation, "Device or install location expected")
            this._changed(false, false);{}
if (dev == this._activeDevice || dev == this._activeInstallLocation)
            {
                return;
            }

            if (this._deviceSelectionPaused)
            {
return;
            }

            if (this.hasMultipleInstallLocations)
            {
                this.deviceSelection();
            }
            else
            {
                this._setTo(dev, this.getInstallLocationsForDevice(dev)[0]);
            }
        } catch (e) { error(e); }
    },
    removed : function(dev)
    {
        try {
            assert(dev instanceof CI.ttIDevice || dev instanceof CI.ttIInstallLocation, "Device or install location expected")
            this._changed(false, false);{}
var installLocs = this.getInstallLocationsForDevice();
            if (dev == this._activeInstallLocation && this._deviceSelectionPaused)
            {
                this._setToNull();
            }
            else if (dev == this._activeInstallLocation && installLocs.length == 1)
            {
                this._setTo(this._activeDevice, installLocs[0]);
            }
            else if (dev == this._activeDevice || dev == this._activeInstallLocation)
            {
                this.repopulated();
            }
            else
            {
                return;
            }
        } catch (e) { error(e); }
    },
    repopulated : function()
    {
        try {
            this._changed(false, false);

            var devices = this.getAllDevices();
            var installLocs = this.getInstallLocationsForDevice();{}
if (ArrayContains(devices, this._activeDevice) && ArrayContains(installLocs, this._activeInstallLocation))
            {
                return;
            }

            this._setToNull();
            var installLocs = this.getAllInstallLocations();
            if (installLocs.length > 1)
            {
                this.deviceSelection();
            }
            else if (installLocs.length == 1)
            {
                let [device, installLoc] = installLocs[0];
                this._setTo(device, installLoc);
            }
} catch (e) { error(e); }
    },
    QueryInterface : function(iid)
    {
        if (iid.equals(CI.ttICollectionObserver) ||
            iid.equals(CI.nsISupportsWeakReference) ||
            iid.equals(CI.nsISupports))
            return this;
        throw Components.results.NS_ERROR_NO_INTERFACE;
    },
    _setToNull : function()
    {
        if (this._activeDevice == null)
        {
            return;
        }
this._activeDevice = null;
        this._activeInstallLocation = null;
        this._changed(true, true);
    },
_setTo : function(dev, loc)
    {
        assert(dev instanceof CI.ttIDevice, "need ttIDevice");
        assert(loc instanceof CI.ttIInstallLocation, "need ttIInstallLocation");

        if (this._deviceSelectionPaused)
        {
return;
        }

        if (this._activeDevice == dev && this._activeInstallLocation == loc)
        {
            return;
        }
if (this._activeDevice) {
            try {
                this._activeDevice.installLocations().removeObserver(this);
            }
            catch (e) {
            }
        }
        dev.installLocations().addObserver(this);

        var deviceChanged = (this._activeDevice != dev);
        this._activeDevice = dev;
        this._activeInstallLocation = loc;

        function createNSISupportsString(data)
        {
            let stringObj = Cc["@mozilla.org/supports-string;1"].createInstance(CI.nsISupportsString);
            stringObj.data = data;
            return stringObj;
        };

        try {
            var pref = prefService();
            pref.setCharPref("tomtom.lastConnectedDevice.id", dev.id);
            var branch = "tomtom.device." + dev.id + ".";
            pref.setCharPref(branch + "deviceCode", dev.deviceCode);
            pref.setCharPref(branch + "serial", dev.serial);
            pref.setCharPref(branch + "modelID", dev.model.id);
            pref.setCharPref(branch + "modelName", dev.model.modelName);
            pref.setCharPref(branch + "modelFamilyName", dev.model.familyName);
            pref.setIntPref(branch + "modelScreenWidth", dev.model.screenWidth);
            pref.setIntPref(branch + "modelScreenHeight", dev.model.screenHeight);
            pref.setCharPref(branch + "navcoreVersion", dev.versionNavCore);
            pref.setCharPref(branch + "navcoreBuildNumber", dev.buildNumberNavCore);
            pref.setComplexValue(branch + "capabilities", CI.nsISupportsString,
                    createNSISupportsString(this.activeDeviceCapabilitiesString));
            var parameters = this.getDeviceParametersStringForDevice(this.activeDevice, true);
            pref.setComplexValue(branch + "parameters", CI.nsISupportsString,
                    createNSISupportsString(parameters));
            pref.setBoolPref(branch + "mapShareJoined", this.activeDeviceMapShareJoined);
        } catch (e) { unimportantError(e); }
        if (dev.versionNavCore == "0.000" && dev.model.familyName != "PocketPC")
        {
            this.showApplicationUpdateDialog();
        }

        this._changed(deviceChanged, true);
    },
    _notifyChange : function(deviceChanged, locationChanged)
    {
        var observers = this._observers.slice();
        for (var i = 0; i < observers.length; i++)
        {
            try {
                if (this._observers.indexOf(observers[i]) >= 0)
                    observers[i].changed(deviceChanged, locationChanged);
            } catch (e) { error(e); }
        }
    },
_notifyDeviceAccess : function()
    {
        var isLocked = this.activeDevice.isLocked;
        var observers = this._deviceAccessObservers.slice();
        for (var i = 0; i < observers.length; i++)
        {
            try {
                if (this._deviceAccessObservers.indexOf(observers[i]) >= 0)
                    observers[i].deviceAccessed(isLocked);
            } catch (e) { error(e); }
        }
    },
_changed : function(deviceChanged, locationChanged)
    {
        if (this.activeDevice)
        {
            this._startDeviceAccessPolling();
        }
        else
        {
            this._stopDeviceAccessPolling();
        }
        this._notifyChange(deviceChanged, locationChanged);
    },
showApplicationUpdateDialog : function()
    {
        var stringBundle = this._stringBundle;
        return ttMessageService().confirm(stringBundle.GetStringFromName("update.title"),
                                          stringBundle.GetStringFromName("update.message"),
                                          stringBundle.GetStringFromName("update.cancel"),
                                          stringBundle.GetStringFromName("update.confirm"),
                                          function()
                                          {
                                              var mainwin = gApplication.getSomeMainWindow();
                                              if (!mainwin || mainwin.currentTask())
                                                  return;
                                              mainwin.loadTaskByName("update");
                                              mainwin.bringToFront();
                                          },
                                          null, MessageService.SEVERITY_INFORMATIVE);
    },
getAllDevices : function()
    {
        return IteratorToArray(gApplication.homeServices.devices().iterator, CI.ttIDevice);
    },
_defaultParameters : {
        ApplicationVersionVersionNumber: true,
        ApplicationVersion: true,
        VehicleBrand: true,
        VehicleModel: true,
        VehicleIdentification: true
    },
getDeviceParametersStringForDevice : function(dev, unfiltered)
    {
        if (!dev)
            return "";
        var deviceParametersEnum = dev.parameters;
        var deviceParameters = dev.parameters;
        if (deviceParametersEnum instanceof CI.nsISimpleEnumerator)
        {
            deviceParameters = { __proto__: null };
            while (deviceParametersEnum.hasMoreElements())
                let(parameter = deviceParametersEnum.getNext().QueryInterface(CI.ttIParameter))
                    deviceParameters[parameter.name] = parameter.value;
        }

        var filteredParameters = deviceParameters;
        if (!unfiltered)
        {
            filteredParameters = { __proto__: null };
            for (let prop in deviceParameters)
                if (this._defaultParameters.hasOwnProperty(prop))
                    filteredParameters[prop] = deviceParameters[prop];
        }

        return JSON.stringify(filteredParameters);
    },
getInstallLocationsForDevice : function(dev)
    {
        if (typeof(dev) == "undefined")
            dev = this.activeDevice;

        if (!dev)
            return new Array();
        return IteratorToArray(dev.installLocations().iterator,
                CI.ttIInstallLocation);
    },
getAllInstallLocations : function()
    {
        var devices = this.getAllDevices();
        var installLocs = [];
        for each (let device in devices)
        {
            for each (let installLoc in this.getInstallLocationsForDevice(device))
                installLocs.push([device, installLoc]);
        }
        return installLocs;
    },
getDeviceType : function()
    {
        var device = this.activeDevice;
        if(device && device.isAutomotive)
        {
            this._activeDeviceType = "automotive";
        }
        else
        {
            this._activeDeviceType = "pnd";
        }
        return this._activeDeviceType;
    },
convertTimestamp :function(timestamp)
    {
        var epoch = new Date(1970, 1, 1, 0, 0, 0, 0).getTime();
        var localDate = new Date(timestamp - epoch);
        var utc = localDate.getTime()+localDate.getTimezoneOffset()*60000*2;
        var utcDate = new Date(utc);
        var dateformat = utcDate.toLocaleFormat("%B %d, %Y - %H:%M");
        return dateformat;
    },
pauseDeviceSelection : function()
    {
        this._deviceSelectionPaused = true;
    },
resumeDeviceSelection : function(device, installLocation)
    {
        if (!this._deviceSelectionPaused)
        {
            return;
        }

        this._deviceSelectionPaused = false;

        if (device)
        {
            this.selectDevice(device, installLocation);
        }
        else if (!this._activeDevice)
        {
            this.repopulated();
        }
    },
selectDevice : function(device, installLocation)
    {
        assert(device instanceof CI.ttIDevice, "need ttIDevice");

        var installLocations = this.getInstallLocationsForDevice(device);

        if (installLocation)
        {
            this._setTo(device, installLocation)
        }
        else if (installLocations.length > 1)
        {
            this.deviceSelection(device);
        }
        else if (installLocations.length == 1)
        {
            this._setTo(device, installLocations[0]);
        }
    },
_startDeviceAccessPolling : function()
    {
        if (this._deviceAccessObservers.length > 0)
        {
            this._deviceAccessPoller.initWithCallback(
                makeCallback(this, function()
                {
                    this._notifyDeviceAccess();
                }),
                500,
                CI.nsITimer.TYPE_REPEATING_SLACK);
        }
    },
_stopDeviceAccessPolling : function()
    {
        this._deviceAccessPoller.cancel();
    },

    get _stringBundle()
    {
        return memoize(this, "_stringBundle", getStringBundle("chrome://tthome/locale/logic/deviceManager.properties"));
    },

    __end : null
}
function LastConnectedDevice()
{
    this._id = prefService().getCharPref("tomtom.lastConnectedDevice.id");
    this._branch = "tomtom.device." + this._id + ".";
}
LastConnectedDevice.prototype =
{
    _id : null,
    _branch : null,

    get id()
    {
        return this._id;
    },
    get serial()
    {
        return prefService().getCharPref(this._branch + "serial");
    },
    get deviceCode()
    {
        return prefService().getCharPref(this._branch + "deviceCode");
    },
    get model()
    {
        var branch = this._branch;
        return {
            get id()
            {
                return prefService().getCharPref(branch + "modelID");
            },
            get modelName()
            {
                return prefService().getCharPref(branch + "modelName");
            },
            get familyName()
            {
                return prefService().getCharPref(branch + "modelFamilyName");
            },
            get screenWidth()
            {
                return prefService().getIntPref(branch + "modelScreenWidth");
            },
            get screenHeight()
            {
                return prefService().getIntPref(branch + "modelScreenHeight");
            },
            get capabilitiesString()
            {
                try {
                    return prefService().getComplexValue(branch + "capabilities", CI.nsISupportsString).data;
                } catch (e) {
                    unimportantError(e);
                    return "";
                }
            }
        };
    },
    get versionNavCore()
    {
        return prefService().getCharPref(this._branch + "navcoreVersion");
    },
    get buildNumberNavCore()
    {
        try {
            return prefService().getCharPref(this._branch + "navcoreBuildNumber");
        } catch (e) {
            unimportantError(e);
            return "";
        }
    },
    get mapShareJoined()
    {
        try {
            return prefService().getBoolPref(this._branch + "mapShareJoined");
        } catch (e) {
            unimportantError(e);
            return null;
        }
    },
    get parameters()
    {
        try {
            return JSON.parse(prefService().getComplexValue(this._branch + "parameters", CI.nsISupportsString).data);
        } catch (e) {
            unimportantError(e);
            return {};
        }
    }
}
