/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/


EXPORTED_SYMBOLS.push(
    "BaseListener",
    "ProgressListener",
    "CopyListener",
    "DeleteListener",
    "DownloadListener",
    "ScanListener",
    "Abortable",
    "NoOpAbortable"
);

var gDLLUtilsStringBundle = getStringBundle("chrome://tthome/locale/logic/dllUtils.properties");
function _getLeafName(path)
{
    path = path.replace(/[\/\\]+$/);
    path = path.replace(/.*[\/\\]/, "");

    return path;
}
function BaseListener(progressCallback, successCallback, errorCallback)
{
    this._progressCallback = progressCallback;
    this._errorCallback = errorCallback;
    this._successCallback = successCallback;
    this.startedText = gDLLUtilsStringBundle.GetStringFromName("actionStarted");
    this.progressText = gDLLUtilsStringBundle.GetStringFromName("processing");
    this.successText = gDLLUtilsStringBundle.GetStringFromName("done");
}
BaseListener.prototype =
{
    startedText : "",
    progressText : "",
    successText : "",
    currentPercentage : -1,
    started : function()
    {
        this._progressCallback(0, this.startedText);
    },
    progress : function(percent)
    {
        this.currentPercentage = percent;
        this._progressCallback(percent, this.progressText);
    },
    done : function(errorCode)
    {
        if (errorCode)
        {
return;
        }
        try {
this._progressCallback(100, this.successText);
            this._success();
        } catch (e) {
            this._errorFatal(e);
        }
    },
    error : function(error, retryOut)
    {
if (error.errorCode == CI.ttIErrorCode.kInterruptedByUser)
            error = new UserCancelledException();
        this._progressCallback(-1, gDLLUtilsStringBundle.formatStringFromName("error", [errorToText(error)], 1));

        this._errorFatal(error);
    },
    _success : function()
    {
        this._successCallback(this.successText);
    },
    _errorFatal : function(e)
    {
        this._errorCallback(e);
    },
    __end : null
}
function ProgressListener()
{
    BaseListener.apply(this, arguments);
}
ProgressListener.prototype = {
    QueryInterface : function(iid)
    {
        if (iid.equals(CI.ttIProgressListener) ||
            iid.equals(CI.nsISupports))
            return this;
        throw Components.results.NS_NOINTERFACE;
    }
}
extend(ProgressListener, BaseListener);
function CopyListener(progressCallback, successCallback, errorCallback)
{
    BaseListener.apply(this, arguments);
    this.startedText = gDLLUtilsStringBundle.GetStringFromName("copyStarted");
    this.progressText = gDLLUtilsStringBundle.GetStringFromName("copying");
    this.successText = gDLLUtilsStringBundle.GetStringFromName("copyDone");
}
CopyListener.prototype =
{
    startedFile : function(path)
    {
        this._progressCallback(this.currentPercentage, gDLLUtilsStringBundle.formatStringFromName("copyingFile", [_getLeafName(path)], 1));
    },
    QueryInterface : function(iid)
    {
        if (iid.equals(CI.ttICopyListener) ||
            iid.equals(CI.ttIProgressListener) ||
            iid.equals(CI.nsISupports))
            return this;
       throw Components.results.NS_NOINTERFACE;
    }
}
extend(CopyListener, BaseListener);
function DeleteListener(progressCallback, successCallback, errorCallback)
{
    BaseListener.apply(this, arguments);
    this.startedText = gDLLUtilsStringBundle.GetStringFromName("deleteStarted");
    this.progressText = gDLLUtilsStringBundle.GetStringFromName("deleting");
    this.successText = gDLLUtilsStringBundle.GetStringFromName("deleteDone");
}
DeleteListener.prototype =
{
    startedFile : function(path)
    {
        this._progressCallback(this.currentPercentage, gDLLUtilsStringBundle.formatStringFromName("deletingFile", [_getLeafName(path)], 1));
    },
    QueryInterface : function(iid)
    {
        if (iid.equals(CI.ttICopyListener) ||
            iid.equals(CI.ttIProgressListener) ||
            iid.equals(CI.nsISupports))
            return this;
       throw Components.results.NS_NOINTERFACE;
    }
}
extend(DeleteListener, BaseListener);
function DownloadListener(progressCallback, successCallback, errorCallback)
{
    BaseListener.apply(this, arguments);
    this.startedText = gDLLUtilsStringBundle.GetStringFromName("downloadStarted");
    this.progressText = gDLLUtilsStringBundle.GetStringFromName("downloading");
    this.successText = gDLLUtilsStringBundle.GetStringFromName("downloadDone");
}
DownloadListener.prototype =
{
    QueryInterface : function(iid)
    {
        if (iid.equals(CI.ttIDownloadListener) ||
            iid.equals(CI.ttIProgressListener) ||
            iid.equals(CI.nsISupports))
            return this;
       throw Components.results.NS_NOINTERFACE;
    }
}
extend(DownloadListener, BaseListener);
function ScanListener(progressCallback, successCallback, errorCallback)
{
    BaseListener.apply(this, arguments);
    this.startedText = gDLLUtilsStringBundle.GetStringFromName("scanStarted");
    this.progressText = gDLLUtilsStringBundle.GetStringFromName("scanning");
    this.successText = gDLLUtilsStringBundle.GetStringFromName("scanDone");
}
ScanListener.prototype =
{
    foundFile : function(path)
    {
    },
    foundFolder : function(path)
    {
        this._progressCallback(this.currentPercentage,
            gDLLUtilsStringBundle.formatStringFromName("scanFolder",
                [_getLeafName(path)], 1));
    },
    QueryInterface : function(iid)
    {
        if (iid.equals(CI.ttIScanListener) ||
            iid.equals(CI.ttIProgressListener) ||
            iid.equals(CI.nsISupports))
            return this;
       throw Components.results.NS_NOINTERFACE;
    }
}
extend(ScanListener, BaseListener);
function Abortable(abortable)
{
    assert(abortable instanceof CI.ttIAbortable);
    this._abortable = abortable;{}
}
Abortable.prototype =
{
    cancel : function()
    {
this._abortable.abort();
    },
    __end : null
};
function NoOpAbortable() {}

NoOpAbortable.prototype =
{
    cancel : function()
    {
},
    __end : null
};
extend(NoOpAbortable, Abortable);
