/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/


EXPORTED_SYMBOLS.push(
    "Downloader",
    "Uploader",
    "ProgressTracker"
);
function Downloader() {}

Downloader.prototype =
{
resumeDownloadEnabled: false,
temporaryDirectory: null,
deferFinalize: false,
download: function(
        aDownloadURL,
        aTargetFile,
        aProgressCallback,
        aSuccessCallback,
        aErrorCallback)
    {
        return new ProgressTracker(
            "GET",
            null,
            aDownloadURL,
            aTargetFile,
            this.resumeDownloadEnabled,
            this.temporaryDirectory,
            this.deferFinalize,
            aProgressCallback,
            aSuccessCallback,
            aErrorCallback);
    }

};
function Uploader() {}

Uploader.prototype =
{
upload: function(
        aSource,
        aTargetURL,
        aMimeType,
        aProgressCallback,
        aSuccessCallback,
        aErrorCallback)
    {
        return new ProgressTracker(
            "PUT",
            aMimeType,
            aTargetURL,
            aSource,
            false,
            null,
            false,
            aProgressCallback,
            aSuccessCallback,
            aErrorCallback);
    }
};
function ProgressTracker(
    aMethod,
    aMimeType,
    aURL,
    aFile,
    aResumeDownloadEnabled,
    aTemporaryDirectory,
    aDeferFinalize,
    aProgressCallback,
    aSuccessCallback,
    aErrorCallback)
{
    assert(aMethod == "GET" || aMethod == "PUT",
           "aMethod must be 'GET' or 'PUT'");
    assert(aFile instanceof CI.nsILocalFile,
           "aFile must be a nsILocalFile");
    assert(aURL instanceof CI.nsIURL,
           "aURL must be a nsIURL");
    assert(!aProgressCallback || typeof aProgressCallback == "function",
           "aProgressCallback must be null or a function");
    assert(typeof aSuccessCallback == "function",
           "aSuccessCallback must be a function");
    assert(typeof aErrorCallback == "function",
           "aErrorCallback must be a function");

    aURL.QueryInterface(CI.nsIURL);
    aFile.QueryInterface(CI.nsILocalFile);

    if (aMethod == "GET")
    {

        assert(!aMimeType,
               "aMimeType must be null for downloads.");
        assert(!aTemporaryDirectory || aTemporaryDirectory instanceof CI.nsILocalFile,
               "aTemporaryDirectory must be null or an nsILocalFile");

        if (aTemporaryDirectory)
        {
            aTemporaryDirectory.QueryInterface(CI.nsILocalFile);
        }
        if (aFile.exists() && aFile.isDirectory())
        {
            aFile = aFile.clone();
            var fileName = makeSafe.filename(aURL.fileName);

            if (fileName == "")
            {
                fileName = "default";
            }

            aFile.append(fileName);{}
}
    }
    else
    {

        assert(aMimeType,
               "aMimeType must not be null or the empty string");
        assert(!aResumeDownloadEnabled,
               "aResumeDownloadEnabled must be false for uploads");
        assert(!aTemporaryDirectory,
               "aTemporaryDirectory must be null for uploads");
        assert(!aDeferFinalize,
               "aDeferFinalize must be false for uploads");
    }

    this._method = aMethod;
    this._url = aURL;
    this._resumeDownloadEnabled = aResumeDownloadEnabled;
    this._temporaryDirectory = aTemporaryDirectory;
    this._deferFinalize = aDeferFinalize;
    this._progressCallback = aProgressCallback;
    this._successCallback = aSuccessCallback;
    this._errorCallback = aErrorCallback;

    this._startTime = Date.now();

    this._makeChannel();

    if (this._method == "PUT")
    {
var stream =
            Cc["@mozilla.org/network/file-input-stream;1"]
            .createInstance(CI.nsIFileInputStream);
        stream.init(
            aFile,
            0x01,
            0664,
            CI.nsIFileInputStream.CLOSE_ON_EOF);

        this._channel.QueryInterface(CI.nsIHttpChannel);
        this._channel.requestMethod = this._method;
        this._channel.QueryInterface(CI.nsIUploadChannel);
        this._channel.setUploadStream(stream, aMimeType, -1);

        this._channel.asyncOpen(this, null);
    }
    else
    {
this._file = aFile;

        if (this._configureResumeDownload())
        {
if (this._md5File != null)
            {
                if (!this._md5FileChunkSizeInfoReceivedFromServer || !this._md5FileChunkHashReceivedFromServer)
                {
                    this._readMD5FileChunkDataObtainedFromServer();
                }
                
                if (this._md5FileChunkSizeInfoReceivedFromServer.length == 0 || this._md5FileChunkHashReceivedFromServer.length == 0)
                {
this._md5File = null;
                }
                
                if (this._md5File != null)
                {
                    this._md5Checker = Cc["@mozilla.org/security/hash;1"].createInstance(CI.nsICryptoHash);
                    this._md5Checker.init(CI.nsICryptoHash.MD5);
                    this._downloadByteCountSizeToInvokeGC = 0;
                    if (this._resumeOffset > 0)
                    {
var noOfFileChunksToIgnore = Math.floor( (this._resumeOffset) / (parseInt(this._md5FileChunkSizeInfoReceivedFromServer[this._fileChunkNoToBeHashed], 10) ) );{}
this._offset = noOfFileChunksToIgnore * parseInt(this._md5FileChunkSizeInfoReceivedFromServer[this._fileChunkNoToBeHashed], 10);{}
this._fileChunkNoToBeHashed = noOfFileChunksToIgnore + 1;{}
}
                    this._bytesRemainingInChunk = ( parseInt(this._md5FileChunkSizeInfoReceivedFromServer[this._fileChunkNoToBeHashed], 10) ) - (this._resumeOffset - this._offset);{}
}
            }
            this._channel.asyncOpen(this, null);
        }
        else
        {
runAsync(makeCallback(this, this._finishedSuccessfully));
        }
    }
}


ProgressTracker.prototype =
{
    _method                         : false,
    _url                            : null,
    _file                           : null,
    _temporaryFile                  : null,
    _metaFile                       : null,
    _finalFile                      : null,
    _resumeDownloadEnabled          : false,
    _temporaryDirectory             : null,
    _deferFinalize                  : false,
    _progressCallback               : null,
    _successCallback                : null,
    _errorCallback                  : null,
    _resumeOffset                   : 0,
    _entityID                       : "",
    _stream                         : null,
    _startTime                      : null,
    _channel                        : null,
    _finishedCallback               : null,
    _contentLength                  : -1,
    _lastProgressUpdate             : 0,
    _error                          : null,
_offset                         : 0,
_md5FileChunkHashReceivedFromServer     : null,
_md5FileChunkSizeInfoReceivedFromServer : null,
_fileChunkNoToBeHashed         : 1,
_md5File                        : null,
_md5Checker                     : null,
_bytesRemainingInChunk          : null,
_checksumFailed          : false,
_metaFileVersion          : "1",
QueryInterface: function(aIID)
    {
        if (aIID.equals(CI.nsISupports)           ||
            aIID.equals(CI.nsIInterfaceRequestor) ||
            aIID.equals(CI.nsIProgressEventSink)  ||
            aIID.equals(CI.nsIStreamListener)     ||
            aIID.equals(CI.nsIRequestObserver))
            return this;

        throw Components.results.NS_NOINTERFACE;
    },
getInterface: function(aIID)
    {
        try {
            return this.QueryInterface(aIID);
        } catch (exception) {
            throw Components.results.NS_NOINTERFACE;
        }
    },
onProgress: function(aRequest, aContext, aProgress, aProgressMax)
    {
var now = Date.now();
                
        if (Math.abs(now - this._lastProgressUpdate) < 500)
            return;

        try {
            if (this._progressCallback)
            {
                if (aProgressMax < 0)
                {
                    this._progressCallback(-1, null, 0);
                }
                else
                {
                    var percentageComplete =
                        ((aProgress + this._resumeOffset) /
                         (aProgressMax + this._resumeOffset)) *
                        100;
                    var actualPercentageComplete =
                        ((aProgress) /
                         (aProgressMax)) *
                        100;
                    var remainingTime =
                        ProgressTracker.calculateRemainingTime(
                            this._startTime,
                            actualPercentageComplete);
                            
                    this._progressCallback(
                          percentageComplete,
                          remainingTime,
                          aProgressMax);
                }
            }
        } catch (exception) {
            unimportantError(exception);
        }

        this._lastProgressUpdate = now;
    },
onStatus: function(aRequest, aContext, aStatus, aStatusArg)
    {
    },
onStartRequest: function(aRequest, aContext)
    {
        if (!this._temporaryFile)
        {
            return;
        }

        this._contentLength = -1;
        try {
            this._contentLength =
                makeSafe.integer(this._channel.getResponseHeader("Content-Length"));
        } catch (e) {
            unimportantError(e);
        }
if (this._resumeDownloadEnabled &&
            this._resumeOffset == 0)
        {
var acceptRanges = null;
            try {
                acceptRanges =
                    this._channel.getResponseHeader("Accept-Ranges");
            } catch (e) {}

            if (acceptRanges)
            {
}
            else
            {
}
            if (!/\bbytes\b/i.test(acceptRanges))
            {
this._resumeDownloadEnabled = false;
            }
            else
            {
this._entityID = "";
                try {
                    this._channel.QueryInterface(CI.nsIResumableChannel);
                    this._entityID = this._channel.entityID;
                } catch (e) {
                    unimportantError(e);
                }
            }

            this._writeMetaFile(false, false);
        }

        try {
            var ioFlags =
                0x02 |
                0x08 |
                (this._resumeOffset ? 0x10 : 0x20);
            var permission = 0664;
            var behaviorFlags = 0;
            var fos =
                Cc["@mozilla.org/network/file-output-stream;1"]
                .createInstance(CI.nsIFileOutputStream);
            fos.init(
                this._temporaryFile,
                ioFlags,
                permission,
                behaviorFlags);
            this._stream =
                Cc["@mozilla.org/network/buffered-output-stream;1"]
                .createInstance(CI.nsIBufferedOutputStream);
            this._stream.init(fos, 1024 * 16);
        } catch (e) {
            var exception =
                new ServerError(
                    "Failed to create output file",
                    Components.results.NS_ERROR_FAILURE,
                    this._url.spec,
                    undefined,
                    this._getRequestInfo());
            exception.inner = e;
            this.cancel(exception);
        }
    },
onStopRequest: function(aRequest, aContext, aStatus)
    {
        if (this._stream)
        {
            try {
                this._stream.close();
                this._stream = null;
            } catch (e) {
                unimportantError(e);
            }
        }

        var exception = null;

        if (this._error &&
            (this._error.code == aStatus))
        {
            exception = this._error;
            if (this._md5File != null)
            {
                if (this._temporaryFile)
                {
                    this._temporaryFile.fileSize = this._offset;
                }
            }
        }
        else
        {
            try
            {
                exception = NetUtil.isRequestSuccess(this._channel, aStatus, this._getRequestInfo());
            }
            catch (e)
            {
                exception = e;
            }
        }

        if(!exception && this._checksumFailed)
        {
if(this._error)
                exception = this._error;
        }
        
        if (exception)
        {
            var notResumable =
                exception.code == Components.results.NS_ERROR_NOT_RESUMABLE;
            var entityChanged =
                exception.code == Components.results.NS_ERROR_ENTITY_CHANGED;
            if (this._resumeDownloadEnabled && (notResumable || entityChanged))
            {
                this._restartDownloadWithoutResume(entityChanged);
                return;
            }

            try {
                this._errorCallback(exception);
            } catch (e) {
                error(e);
            }

            if (this._finishedCallback)
            {
                this._finishedCallback(this);
            }
        }
        else
        {
            this._finishedSuccessfully();
        }

    },
_finishedSuccessfully: function()
    {
        if (this._progressCallback)
        {
            this._progressCallback(
                100, null, Math.max(0, this._contentLength));
        }

        try {
            if (!this._file)
            {
                this._successCallback(null);
            }
            else
            {
                let checkFileExtension = '.zip.md5x';
                if ((this._url.spec.indexOf(checkFileExtension) != -1))
                {
                        var md5FileChunkSizeInfoReceivedFromServer = new Array();
            
                        if(this._temporaryDirectory != null)
                        { 
                            let lines = readFile(this._temporaryFile);
                            for (var i = 0; i < lines.length; i++)
                            {
                                let currentRowDetail = lines[i].split("\t");
                                let fileChunkSize = currentRowDetail[1];
                                md5FileChunkSizeInfoReceivedFromServer.push(fileChunkSize);
                            }
                            
                            let totalFileSize = parseInt(md5FileChunkSizeInfoReceivedFromServer[0], 10);
                            let totalChunkSize = 0;
                            
                            for (var i = 1; i < lines.length; i++)
                            {
                                totalChunkSize = totalChunkSize + parseInt(md5FileChunkSizeInfoReceivedFromServer[i], 10);
                            }
                            
                            if (totalFileSize != totalChunkSize)
                            {
                                this._stringBundle = getStringBundle("chrome://tthome/locale/logic/downloader.properties");
                                throw new Exception(this._stringBundle.GetStringFromName("checksumMismatch"));
                            }
                        }
                }
               
                if (this._md5File != null)
                {                  
                    if (this._bytesRemainingInChunk && this._bytesRemainingInChunk != 0)
                    {
if(this._temporaryFile)
                            this._temporaryFile.fileSize = this._offset;
                        this._stringBundle = getStringBundle("chrome://tthome/locale/logic/downloader.properties");
                        throw new Exception(this._stringBundle.GetStringFromName("checksumMismatch"));
                    }
                    else
                    {
                        var downloadedFile = this._temporaryFile.clone();
                        if (!this._md5FileChunkSizeInfoReceivedFromServer)
                        {
                            this._readMD5FileChunkDataObtainedFromServer();
                        }
                
                        if (this._md5FileChunkSizeInfoReceivedFromServer.length == 0 )
                        {
}
                        else 
                        {
                            if (downloadedFile.fileSize != parseInt(this._md5FileChunkSizeInfoReceivedFromServer[0], 10))
                            {
                                if (this._stream)
                                {
                                    try {
                                        this._stream.close();
                                        this._stream = null;
                                    } catch (e) {
                                    unimportantError(e);
                                    }    
                                }
                                this._md5File = null;{}
if (this._temporaryFile)
                                    this._temporaryFile.fileSize = 0;
                                this._checksumFailed = true;
                                this._stringBundle = getStringBundle("chrome://tthome/locale/logic/downloader.properties");
                                throw new Exception(this._stringBundle.GetStringFromName("checksumMismatch"));
                            }
                            else{}
}
                    }
   
                    this._md5Checker = null;
                }
                this._writeMetaFile(true, false);
                
                if (this._deferFinalize)
                {
                    this._successCallback(null);
                }
                else
                {
                    this._successCallback(
                        this.finalizeDownload());
                }
            }
        } catch (exception) {
            this._errorCallback(exception);
            exception.inner = "Download Url: Failed to download data from URL: <" + this._url.spec + ">.";
            error(exception);
        }

        if (this._finishedCallback)
        {
            this._finishedCallback(this);
        }
    },
onDataAvailable: function(aRequest, aContext, aInputStream, aOffset, aCount)
    {
        if (!this._stream)
            return; 

        try {
                if (this._md5Checker) 
                {
                    if (this._downloadByteCountSizeToInvokeGC && this._downloadByteCountSizeToInvokeGC >= 5*1024*1024)
                    {
this._downloadByteCountSizeToInvokeGC = 0;
                        Components.utils.forceGC();
                    }
                    var binaryInputStream = Cc["@mozilla.org/binaryinputstream;1"].createInstance(CI.nsIBinaryInputStream);
                    var binaryOutputStream = Cc["@mozilla.org/binaryoutputstream;1"].createInstance(CI.nsIBinaryOutputStream);
                        
                    binaryInputStream.setInputStream(aInputStream);
                    while (aCount > 0)
                    {
                        
                        var toRead = Math.min(this._bytesRemainingInChunk, aCount);

                         var bytes = [];
                         
                         bytes = binaryInputStream.readByteArray(toRead);
                         var result = true;
                         this._md5Checker.update(bytes, bytes.length);
                         this._bytesRemainingInChunk -= bytes.length;
                         if (this._bytesRemainingInChunk == 0)
                         {
                              var hash = this._md5Checker.finish(false);
                              var hexHash = "";
                              for (let i in hash)
                              {
                                  hexHash += ("0" + hash.charCodeAt(i).toString(16)).slice(-2);
                              }
                              result = this._checkFileChunkHash(this._fileChunkNoToBeHashed, hexHash);{}
                              if (result)
                              {
                                  this._md5Checker = null;
                                  this._offset = this._offset + parseInt(this._md5FileChunkSizeInfoReceivedFromServer[this._fileChunkNoToBeHashed], 10);
                                  this._fileChunkNoToBeHashed++;
                                  if (this._fileChunkNoToBeHashed < this._md5FileChunkSizeInfoReceivedFromServer.length)
                                  {
                                      this._md5Checker = Cc["@mozilla.org/security/hash;1"].createInstance(CI.nsICryptoHash);
                                      this._md5Checker.init(CI.nsICryptoHash.MD5);
                                      this._bytesRemainingInChunk = parseInt(this._md5FileChunkSizeInfoReceivedFromServer[this._fileChunkNoToBeHashed], 10);{}
                                  }
                                    
                              }
                              else
                              {
                                  this._md5File = null;
                                  this._md5Checker = null;
                                  this._bytesRemainingInChunk = null;
                                  return;
                              }
                          }
                          if (this._stream)
                          {
                            binaryOutputStream.setOutputStream(this._stream);
                            binaryOutputStream.writeByteArray(bytes,bytes.length);
                          }
                          this._downloadByteCountSizeToInvokeGC +=  aCount;
                          aCount -= bytes.length;
                    }
                }
                else
                {
                    while(aCount > 0)
                        aCount -= this._stream.writeFrom(aInputStream, aCount);
                }
            
        } catch (e) {
            var exception =
                new ServerError(
                    "Failed to write to output file",
                    Components.results.NS_ERROR_FAILURE,
                    this._url.spec,
                    undefined,
                    this._getRequestInfo());
            exception.inner = e;
            this.cancel(exception);
        }
    },
cancel: function(ex)
    {
        if (!ex)
            ex = new UserCancelledException();
        this._error = ex;

        this._channel.cancel(ex.code);
    },
setFinishedCallback: function (aFinishedCallback)
    {
        this._finishedCallback = aFinishedCallback;
    },
finalizeDownload: function()
    {
        this._finalFile = createUniqueFile(this._file);
        removeFile(this._finalFile, true);
        
        try
        {
            var completedFile = this._temporaryFile.clone();
            if (completedFile.fileSize > 0)
            {
                moveFile(this._temporaryFile, this._finalFile.parent, this._finalFile.leafName);
            }
            else
            {
                this._stringBundle = getStringBundle("chrome://tthome/locale/logic/downloader.properties");
                throw new Exception(this._stringBundle.GetStringFromName("checksumMismatch"));
            }
        }
        catch(e)
        {
            var downloadUrl = this._url.spec;
            e.inner = "Download Url: Failed to download data from URL: <" + downloadUrl + ">.\nError Message: 0kb File downloaded";
            e.severity = 500;
            e = ensureExceptionIsObject(e);
            error(e);
        }
        if (this._metaFile.exists())
        {
            removeFile(this._metaFile, true);
        }

        return this._finalFile;
    },
_getRequestInfo: function()
    {
        var info = {
            method: this._method,
            resumeOffset: this._resumeOffset,
            resumeDownloadEnabled: this._resumeDownloadEnabled,
            entityID: this._entityID
        };

        try
        {
            info.file = this._file.path;
        } catch(e) {}
        try
        {
            info.metaFile = this._metaFile.path;
        } catch(e) {}
        try
        {
            info.tempFile = this._temporaryFile.path;
        } catch(e) {}
        return info;
    },
_normalizeURL: function(url)
    {
        return url.spec.replace(/\|\|\|strCheck=[^&#]*/, "");
    },
_writeMetaFile: function(completed, allowResumingLater)
    {
        this._removeMetaFile();
        var tryResuming =
            this._resumeDownloadEnabled ||
            allowResumingLater;

        var entityID = this._entityID;
        var url = this._url.spec;
        completed = completed ? "true" : "false";
        tryResuming = tryResuming ? "true" : "false";

        var metaData =
            this._metaFileVersion + "\n" +
            entityID + "\n" +
            url + "\n" +
            completed + "\n" +
            tryResuming;

        try {
            writeStringToFile(this._metaFile, metaData);
        } catch (e) {
            unimportantError(e);
        }
},
_removeTemporaryFile: function()
    {
        if (this._temporaryFile.exists())
            this._temporaryFile.remove(true);
    },
_removeMetaFile: function()
    {
        if (this._metaFile.exists())
            this._metaFile.remove(true);
    },
_restartDownloadWithoutResume: function(allowResumingLater)
    {
this._resumeDownloadEnabled = false;
        this._resumeOffset = 0;
        this._writeMetaFile(false, allowResumingLater);
        this._removeTemporaryFile();

        if (this._md5File != null)
        {
            if (!this._md5FileChunkSizeInfoReceivedFromServer || !this._md5FileChunkHashReceivedFromServer)
            {
                this._readMD5FileChunkDataObtainedFromServer();
            }
                
            if (this._md5FileChunkSizeInfoReceivedFromServer.length == 0 || this._md5FileChunkHashReceivedFromServer.length == 0)
            {
this._md5File = null;
            }
                
            if (this._md5File != null)
            {
                this._md5Checker = Cc["@mozilla.org/security/hash;1"].createInstance(CI.nsICryptoHash);
                this._md5Checker.init(CI.nsICryptoHash.MD5);
                this._downloadByteCountSizeToInvokeGC = 0;
                    
                this._offset = 0;
                this._fileChunkNoToBeHashed = 1;
                    
                this._checksumFailed = false;
                this._bytesRemainingInChunk = ( parseInt(this._md5FileChunkSizeInfoReceivedFromServer[this._fileChunkNoToBeHashed], 10) );{}
}
        }
        
        this._makeChannel();
        this._channel.asyncOpen(this, null);
    },
_makeChannel: function()
    {
        this._channel =
            getIOService().newChannelFromURI(this._url);
        if ((this._channel instanceof CI.nsIHttpChannel) &&
            gApplication &&
            gApplication.settings)
        {
            this._channel.setRequestHeader(
                "TT-Generic-Language",
                gApplication.settings.networkLocale,
                false);
            this._channel.setRequestHeader(
                "TT-Generic-Country",
                gApplication.settings.country,
                false);
        }

        this._channel.notificationCallbacks = this;
    },
_configureResumeDownload: function()
    {
        this._resumeDownloadEnabled =
            this._resumeDownloadEnabled &&
            (this._channel instanceof CI.nsIHttpChannel);

        if (this._temporaryDirectory)
        {
            this._temporaryFile =
                this._temporaryDirectory.clone();
            this._metaFile =
                this._temporaryDirectory.clone();
        }
        else
        {
            this._temporaryFile =
                this._file.parent.clone();
            this._metaFile =
                this._file.parent.clone();
        }
        var normalURL = this._normalizeURL(this._url);
        var hash = this._computeHashMD5(normalURL);
        
        var actualName =  '.zip';
        var replaceName = '.zip.md5x';
        if (normalURL.indexOf(actualName) != -1)
        {
                var downloadUrlMD5 = normalURL.replace(actualName, replaceName);
        
                var md5FilePath = this._temporaryFile.clone();
                md5FilePath.append(this._computeHashMD5(downloadUrlMD5) + ".tmp");
        
                var file = Cc["@mozilla.org/file/local;1"]
              .createInstance(CI.nsILocalFile);

                file.initWithPath(md5FilePath.path);

                if (file.exists())
                {
                        this._md5File = file.clone();
                }
                
        }

        this._temporaryFile.append(hash + ".tmp");
        this._metaFile.append(hash + ".tmp.meta");

        function getResumeData()
        {
            if (!this._resumeDownloadEnabled ||
                !this._temporaryFile.exists() ||
                !this._temporaryFile.isFile() ||
                !this._temporaryFile.isWritable() ||
                !this._metaFile.exists() ||
                !this._metaFile.isFile() ||
                !this._metaFile.isWritable())
            {
return null;
            }

            if (this._md5File)
            {
                if(this._temporaryFile.fileSize > 0)
                {
                    if (!this._md5FileChunkSizeInfoReceivedFromServer)
                    {
                       this._readMD5FileChunkDataObtainedFromServer();
                    }
                
                    if (this._md5FileChunkSizeInfoReceivedFromServer.length == 0)
                    {
this._md5File = null;
                    }
                    else
                    {
var noOfFileChunksToIgnore = Math.floor( (this._temporaryFile.fileSize) / (parseInt(this._md5FileChunkSizeInfoReceivedFromServer[this._fileChunkNoToBeHashed], 10) ) );{}
this._offset = noOfFileChunksToIgnore * parseInt(this._md5FileChunkSizeInfoReceivedFromServer[this._fileChunkNoToBeHashed], 10);{}
this._temporaryFile.fileSize = this._offset;
                    }
                }
            }
            let resumeOffset = this._temporaryFile.fileSize;

            let lines = readFile(this._metaFile);
            let version = lines[0];
            if (version != this._metaFileVersion ||
                lines.length != 5)
            {
return null;
            }

            let entityID = lines[1];

            var metaURL;
            try {
                metaURL = getIOService().newURI(lines[2], null, null);
                metaURL.QueryInterface(CI.nsIURL);
            } catch (e) {
unimportantError(e);
                return null;
            }
            var normalMetaURL = this._normalizeURL(metaURL);
            if (normalMetaURL != normalURL)
            {
return null;
            }

            var finished;
            try {
                finished = makeSafe.bool(lines[3]);
            } catch (e) {
unimportantError(e);
                return null;
            }

            var tryResuming;
            try {
                tryResuming = makeSafe.bool(lines[4]);
            } catch (e) {
unimportantError(e);
                return null;
            }
            if (!tryResuming)
            {
return null;
            }

            try {
                this._channel.QueryInterface(CI.nsIResumableChannel);
                this._channel.resumeAt(
                    resumeOffset,
                    entityID);
            } catch (e) {
unimportantError(e);
                return null;
            }

            return [resumeOffset, entityID, finished];
        }

        let resumeData = getResumeData.apply(this);
        if (resumeData)
        {
            let finished;
            [this._resumeOffset, this._entityID, finished] = resumeData;

            if (finished)
            {
this._contentLength = this._resumeOffset;

                return false;
            }
}
        else
        {
            this._removeTemporaryFile();
        }
        return true;
    },
_computeHashMD5 : function (aString)
    {
        var stream = Cc['@mozilla.org/io/string-input-stream;1']
            .createInstance(CI.nsIStringInputStream);
        stream.setData(aString, aString.length);

        try {
            var cryptoEngine = Cc["@mozilla.org/security/hash;1"]
                .createInstance(CI.nsICryptoHash);
            cryptoEngine.init(CI.nsICryptoHash.MD5);
            cryptoEngine.updateFromStream(stream, stream.available());
            var hash = cryptoEngine.finish(false);
            var hexHash = "";
            for (let i in hash)
            {
                hexHash += ("0" + hash.charCodeAt(i).toString(16)).slice(-2);
            }
            return hexHash;
        } finally {
            stream.close();
        }
    },
_computeHashMD5ForFileChunk : function (aArray)
    {
        var cryptoEngine = Cc["@mozilla.org/security/hash;1"]
                            .createInstance(CI.nsICryptoHash);
        cryptoEngine.init(CI.nsICryptoHash.MD5);
        cryptoEngine.update(aArray, aArray.length);
        var hash = cryptoEngine.finish(false);
        var hexHash = "";
        for (let i in hash)
        {
            hexHash += ("0" + hash.charCodeAt(i).toString(16)).slice(-2);
        }
        return hexHash;
    },
_checkFileChunkHash : function (afileChunkNoToBeHashed, aMD5Hash)
    {
        if (!this._md5FileChunkHashReceivedFromServer)
        {
            this._readMD5FileChunkDataObtainedFromServer();
        }
        
        if (this._md5FileChunkHashReceivedFromServer.length != 0)
        {
            var md5HashFromServer = this._md5FileChunkHashReceivedFromServer[afileChunkNoToBeHashed];
            try
            {
                if (aMD5Hash == md5HashFromServer)
                {
                    return true;
                }
                else
                {
                    if (this._stream)
                    {
                        try {
                            this._stream.close();
                            this._stream = null;
                        } catch (e) {
                        unimportantError(e);
                        }    
                    }
                    this._md5File = null;
                    this._md5Checker = null;{}
if(this._temporaryFile)
                        this._temporaryFile.fileSize = this._offset;
                    
                    this._checksumFailed = true;
                    this._stringBundle = getStringBundle("chrome://tthome/locale/logic/downloader.properties");
                    throw new Exception(this._stringBundle.GetStringFromName("checksumMismatch"));
                    
                }
            }
            catch(e)
            {
                var downloadUrl = this._url.spec;
                e.code = e.severity;
                e.inner = "Download Url: Failed to download data from URL: <" + downloadUrl + ">.\nError Message: Checksum Mismatch\nFile Size: " + this._temporaryFile.fileSize;
                e = ensureExceptionIsObject(e);
                this.cancel(e);
                error(e);
            }
        }
        return false;
    },
_readMD5FileChunkDataObtainedFromServer : function ()
    {
        this._md5FileChunkHashReceivedFromServer = new Array();
        this._md5FileChunkSizeInfoReceivedFromServer = new Array();
        
        if(this._md5File != null)
        { 
            let lines = readFile(this._md5File);
            for (var i = 0; i < lines.length; i++)
            {
                let currentRowDetail = lines[i].split("\t");
                let serverHashCode = currentRowDetail[0];{}
let fileChunkSize = currentRowDetail[1];
                this._md5FileChunkHashReceivedFromServer.push(serverHashCode);
                this._md5FileChunkSizeInfoReceivedFromServer.push(fileChunkSize);
            }
        }
    }

};
extend(ProgressTracker, NetworkCall);


ProgressTracker._stringBundle = null;

ProgressTracker._getStringBundle = function()
{
    if (ProgressTracker._stringBundle)
        return ProgressTracker._stringBundle;
    ProgressTracker._stringBundle =
        getStringBundle("chrome://tthome/locale/logic/downloader.properties");

    return ProgressTracker._stringBundle;
};
ProgressTracker.calculateRemainingTime = function(
    aStartTime,
    aPercentageComplete)
{
    var elapsed = Date.now() - aStartTime;
    var minElapsed = 1000 * 5;
    var result = "...";

    if ((aPercentageComplete > 0) &&
        (elapsed >= minElapsed))
    {
        var remainingTimeString = "";
        var remainingTime =
            Math.round((((100 - aPercentageComplete) / aPercentageComplete) * elapsed) / 1000);

        var remainingHours = 0;
        var remainingMinutes = 0;

        if (remainingTime >= 3600)
        {
            remainingHours = Math.floor(remainingTime / 3600);
            remainingTime  = remainingTime % (remainingHours * 3600);
        }

        if (remainingTime >= 60)
        {
            remainingMinutes = Math.floor(remainingTime / 60);
            remainingTime = remainingTime % (remainingMinutes * 60);
        }

        remainingTimeString =
            (remainingHours > 0 ? remainingHours + ":" : "") +
            ProgressTracker._formatNumber(remainingMinutes, 2) + ":" +
            ProgressTracker._formatNumber(remainingTime, 2);

        result =
            ProgressTracker._getStringBundle()
            .formatStringFromName(
                "remainingTime",
                [remainingTimeString],
                1);

    }

    return result;
};
ProgressTracker._formatNumber = function (aNumber, aNoOfDigits)
{
    var formattedNumber = null;

    formattedNumber = aNumber.toString();

    if (aNumber.toString().length < aNoOfDigits)
    {
        for (var i = 0; i < (aNoOfDigits - aNumber.toString().length); i++)
        {
            formattedNumber = "0" + formattedNumber;
        }
    }

    return formattedNumber;
};
