/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/


EXPORTED_SYMBOLS.push("ttMap");
function ttMap()
{
    ttNavItem.apply(this);
}
ttMap.prototype =
{
_buildNumber: null,
_regions: null,
_baseMap : null,
_coveredRegions : null,
_mapzonePreview : null,
_basemapRef : null,
_replacedMap : null,
_oldbaseMap : null,
_mapType : 0,

    _listEntry :
    {
        modelType : ttMap,
        xmlTag : "map",
        tocTag: "Map",
        priority : 160,
        itemtypeEnum : CI.ttIItem.kItemMap,
        idlType : CI.ttIMap
    },
get significantIDs()
    {
        let result = ttNavItem.prototype.__lookupGetter__("significantIDs").apply(this);
        if (this._baseMap)
            result.push("_basemap/" + this._baseMap);
        else
            result.push("_basemap/" + this._idstr);
        
        if (this._oldbaseMap)
            result.push("_basemap/" + this._oldbaseMap);
            
        if (this._replacedMap)
            result.push("_replacedmap/" +this._replacedMap);
            
        return result;
    },
get regions()
    {
        return this._coveredRegions;
    },

    get version()
    {
        var result = ttNavItem.prototype.__lookupGetter__("version").call(this);
        if (this._buildNumber)
        {
            result += "." + this._buildNumber;
        }
        return result;
    },
    set version(val)
    {
        return ttNavItem.prototype.__lookupSetter__("version").call(this, val);
    },
get basemap()
    {
        return this._baseMap;
    },
get replacedMap()
    {
        return this._replacedMap;
    },
get oldbaseMap()
    {
        return this._oldbaseMap;
    },
get mapType()
    {
        if (this.basemap)
        {
            if (this.basemap == this.idstr)
                this._mapType = 1;
            else
                this._mapType = 2;
        }
        return this._mapType;
    },
get basemapRef()
    {
        return this._basemapRef;
    },
set basemapRef(newVal)
    {
        assert(this.mapType == 2, "Base map reference can only be set on a zoned map");
        assert(newVal == null || newVal instanceof ttMap, "Base map reference should be a ttMap");
        assert(newVal == null || newVal.mapType == 1, "Base map reference must be a zoned full map");
        this._basemapRef = newVal;
    },

    get mapzonePreview()
    {
        return this._mapzonePreview;
    },

    set mapzonePreview(newVal)
    {
        assert(newVal instanceof ttImage || newVal == null, "Map Zone Preview must be a ttImage");
        this._mapzonePreview = newVal;
        this._ensureXML();

        for each (let image in this._xml.image.(@usage == "mapzone"))
        {
            delete this._xml.children()[image.childIndex()];
        }

        if (newVal)
        {
            var xml = newVal.exportToXML();
            xml.@usage = "mapzone";
            this._xml.appendChild(xml);
        }
    },

    createTOC : function()
    {
        var table = ttNavItem.prototype.createTOC.apply(this, arguments);
        var toc = table.elements()[0];

        if (this.basemap)
            toc.@basemap = this.basemap;
        if (this.replacedMap)
        {
            toc.ReplacedMap = this.replacedMap;
        }
        if (this.oldbaseMap)
        {
            toc.@oldbasemap = this.oldbaseMap;
        }

        return table;
    },

    importFromXML : function(xml)
    {
        ttNavItem.prototype.importFromXML.apply(this, arguments);
        if (xml.hasOwnProperty("@replacedmap"))
        {
            this._replacedMap = makeSafe.string(xml.@replacedmap);
        }
        if (xml.hasOwnProperty("@oldbasemap"))
        {
            this._oldbaseMap = makeSafe.string(xml.@oldbasemap);
        }
        if ("build" in xml)
        {
            this._buildNumber = makeSafe.integer(xml.build);
        }
        if ("regions" in xml)
        {
            this._regions = [];
            for each (let region in xml.regions.region)
            {
                let regionItem = new ttRegion();
                regionItem.importFromXML(region);
                this._regions.push(regionItem);
            }
        }

        if ("@basemap" in xml)
            this._baseMap = makeSafe.contentidstr(xml.@basemap);

        if ("region" in xml)
        {   
            try {
                this._coveredRegions = makeSafe.list(xml.region, makeCallback(makeSafe, makeSafe.alphanumspace));
            } catch (e) { 
                unimportantError(e); 
            }
        }

        for each (let mapzonePreview in xml.image.(@usage == "mapzone"))
        {
            try {
                this._mapzonePreview = ttImage.prototype.createFromServer(mapzonePreview);
            } catch (e) { unimportantError(e); }
        }

    },

    importFromDLL : function(item)
    {
        ttNavItem.prototype.importFromDLL.apply(this, arguments);
        this._dll = this._dll.QueryInterface(CI.ttIMap);

        if (this._dll.region)
        {
            try {
                this._coveredRegions = makeSafe.list(this._dll.region, makeCallback(makeSafe, makeSafe.alphanumspace));
            } catch (e) { 
                unimportantError(e); 
            }
        }

        if (this._dll.buildNumber)
            this._buildNumber = makeSafe.integer(this._dll.buildNumber);
        else{}
if (this.locationType == 3 && !this._hasHOME2Metadata)
        {
            this._drm = true;
            this._drmScheme = "activation-code";
            this._drmNeedsKey = true;
        }

        if (this._dll.incompleteMap)
        {
            this._isIncompleteInstall = makeSafe.bool(this._dll.incompleteMap);
        }
    },
createCopyWithActivationCode : function(activationCode)
    {
        assert(this._drm && this._drmScheme == "activation-code" && this._drmNeedsKey, "item missing activation code expected");

        return this._createModifiedItem(function(item) {
            item.drmActivationCode = activationCode;
            item.drmNeedsKey = false;
            item.drmDeviceCode = gApplication.deviceManager.activeDevice.deviceCode;
        });
    },
canInstallOn : function(device)
    {
        let canInstallOn = ttNavItem.prototype.canInstallOn.apply(this, arguments);
        let isZoneableMap = this.basemap && (this.basemap == this.idstr);
        let isMapTooBig = canInstallOn == 5;

        if (isZoneableMap && isMapTooBig)
            canInstallOn = 0;

        return canInstallOn;
    },

  __end : null
}
extend(ttMap, ttNavItem);
AddSubtypeV1(ttMap.prototype._listEntry);
