/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/


EXPORTED_SYMBOLS.push(
    "ttVoice",
    "ttTTSVoice",
    "ttLoquendoVoice",
    "ttVocalizerVoice"
);
function ttVoice()
{
    ttNavItem.apply(this);
    ttAudioSampleMixin.apply(this);
}
ttVoice.prototype =
{
  _gender : null,
  _voiceName : null,
  _language : null,

  _listEntry :
  {
    modelType : ttVoice,
    xmlTag : "voice",
    tocTag: "Voice",
    priority : 150,
    itemtypeEnum : CI.ttIItem.kItemVoice,
    idlType : CI.ttIVoice
  },

  importFromDLL : function(item)
  {
    ttNavItem.prototype.importFromDLL.apply(this, arguments);
    item = item.QueryInterface(CI.ttIVoice);
    var gender = item.gender;
    if (gender == CI.ttIVoice.kGenderMale)
        this._gender = "m";
    else if (gender == CI.ttIVoice.kGenderFemale)
        this._gender = "f";
    if (item.contentLanguage)
        this._language = makeSafe.languageCode(item.contentLanguage);
  },
  importFromXML : function(xml)
  {
    ttNavItem.prototype.importFromXML.apply(this, arguments);
    this._importFromXMLTTAudioSample(xml)

    this._language = makeSafe.languageCode(xml.language);

    this._gender = makeSafe.enumeration(xml.gender, ["m", "f", "youngfemale", "youngmale", "animal", "computer", "other"]);
  },
  get gender()
  {
    return this._gender;
  },
  get voiceName()
  {
    return this._voiceName;
  },
  get language()
  {
    return this._language;
  },

  createTOC : function()
  {
    var table = ttNavItem.prototype.createTOC.apply(this, arguments);
    var toc = table.elements()[0];

    toc.@language = this.language;

    if (this.gender == "m")
        toc.@gender = "male";
    else if (this.gender == "f")
        toc.@gender = "female";
    else if (this.gender == "youngfemale")
        toc.@gender = "young female";
    else
        toc.@gender = "male";

    return table;
  },

  compareTo : function(other)
  {
    if (other instanceof ttVoice || other instanceof ttLoquendoVoice)
    {
        if (this.language != other.language) {
            if (this.language == gApplication.settings.uiLocale)
                return -1;
            else if (other.language == gApplication.settings.uiLocale)
                return 1;
        }
        var langNameThis = getLanguageName(this.language);
        var langNameOther = getLanguageName(other.language);
        if (langNameThis < langNameOther)
            return -1;
        else if (langNameThis > langNameOther)
            return 1;
    }
    return 0;
  },

  set gender(newVal)
  {
    this._gender = makeSafe.enumeration(newVal, ["m", "f", "youngfemale", "youngmale", "animal", "computer", "other"]);
    this._ensureXML();
    this._xml.gender = newVal;
  },
  set language(newVal)
  {
    this._language = makeSafe.languageCode(newVal);
    this._ensureXML();
    this._xml.language = newVal;
  },
_ensureXML : function()
  {
    if (this._xml)
      return;
    ttItem.prototype._ensureXML.apply(this, arguments);
    if (this.language)
        this.language = this.language;
    if (this.gender)
        this.gender = this.gender;{}
},

  __end : null
}
extend(ttVoice, ttNavItem);
extendMixIn(ttVoice, ttAudioSampleMixin);
AddSubtypeV1(ttVoice.prototype._listEntry);
function ttTTSVoice()
{
    ttVoice.apply(this);
}
ttTTSVoice.prototype =
{
    _engineVersion : null,

    importFromDLL : function(item)
    {
        ttVoice.prototype.importFromDLL.apply(this, arguments);
        item = item.QueryInterface(CI.ttITTSVoice);
        this._engineVersion = makeSafe.version(item.engineVersion);
    },
    importFromXML : function(xml)
    {
        ttVoice.prototype.importFromXML.apply(this, arguments);
        if ("engineversion" in xml)
            this._engineVersion = makeSafe.version(xml.engineversion);
    },

    get displayCategory()
    {
        return "voice";
    },
    get engineVersion()
    {
        return this._engineVersion;
    },

    set engineVersion(newVal)
    {
        this._engineVersion = makeSafe.version(newVal);
        this._ensureXML();
        this._xml.engineversion = newVal;
    },

    createTOC : function()
    {
        var table = ttVoice.prototype.createTOC.apply(this, arguments);
        var toc = table.elements()[0];

        if (this.engineVersion)
            toc.@engineVersion = this.engineVersion;

        return table;
    },

    _ensureXML : function()
    {
        if (this._xml)
            return;
        ttVoice.prototype._ensureXML.apply(this, arguments);
        this.engineVersion = this.engineVersion;
    }
}
extend(ttTTSVoice, ttVoice);
function ttLoquendoVoice()
{
    ttTTSVoice.apply(this);
}
ttLoquendoVoice.prototype =
{
    _listEntry :
    {
        modelType : ttLoquendoVoice,
        xmlTag : "loquendovoice",
        tocTag : "LoquendoVoice",
        priority : 10,
        itemtypeEnum : CI.ttIItem.kItemLoquendoVoice,
        idlType : CI.ttITTSVoice
    }
}
extend(ttLoquendoVoice, ttTTSVoice);
AddSubtypeV1(ttLoquendoVoice.prototype._listEntry);
function ttVocalizerVoice()
{
    ttTTSVoice.apply(this);
}
ttVocalizerVoice.prototype =
{
    _listEntry :
    {
        modelType : ttVocalizerVoice,
        xmlTag : "vocalizervoice",
        tocTag : "VocalizerVoice",
        priority : 10,
        itemtypeEnum : CI.ttIItem.kItemVocalizerVoice,
        idlType : CI.ttITTSVoice
    }
}
extend(ttVocalizerVoice, ttTTSVoice);
AddSubtypeV1(ttVocalizerVoice.prototype._listEntry);
