/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/


EXPORTED_SYMBOLS.push("ttCurrency");
function ttCurrency(value, decpoint, currencyCode)
{
  this.value = value;
  this.decpoint = decpoint;
  this.currencyCode = currencyCode;
}
ttCurrency.prototype =
{
  value : null,
  decpoint : null,
  currencyCode : null,

  get free()
  {
    return this.value == 0;
  },
  set free(value)
  {
    assert(value, "can set free only to true");

    this.value = 0;
  },

  displayString : function()
  {
    if (!this.__proto__._formatStringBundle)
    {
        var serv = Cc["@mozilla.org/intl/stringbundle;1"]
                   .getService(CI.nsIStringBundleService);
        this.__proto__._formatStringBundle =
            serv.createBundle("chrome://tthome/locale/logic/model/currencyFormat.properties");
        this.__proto__._currenciesStringBundle =
            serv.createBundle("chrome://tthome/locale/logic/model/currencies.properties");
    }
    var sbFormat = this.__proto__._formatStringBundle;

    if (this.free)
        return sbFormat.GetStringFromName("free");
    var localeDecimalSep = sbFormat.GetStringFromName("decimalSeparator");
    var localeThousandSep = sbFormat.GetStringFromName("thousandSeparator");
    localeThousandSep = localeThousandSep.replace("_", " ");
    var localeCurrencySep = sbFormat.GetStringFromName("currencySeparator");
    localeCurrencySep = localeCurrencySep.replace("_", " ");
    var localeCurrencyAfter = sbFormat.GetStringFromName("currencyAfterValue") != "false";
    var localeCurrencyName = this.currencyCode;
    try {
        localeCurrencyName = this.__proto__._currenciesStringBundle
                             .GetStringFromName(this.currencyCode);
    } catch (e) {}

    var numstr = this.value.toFixed(this.decpoint);
    var dotPos = numstr.indexOf(".");
    if (dotPos >= 0)
        numstr[dotPos] = localeDecimalSep;
    else
        dotPos = numstr.length;

    for (var i = dotPos - 3; i > 0; i -= 3)
        numstr = numstr.substr(0, i) + localeThousandSep + numstr.substr(i);

    var result = "";
    if (!localeCurrencyAfter)
        result += localeCurrencyName + localeCurrencySep;
    result += numstr;
    if (localeCurrencyAfter)
        result += localeCurrencySep + localeCurrencyName;
    return result;
  },

  toString : function()
  {
    return this.displayString();
  },
createFromServer : function(xml)
  {
    var result = new ttCurrency();
    if ("@amount" in xml)
    {
        [result.value, result.decpoint] = this.parseCurrencyString(xml.@amount);
        result.currencyCode = makeSafe.alphanum(xml.@currency);
    }
    if (xml.@free == "true")
    {
        result.free = true;
    }
    else if (result.value == 0)
        throw new Bug("Server says this costs nothing, but didn't say that it's free. This makes no sense (inconsistent).", xml);
    return result;
  },
parseCurrencyString : function(string)
  {
    string = makeSafe.nonemptystring(string);

    var value = makeSafe.float(string);

    var dotPos = string.indexOf(".");
    if (dotPos < 0)
        dotPos = string.length - 1;
    var decpoint = makeSafe.rangeInt(string.length - 1 - dotPos, 0, 5);

    return [value, decpoint];
  },
add : function(a, b)
  {
    if (!a || a.value == 0)
        return (b ? b.copy() : null);
    if (!b || b.value == 0)
        return a.copy();
    if (!a.compatible(b))
        throw new InvalidValue("Mismatching currencies", a + " vs. " + b);
    var result = a.copy();
    result.value = a.value + b.value;
    return result;
  },
substract : function(a, b)
  {
    if (!a || a.value == 0)
        return null;
    if (!b || b.value == 0)
        return a.copy();
    if (!a.compatible(b))
        throw new InvalidValue("Mismatching currencies", a + " vs. " + b);
    var result = a.copy();
    result.value = a.value - b.value;
    return result;
  },
multiply : function(a, b)
  {
    var result = a.copy();
    result.value *= b;
    return result;
  },
divide : function(a, b)
  {
    if (b == 0)
        throw new InvalidValue("Division by 0", b);
    var result = a.copy();
    result.value /= b;
    return result;
  },
compatible : function(a)
  {
    if (!a)
        return false;
    return this.decpoint == a.decpoint && this.currencyCode == a.currencyCode;
  },
copy : function()
  {
    var result = new ttCurrency();
    result.value = this.value;
    result.decpoint = this.decpoint;
    result.currencyCode = this.currencyCode;
    return result;
  },
  exportToXML : function()
  {
    var xml = <price/>
    if (this.free)
        xml.@free = "true";
    else
    {
        xml.@amount = this.value.toFixed(this.decpoint);
        xml.@currency = this.currencyCode;
    }
    return xml;
  },
  __end : null
}
