/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/

EXPORTED_SYMBOLS.push(
    "ttItemList",
    "ttItemListObserver",
    "ttItemListFilter",
    "ttDuplicateFilter",
    "ttNonRemovableFilter",
    "ttMapHierarchyFilter"
);
function ttItemList()
{
    this._showItems = new Array();
    this._iObserve = new Array();
    this._iAmObservedBy = new Array();
    this._filters = [];
}
ttItemList.prototype =
{
    _showItems : null,
    _iObserve : null,
    _iAmObservedBy : null,
_filters : null,

    destructor : function()
    {
        this._clearObservers();
    },

    _clearObservers : function()
    {
        while (this._iObserve.length != 0)
        {
            var cur = this._iObserve.pop();
            cur.dllItemList.removeObserver(cur.observer);
        }
    },
addItemList : function(itemList)
    {
        assert(itemList instanceof ttItemList, "itemList must be a ttItemList");

        let items = itemList.contents.slice();
        items.sort(this._compareItems);
        for each (let item in items)
            this.addItem(item);

        let observer = new ttItemList_ItemListObserver(this);
        itemList.addObserver(observer);
        this._iObserve.push({ dllItemList: itemList, observer: observer });
    },
addItemListFromDLL : function(dllItemList, onlyType)
    {
        var models = gApplication.itemFactory.createItemListFromDLL(dllItemList, onlyType);
        models.sort(this._compareItems);
        for (var i = 0; i < models.length; i++)
        {
            this.addItem(models[i]);
        }

        var observer = new ttItemList_DLLObserver(this, onlyType);
        dllItemList.addObserver(observer);{}
this._iObserve.push({ dllItemList: dllItemList, observer: observer });
        return models;
    },
addItemListFromServer : function(xml)
    {
        var models = gApplication.itemFactory.createItemListFromServer(xml);
        models.sort(this._compareItems);
        for (var i = 0; i < models.length; i++)
        {
            this.addItem(models[i]);
        }
        return models;
    },
addItemListArray : function(array, noSort)
    {
        if (!noSort)
            array.sort(this._compareItems);
        for (var i = 0; i < array.length; i++)
        {
            this.addItem(array[i]);
        }
    },

    addItem : function(ttitem)
    {
        var items = [ttitem];
        for each (let filter in this._filters)
        {
            for (let i = 0; i < items.length; i++)
            {
                let item = items[i];
                let replacement = filter.checkItem(item);
                if (replacement.length != 1 || replacement[0] != item)
                {
items.splice.apply(items, [i, 1].concat(replacement));
                    i += replacement.length - 1;
                }
            }
        }
        for each (var item in items)
            this._addItem(item);
    },

    removeItem : function(ttitem)
    {
        this._removeItem(ttitem);
        for each (let filter in this._filters)
            filter.resetItem(ttitem);
    },
updateItem : function(item)
    {
        this._triggerObservers("updated", item);
    },
get contents()
    {
        return this._showItems;
    },
get length()
    {
        return this._showItems.length;
    },
addFilter : function(filter)
    {
        assert(filter instanceof ttItemListFilter, "A filter must be a subclass of ttItemListFilter");

        this._filters.push(filter);
        filter.itemlist = this;
    },
removeFilter : function(filter)
    {
        ArrayRemoveAllMatching(this._filters, filter);
    },

    clear : function()
    {
        this._clearObservers();
        while (this._showItems.length > 0)
        {
            this._removeItem(this._showItems[0]);
        }
        for each (let filter in this._filters)
            filter.resetAll();
        
    },
sort : function(sortFunction)
    {
        for each (var observer in this._iAmObservedBy)
        {
            for each (var item in this._showItems)
            {
                try {
                    observer.removedItem(item);
                } catch (e) { error(e); }
            }
        }
        this._showItems.sort(this._compareItems);
        if (sortFunction)
            this._showItems.sort(sortFunction);
        for each (var observer in this._iAmObservedBy)
        {
            for each (var item in this._showItems)
            {
                try {
                    observer.addedItem(item);
                } catch (e) { error(e); }
            }
        }
    },

    addObserver : function(observer)
    {
        this._iAmObservedBy.push(observer);
    },
    removeObserver : function(observer)
    {
        ArrayRemoveAllMatching(this._iAmObservedBy, observer);
    },

    debugContents : function()
    {
        for (var i = 0; i < this.contents.length; i++)
        {
            var item = this.contents[i];{}
}
    },
_triggerObservers : function(action, item)
    {
        for each (var observer in this._iAmObservedBy)
        {
            try {
                if (typeof observer[action + "Item"] == "function")
                    observer[action + "Item"](item);
                else
                    unimportantError(new Exception("Item list observer doesn't have method " + action + "Item, ignoring."));
            } catch (e) { error(e); }
        }
    },

    _addItem : function(ttitem)
    {
        this._showItems.push(ttitem);
        this._triggerObservers("added", ttitem);
    },

    _removeItem : function(ttitem)
    {
        var itempos = this._showItems.indexOf(ttitem);
        if (itempos < 0)
            return;

        this._showItems.splice(itempos, 1);
        this._triggerObservers("removed", ttitem);
    },

    _compareItems : function(item1, item2)
    {

        var locationPriority =
        {
            1 : 1,
            2 : 4,
            3 : 4,
            4 : 3,
            6 : 3,
            __end: null
        };

        var locPriority1 = (item1.isSharing ? 2 : locationPriority[item1.locationType] || 0);
        var locPriority2 = (item2.isSharing ? 2 : locationPriority[item2.locationType] || 0);

        if (locPriority1 != locPriority2)
            return locPriority2 - locPriority1;
        if (item1.typePriority != item2.typePriority)
            return item2.typePriority - item1.typePriority;
        var itemComparison = item1.compareTo(item2);
        if (itemComparison != 0)
            return itemComparison;
        var pricePriority1 = (item1.price && !item1.price.free ? 1 : 0);
        var pricePriority2 = (item2.price && !item2.price.free ? 1 : 0);
        if (pricePriority1 != pricePriority2)
            return pricePriority2 - pricePriority1;
        var name1 = item1.name.toLowerCase();
        var name2 = item2.name.toLowerCase();
        if (name1 < name2)
            return -1;
        else if (name1 > name2)
            return 1;
        else
            return 0;
    },

    __end : null
}
function ttItemListObserver()
{
}
ttItemListObserver.prototype =
{
    addedItem : function(ttitem)
    {
        throw new NotImplemented("Abstract function - Override this");
    },
    removedItem : function(ttitem)
    {
        throw new NotImplemented("Abstract function - Override this");
    }
}
function ttItemList_DLLObserver(itemlist, onlyType)
{
    this._list = itemlist;
    this._onlyType = onlyType;
}
ttItemList_DLLObserver.prototype =
{
    added : function(item)
    {
item = item.QueryInterface(CI.ttIItem);
        model = gApplication.itemFactory.createItemFromDLL(item, this._onlyType);
        if (model)
            this._list.addItem(model);
    },
    removed : function(item)
    {
item = item.QueryInterface(CI.ttIItem);
        for (var i = 0; i < this._list._showItems.length; i++)
        {
            var cur = this._list._showItems[i];
            if (!cur.isServer && cur._dll == item)
            {
                this._list.removeItem(cur);
                break;
            }
        }
    },
    repopulated : function()
    {
},
    QueryInterface : function(iid)
    {
        if (iid.equals(CI.ttICollectionObserver) ||
            iid.equals(CI.nsISupportsWeakReference) ||
            iid.equals(CI.nsISupports)) {
            return this;
        }
        throw Components.results.NS_NOINTERFACE;
    }
}
function ttItemList_ItemListObserver(itemlist)
{
    this._list = itemlist;
}
ttItemList_ItemListObserver.prototype =
{
    addedItem : function(item)
    {
        this._list.addItem(item);
    },
    removedItem : function(item)
    {
        this._list.removeItem(item);
    }
}
function ttItemListFilter()
{
}
ttItemListFilter.prototype =
{
_itemlist : null,
get itemlist()
    {
        return this._itemlist;
    },
    set itemlist(value)
    {
        this.resetAll();
        return (this._itemlist = value);
    },
checkItem : function(item)
    {
        return [item];
    },
resetItem : function(item)
    {
    },
resetAll : function()
    {
    }
}
function ttDuplicateFilter()
{
    ttItemListFilter.apply(this);
    this.resetAll();
}
ttDuplicateFilter.prototype =
{
_knownItems : null,

    checkItem : function(item)
    {
        var checkIdstrlong = true;
        if (this._knownItems[item.idstrlong] && item.basemap)
        {
            checkIdstrlong = this._knownItems[item.idstrlong].basemap == item.basemap;
        }
        let previous = checkIdstrlong ?
                       this._knownItems[item.idurl] || this._knownItems[item.idstrlong] || null :
                       this._knownItems[item.idurl] || null; 

        if (previous)
        {
            let device = gApplication.deviceManager.activeDevice;

            if (device)
            {
                let newIsNavItem = item instanceof ttNavItem;
                let newIsCompatible = !newIsNavItem || (item.canInstallOn(device) == 0);
                let previousIsNavItem = previous instanceof ttNavItem;
                let previousIsCompatible = !previousIsNavItem || (previous.canInstallOn(device) == 0);
                let isSameVersion = !item.newerThan(previous) && !previous.newerThan(item);

                if ((!previousIsCompatible && (newIsCompatible || item.newerThan(previous))) ||
                    (previousIsCompatible && newIsCompatible && item.newerThan(previous)) ||
                    (isSameVersion && !previousIsNavItem && newIsNavItem && newIsCompatible))
                {
                    this.itemlist.removeItem(previous);
                }
                else
                    return [];
            }
            else
            {
                if (item.newerThan(previous))
                    this.itemlist.removeItem(previous);
                else
                    return [];
            }
        }
        this._knownItems[item.idurl] = item;
        this._knownItems[item.idstrlong] = item;
        return [item];
    },

    resetItem : function(item)
    {
        if (item.idurl in this._knownItems)
            delete this._knownItems[item.idurl];
        if (item.idstrlong in this._knownItems)
            delete this._knownItems[item.idstrlong];
    },

    resetAll : function()
    {
        this._knownItems = {__proto__: null};
    }
}
extend(ttDuplicateFilter, ttItemListFilter);
function ttNonRemovableFilter(doNotShowItems)
{
    ttItemListFilter.apply(this);

    this._blacklist = {};
    if (doNotShowItems)
    {
        for each (let item in doNotShowItems)
            for each (let id in item.significantIDs)
                this._blacklist[id] = item;
    }
}
ttNonRemovableFilter.prototype =
{
    _blacklist : null,

    checkItem : function(item)
    {
        if (item.significantIDs.some(function(id) id in this._blacklist, this) ||
            item instanceof ttPartialUpdate ||
            !item.sourceDLL.canDeleteFromDevice ||
            item instanceof ttProgram)
        {
            return [];
        }

        return [item];
    }
}
extend(ttNonRemovableFilter, ttItemListFilter);
function ttMapHierarchyFilter()
{
    ttItemListFilter.apply(this);
    this.resetAll();
}
ttMapHierarchyFilter.prototype =
{
_filteredOutZonableFullMaps : null,

    checkItem : function(item)
    {
        if (!(item instanceof ttMap))
            return [item];

        if (item.mapType == 1)
            return this._linkFullMapToZones(item);
        else if (item.mapType == 2)
            return this._linkZoneToFullMap(item);
        return [item];
    },

    _linkFullMapToZones : function(fullMap)
    {
        let zoneFound = false;
        for each (let existingItem in this.itemlist.contents)
        {
            if (existingItem instanceof ttMap         &&
                existingItem.mapType == 2             &&
                (existingItem.basemap == fullMap.idstr || 
                (fullMap.oldbaseMap && fullMap.oldbaseMap == existingItem.basemap)))
            {
                existingItem.basemapRef = fullMap;
                this.itemlist.updateItem(existingItem);
                zoneFound = true;
            }
        }

        if (zoneFound)
        {
            this._filteredOutZonableFullMaps.push(fullMap);
            return [];
        }
        else
            return [fullMap]
    },

    _linkZoneToFullMap : function(mapZone)
    {
        let fullMapFound = false;
        for each (let existingItem in this.itemlist.contents)
        {
            if (existingItem instanceof ttMap           &&
                existingItem.mapType == 1               &&
                (existingItem.idstr   == mapZone.basemap ||
                (mapZone.oldbaseMap && mapZone.oldbaseMap == existingItem.idstr)))
            {
                mapZone.basemapRef = existingItem;
                this.itemlist.removeItem(existingItem);
                this._filteredOutZonableFullMaps.push(existingItem);
                fullMapFound = true;
                break;
            }
        }
        if (!fullMapFound)
        {
            for each (zonableFullMap in this._filteredOutZonableFullMaps)
            {
                if (zonableFullMap.idstr == mapZone.basemap ||
                (mapZone.oldbaseMap && mapZone.oldbaseMap == zonableFullMap.idstr)) {
                    mapZone.basemapRef = zonableFullMap;
                    break;
                }
            }
        }
        return [mapZone];
    },

    resetAll : function()
    {
        this._filteredOutZonableFullMaps = [];
    }
}
extend(ttMapHierarchyFilter, ttItemListFilter);
