/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/

EXPORTED_SYMBOLS.push("ttItemRef");
function ttItemRef(type, idstr, version, idurl, idnum, dependencyType)
{
    this.type = type;
    this.idstr = idstr;
    if (version)
        this.version = version;
    if (idurl)
        this.idurl = idurl;
    if (idnum)
        this.idnum = idnum;
    if (dependencyType)
        this.dependencyType = dependencyType;
}
ttItemRef.prototype =
{
type: null,
idstr: null,
version: null,
idurl: null,
idnum: null,
dependencyType: null,
get idstrlong()
    {
        return (this.type ? this.type.xmlTag : "unknown") + "/" + this.idstr;
    },
matches: function(item)
    {
        assert(item instanceof ttItem, "need ttItem");

        if (item.listEntry.xmlTag != this.type.xmlTag || item.idstr != this.idstr)
            return false;
        if (this.version != null && this.version != item.version)
            return false;
        if (this.idurl != null && item.hasIDURL && this.idurl != item.idurl)
            return false;
        if (this.idnum != null && item.idnum != null && this.idnum != item.idnum)
            return false;

        return true;
    },
createFromServer: function(xml)
    {
        assert(typeof xml == "xml", "need XML");

        var type = makeSafe.alphanum(xml.@type);
        var listEntry = gApplication.itemFactory.findSubtype("xmlTag", type);
        if (!listEntry)
            throw new InvalidValue(getStringBundle("chrome://tthome/locale/logic/model/ttitem.properties").GetStringFromName("unknownItemType"), type);
        var idstr = makeSafe.contentidstr(xml.@idstr);
        var version = null;
        if ("@version" in xml)
            version = makeSafe.version(xml.@version);
        var idurl = null;
        if ("@idurl" in xml)
            idurl = makeSafe.contentidurl(xml.@idurl);
        var idnum = null;
        if ("@idnum" in xml)
            idnum = makeSafe.contentidnum(xml.@idnum)
        var dependencyType = null;
        if ("@dependencytype" in xml)
            dependencyType = makeSafe.string(xml.@dependencytype);
        return new ttItemRef(listEntry, idstr, version, idurl, idnum, dependencyType);
    },
createFromDLL : function(dependency)
    {
        assert(dependency instanceof CI.ttIItemDependency, "need ttIItemDependency");

        var type = makeSafe.integer(dependency.dependsOnItemType);
        var listEntry = gApplication.itemFactory.findSubtype("itemtypeEnum", type);
        if (!listEntry)
            throw new InvalidValue(getStringBundle("chrome://tthome/locale/logic/model/ttitem.properties").GetStringFromName("unknownItemType"), type);

        var idstr = makeSafe.contentidstr(dependency.dependsOnItemID);
        var version = null;
        if (dependency.dependsOnItemVersion)
            version = makeSafe.version(dependency.dependsOnItemVersion);

        return new ttItemRef(listEntry, idstr, version);
    },
toXML : function()
    {
        var xml = <itemref type={this.type.xmlTag} idstr={this.idstr}/>;
        if (this.version)
            xml.@version = this.version;
        if (this.idurl)
            xml.@idurl = this.idurl;
        if (this.idnum)
            xml.@idnum = this.idnum;
        if (this.dependencyType)
            xml.@dependencytype = this.dependencyType;
        return xml;
    }
}
