/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/


EXPORTED_SYMBOLS.push(
    "kErrorUserCancelled",
    "NetUtil",
    "ServerError",
    "MozillaException"
);
const kHomeBase = 0x80760000;
const kErrorUserCancelled = kHomeBase + 1;

const kNetErrorStringbundleURI = "chrome://tthome/locale/logic/netutil.properties";
const NetUtil =
{
    _kNetErrorStringbundle: null,

    _getNetErrorStringbundle: function ()
    {
        if (!this._kNetErrorStringbundle)
            this._kNetErrorStringbundle = getStringBundle(kNetErrorStringbundleURI);

        return this._kNetErrorStringbundle;
    },
_nameForException: function (ex)
    {
        if (!(ex instanceof CI.nsIException))
            return null;

        if (ex.name)
            return ex.name.replace(/^NS_(ERROR_)?/, "");
        var nssError = null;
        nssError = this.nssMessageForStatusCode(ex.result);
        if (nssError)
            return nssError;
        var module = ((ex.result & ~0x80000000) >> 16) - 0x45;
        var errorNum = (ex.result & 0xFFFF);
        return "Unknown error code: module " + module + ", error number " + errorNum;
    },
nssMessageForStatusCode: function (status)
    {
        var message = null;
        try {
            message = Cc["@mozilla.org/psm;1"].getService(CI.nsINSSErrorsService)
                .getErrorMessage(status);
        } catch (e) {
}

        return message;
    },
errorMessageForException: function (ex)
    {
        var statusName = this._nameForException(ex);

        if (!statusName)
            return null;

        try
        {
            return this._getNetErrorStringbundle().GetStringFromName(statusName);
        }
        catch (e)
        {
return statusName;
        }
    },
exceptionForStatusCode: function (status, uri, additionalInfo)
    {
        var exception = Cc["@mozilla.org/js/xpc/Exception;1"].createInstance(CI.nsIXPCException);
        exception.initialize(null, status, null, null, null, null);
        var message = NetUtil.errorMessageForException(exception);
        if (!message)
            message = this._getNetErrorStringbundle().formatStringFromName("unknownError", [status], 1);
        return new ServerError(message, status, uri, undefined, additionalInfo);
    },
isRequestSuccess: function (aChannel, aStatus, additionalInfo)
    {
        if (Components.isSuccessCode(aStatus) || aStatus == 0x80540008 || aStatus == 0x805D0021)
        {
            if (aChannel instanceof CI.nsIHttpChannel)
            {
                aChannel.QueryInterface(CI.nsIHttpChannel);
try
                {
                    if (200 <= aChannel.responseStatus &&
                        aChannel.responseStatus < 300)
                    {
                        return null;
                    }
                    else
                    {
                        var responseStatusCode = aChannel.responseStatus;
                        var responseStatusText = aChannel.responseStatusText;
                        return new ServerError(responseStatusText, -responseStatusCode, aChannel.URI.spec, undefined, additionalInfo);
                    }
                }
                catch (e)
                {
                    let exception = NetUtil.exceptionForStatusCode(Components.results.NS_ERROR_OFFLINE, aChannel.URI.spec, additionalInfo);
                    exception.inner = e;
                    return exception;
                }
            }
            else
            {
return null;
            }
        } else {
            return NetUtil.exceptionForStatusCode(aStatus, aChannel.URI.spec, additionalInfo);
        }
    }
};
function ServerError(errorStr, errorCode, url, severity, additionalInfo, id)
{
    var extendedID = (id ? id + "/#" + errorCode + "/" : errorCode + "/#") + url;
    Exception.call(this, null, extendedID);
    this.name = "Error from server";
    this.code = errorCode;
    this.url = url;
    if (errorStr)
        this.message = errorStr;
    this.additionalInfo = additionalInfo;
    if (severity != null)
        this.severity = severity;
    this.errorStr = errorStr;
}

ServerError.prototype =
{
    code : null,
    url : null,
    additionalInfo : null,
    func : null,
    debugInfoFromServer : null,

    get message()
    {
        if (this.code >= 100 && this.code < 200)
            return NetUtil._getNetErrorStringbundle().GetStringFromName("ServerProtocolError");
        if ((this.code >= 200 && this.code < 400) ||
            (this.code <= -500 && this.code > -600))
            return NetUtil._getNetErrorStringbundle().GetStringFromName("InternalServerError");
        if (this.code >= 780 && this.code < 790)
            return this._message;
        return NetUtil._getNetErrorStringbundle().formatStringFromName("ServerErrorPrefix", [ this._message ], 1);
    },
    set message(message)
    {
        return this._message = message;
    },
set debugInfo(debugInfo)
    {
        throw new Bug("Setting debugInfo not supported on ServerError");
    },
    get debugInfo()
    {
        var result = this.errorStr;
        if (this.code <= -100 && this.code > -1000)
            result += "\nError code: HTTP " + (-this.code);
        else
        {
            var exception = Cc["@mozilla.org/js/xpc/Exception;1"].createInstance(CI.nsIXPCException);
            exception.initialize(null, this.code, null, null, null, null);
            if (exception.name)
                result += "\nError code: " + this.code + " (" + exception.name + ")";
            else
            {
                let message = NetUtil.nssMessageForStatusCode(this.code);
                if (message)
                    result += "\nError code: " + this.code + " (" + message + ")";
                else
                    result += "\nError code: " + this.code;
            }
        }

        if (this.debugInfoFromServer)
            result += "\nDebug info from server: " + this.debugInfoFromServer;

        result += "\nURL: " + this.url;

        if (this.additionalInfo)
            result += "\nAdditional information: " + dumpObject(this.additionalInfo, "info", 3);

        return result;
    }
};
extend(ServerError, Exception);
function MozillaException(e)
{
    assert(e instanceof CI.nsIException, "MozillaException wraps only XPCOM exceptions");
    Exception.call(this, null, e.result.toString(16));
    this.name = "Exception from an XPCOM module";
    this._inner = e;
    this.debugInfo = errorToText(e);
}
MozillaException.prototype =
{
    get message()
    {
        var msg = NetUtil.errorMessageForException(this._inner);
        if (msg)
            return msg;
        return this._message;
    },
    set message(message)
    {
        return this._message = message;
    }
};
extend(MozillaException, Exception);
function wrapMozillaException(e)
{
    if (!(e instanceof CI.nsIException) ||
        (e.result >= CI.ttIErrorCode.k_BASE && e.result < CI.ttIErrorCode.k_BASE + 0x1000))
        return e;
    return new MozillaException(e);
}
