/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/


EXPORTED_SYMBOLS.push("GlobalSettings");
function GlobalSettings()
{
}
GlobalSettings.prototype =
{
    _installDir : null,
    _profileDir : null,
    _dataDir : null,
    _version : null,
    _uiLocale : null,
    _networkLocale : null,
    _country : null,
    _prefsFoldersObserver : null,
    _operatingSystem : null,
    _kMapSharePatchCache : "MapShareDownloadCache",
    _previousStartupTime : null,
    _downloadItemsDirs : null,

    get stringBundle()
    {
        var sb = getStringBundle("chrome://tthome/locale/logic/settings.properties");
        return memoize(this, "stringBundle", sb);
    },
get installDir()
    {
        return this._installDir.clone();
    },
get profileDir()
    {
        return this._profileDir.clone();
    },
get dataDir()
    {
        return this._dataDir.clone();
    },
get fromdevItemsDir()
    {
        var fromDevItemsDir = this._prefsFoldersObserver.fromdevItemsDir;
        if (!fromDevItemsDir)
            throw new Exception(this.stringBundle.GetStringFromName("copyDirMissing"));
        return ensureDirExists(fromDevItemsDir.clone());
    },
get downloadItemsDirs()
    {
        var downloadItemsDir = this._prefsFoldersObserver.downloadItemsDir;
        if (!downloadItemsDir)
            throw new Exception(this.stringBundle.GetStringFromName("downloadDirMissing"));
        downloadItemsDir = downloadItemsDir.clone();

        if (this._downloadItemsDirs == null ||
            !this._downloadItemsDirs[0].equals(downloadItemsDir) ||
            !downloadItemsDir.exists())
        {
            ensureDirExists(downloadItemsDir);

            var downloadTempItemsDir = downloadItemsDir.clone();
            downloadTempItemsDir.append("temp");
            ensureDirExists(downloadTempItemsDir);

            var downloadCompleteItemsDir = downloadItemsDir.clone();
            downloadCompleteItemsDir.append("complete");
            ensureDirExists(downloadCompleteItemsDir);
            var entries = downloadItemsDir.directoryEntries;
            var toMove =
                IteratorToArray(
                    downloadItemsDir.directoryEntries,
                    CI.nsIFile).filter(
                    function(file)
                    {
                        return file.leafName != "temp" &&
                            file.leafName != "complete" &&
                            gApplication.itemFactory.findSubtype("xmlTag", file.leafName);
                    });
            for each (let file in toMove)
            {
                var fileName = file.leafName;
                var destDir = downloadCompleteItemsDir;
                try {
                    var newName = destDir.clone();
                    newName.append(fileName);
                    newName = createUniqueDir(newName);
                    removeFile(newName, true);
                    moveFile(file, destDir, newName.leafName);
                } catch (e) {
                    unimportantError(e, "Could not migrate download sub-directory '" + file.path + "' into complete directory '" + destDir.path + "'.");
                }
            }

            this._downloadItemsDirs = [
                downloadItemsDir,
                downloadTempItemsDir,
                downloadCompleteItemsDir
            ];

        }

        return [
            this._downloadItemsDirs[0].clone(),
            this._downloadItemsDirs[1].clone(),
            this._downloadItemsDirs[2].clone()
        ];
    },
get downloadTempItemsDir()
    {
        return this.downloadItemsDirs[1];
    },
get downloadCompleteItemsDir()
    {
        return this.downloadItemsDirs[2];
    },
get sharingItemsDir()
    {
        var sharingItemsDir = this._prefsFoldersObserver.sharingItemsDir;
        if (!sharingItemsDir)
            throw new Exception(this.stringBundle.GetStringFromName("sharedDirMissing"));
        return ensureDirExists(sharingItemsDir.clone());
    },
get backupsDir()
    {
        var backupsDir = this._prefsFoldersObserver.backupsDir;
        if (!backupsDir)
            throw new Exception(this.stringBundle.GetStringFromName("backupsDirMissing"));
        return ensureDirExists(backupsDir.clone());
    },

    get pluginsDir()
    {
        var directoryService = Cc["@mozilla.org/file/directory_service;1"]
                               .getService(CI.nsIProperties);
        var pluginsDir = directoryService.get("APlugns", CI.nsIFile);{}
return pluginsDir.clone();
    },

    get osTempDir()
    {
        var directoryService = Cc["@mozilla.org/file/directory_service;1"]
                               .getService(CI.nsIProperties);
        var dir = directoryService.get("TmpD", CI.nsIFile);{}
return dir.clone();
    },

    get mapShareCacheDir()
    {
        var dir = this.profileDir;
        dir.append(this._kMapSharePatchCache);

        return dir;
    },
get version()
    {
        return this._version;
    },
get uiLocale()
    {
        return this._uiLocale;
    },
get uiCollation()
    {
        var collationFactory = Cc["@mozilla.org/intl/collation-factory;1"]
                               .getService(CI.nsICollationFactory);

        var localeService =  Cc["@mozilla.org/intl/nslocaleservice;1"]
                             .getService(CI.nsILocaleService);
        var currentLocale = localeService.newLocale(this.uiLocale);
        var collation = collationFactory.CreateCollation(currentLocale);

        return collation;
    },
get networkLocale()
    {
        return this._networkLocale;
    },
get country()
    {
        return this._country;
    },
get XPCOMABI(){
        var runtime = Cc["@mozilla.org/xre/app-info;1"].getService(CI.nsIXULRuntime);
        return memoize(this, "XPCOMABI", runtime.XPCOMABI);
    },
get operatingSystemTag(){
        var runtime = Cc["@mozilla.org/xre/app-info;1"].getService(CI.nsIXULRuntime);
        return memoize(this, "operatingSystemTag", runtime.OS);
    },

    get operatingSystem()
    {
        if (!this._operatingSystem)
        {
            var os = this.operatingSystemTag;{}
switch (os)
            {
                case "WINNT":
                    this._operatingSystem = GlobalSettings.kOSWindows;
                    break;
                case "Darwin":
                    this._operatingSystem = GlobalSettings.kOSMacOSX;
                    break;
                case "Linux":
                    this._operatingSystem = GlobalSettings.kOSLinux;
                    break;
                default:
                    this._operatingSystem = GlobalSettings.kOSUnknown;
            }
        }
        return this._operatingSystem;
    },
    isOSWindows : function ()
    {
        return this.operatingSystem == GlobalSettings.kOSWindows;
    },
    isOSMacOSX : function ()
    {
        return this.operatingSystem == GlobalSettings.kOSMacOSX;
    },
    isOSLinux : function ()
    {
        return this.operatingSystem == GlobalSettings.kOSLinux;
    },

    get operatingSystemServerName()
    {
        if (typeof GlobalSettings.osServerNames[this.operatingSystem] == "undefined")
            throw new Exception(this.stringBundle.GetStringFromName("unsupportedOSError"));

        return GlobalSettings.osServerNames[this.operatingSystem];
    },
get previousStartupTime()
    {
        return this._previousStartupTime == null ? null : new Date(this._previousStartupTime * 1000);
    },

    set uiLocale(locale)
    {
        prefService().setCharPref("general.useragent.locale", locale);
        this._uiLocale = locale;
    },
    set networkLocale(locale)
    {
        prefService().setCharPref("general.useragent.contentlocale", locale);
        this._networkLocale = locale;
    },
    set country(country)
    {
        prefService().setCharPref("tomtom.server.country", country);
        this._country = country;
    },
    start : function()
    {
        this._initLocale();
        this._initAppDirectories();
        this._initMisc();
    },
    initLocalItems : function()
    {

        var me = this;
        runAsyncDelayed(function()
        {
            try {
                var ni = gApplication.homeServices.localNavItems();
                ni.addFolder(me.downloadCompleteItemsDir, true, null);
                ni.addFolder(me.fromdevItemsDir, true, null);
                ni.addFolder(me.sharingItemsDir, true, null);
            } catch(e) { error(e); }
        }, 2000);
    },
    _initAppDirectories : function()
    {
        var directoryService = Cc["@mozilla.org/file/directory_service;1"]
                               .getService(CI.nsIProperties);
        this._installDir = directoryService.get("XCurProcD", CI.nsIFile).parent;
        this._profileDir = directoryService.get("ProfD", CI.nsIFile);
        if (this.isOSWindows())
        {
            this._dataDir = directoryService.get("Pers", CI.nsIFile);
        }
        else if (this.isOSMacOSX())
        {
            this._dataDir = directoryService.get("UsrDocs", CI.nsIFile);
        }
        else if (this.isOSLinux())
        {
            this._dataDir = directoryService.get("Home", CI.nsIFile);
        }
        else
            throw new Exception(this.stringBundle.GetStringFromName("unsupportedOSError"));{}
this._dataDir.append("TomTom");
        this._dataDir.append("HOME");{}
{}
{}
},

    _initLocale : function()
    {
        try {
            this._uiLocale = prefService().getCharPref("general.useragent.locale");
            if (!this._uiLocale)
                throw null;
        }
        catch(e)
        {
try {
                var localeService = Cc["@mozilla.org/intl/nslocaleservice;1"]
                                    .getService(CI.nsILocaleService);
                var systemLocale = localeService.getSystemLocale().getCategory("NSILOCALE_CTYPE");
                this._uiLocale = systemLocale;
            }
            catch (e)
            {
                unimportantError(e, "Could not get system locale either: ");
                this._uiLocale = "en-GB";
            }
        }
try {
            this._networkLocale = prefService().getComplexValue("general.useragent.contentlocale",
                                CI.nsIPrefLocalizedString).data;
            if (!this._networkLocale)
                throw null;
        }
        catch(e)
        {
this._networkLocale = this._uiLocale;
        }
try {
            this._country = prefService().getCharPref("tomtom.server.country");
            if (!this._country)
                throw null;
        }
        catch(e)
        {
this._country = this._uiLocale.substr(3, 2);
        }
},

    _initMisc : function()
    {
        try
        {
            this._previousStartupTime = prefService().getIntPref("tomtom.app.lastStartup");
        }
        catch(e){}
        prefService().setIntPref("tomtom.app.lastStartup", Date.now()/1000);

        var xulappinfo = Cc["@mozilla.org/xre/app-info;1"]
                         .createInstance(CI.nsIXULAppInfo);
        this._version = makeSafe.version(xulappinfo.version);
        this._prefsFoldersObserver = new PrefsFoldersObserver();
        this._prefsFoldersObserver.start(this.dataDir);
    },

    __end : null
}
function PrefsFoldersObserver ()
{
}

PrefsFoldersObserver.prototype =
{
    _prefs: null,
    _dataDir: null,
    downloadItemsDir: null,
    backupsDir: null,
    fromdevItemsDir: null,
    _prefToDefaultMap: {
        "tomtom.localitems.download.folder": "Download",
        "tomtom.backup.folder": "Backup",
        "tomtom.localitems.fromdevice.folder": "FromDevice",
        "tomtom.localitems.sharing.folder": "Sharing"
    },
    _prefToFieldMap: {
        "tomtom.localitems.download.folder": "downloadItemsDir",
        "tomtom.backup.folder": "backupsDir",
        "tomtom.localitems.fromdevice.folder": "fromdevItemsDir",
        "tomtom.localitems.sharing.folder": "sharingItemsDir"
    },

    start: function(dataDir)
    {
        this._dataDir = dataDir;

        this._prefs = prefService().QueryInterface(CI.nsIPrefBranch2);
        this._ensureFoldersExist();

        for (var pref in this._prefToFieldMap)
        {
            var field = this._prefToFieldMap[pref];
            try
            {
                this[field] = this._prefs.getComplexValue(pref, CI.nsILocalFile);
            }
            catch (e)
            {
                this[field] = null;
            }
        }
        this._prefs.addObserver("tomtom.", this, true);
    },
observe: function(aSubject, aTopic, aData)
    {
        if (aTopic != "nsPref:changed")
        {
            return;
        }
        if (aData in this._prefToFieldMap)
        {
            var field = this._prefToFieldMap[aData];
            try
            {
                this[field] = this._prefs.getComplexValue(aData, CI.nsILocalFile);
            }
            catch(e)
            {
                this[field] = null;
            }
        }
    },

    _ensureFoldersExist : function()
    {
        try {
            for (var key in this._prefToDefaultMap)
            {
                this._tryAndSetFolderPref(key, this._prefToDefaultMap[key]);
            }
        }
        catch (e)
        {
            error(new ExceptionWithCustomMessage(
                    gApplication.settings.stringBundle.GetStringFromName("createItemFolderFailed"),
                    e));
        }
    },

    _tryAndSetFolderPref : function(prefName, pathAppend)
    {
        if (!this._prefs.prefHasUserValue(prefName)
            || ((gApplication.settings.isOSMacOSX() && this._prefs.getComplexValue(prefName, CI.nsILocalFile).path.indexOf(".Trash") >= 0)))
        {
            var dir = this._dataDir.clone();
            dir.append(pathAppend);

            if (gApplication.settings.isOSMacOSX())
            {
                ensureDirExists(dir);
            }

            this._prefs.setComplexValue(prefName, CI.nsILocalFile, dir);
        }
    },

    QueryInterface : function(iid)
    {
        if (iid.equals(CI.nsIObserver) ||
            iid.equals(CI.nsISupportsWeakReference) ||
            iid.equals(CI.nsISupports))
            return this;
        throw Components.results.NS_ERROR_NO_INTERFACE;
    }
}
GlobalSettings.kOSUnknown = 0;
GlobalSettings.kOSWindows = 1;
GlobalSettings.kOSMacOSX = 2;
GlobalSettings.kOSLinux = 3;
GlobalSettings.osServerNames = [undefined, "windows", "macintosh", "linux"];
