/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/


EXPORTED_SYMBOLS.push(
    "makeCallback",
    "debug",
    "unimportantError",
    "errorToText",
    "errorInternal",
    "dumpObject",
    "runAsync",
    "runAsyncDelayed",
    "memoize",
    "readURL",
    "readURLUTF8",
    "readChromePropertiesIntoArray",
    "assert",
    "extend",
    "extendMixIn",
    "ArrayContains",
    "ArrayRemoveFirst",
    "ArrayRemoveAllMatching",
    "IteratorToArray",
    "ArrayIterator",
    "versionNewerThan",
    "versionOlderThan",
    "buildNewerThan",
    "Map",
    "Exception",
    "DeviceAbsentException",
    "TaskPreconditionsUnmetException",
    "UnsupportedCapabilityException",
    "ExceptionWithCustomMessage",
    "UserCancelledException",
    "InformativeException",
    "BadValue",
    "Malformed",
    "InvalidValue",
    "InvalidArgument",
    "NotImplemented",
    "Bug",
    "NotReached",
    "ensureExceptionIsObject",
    "prefService",
    "getIOService",
    "consoleService",
    "promptService",
    "stringBundleService",
    "getStringBundle",
    "getMainWindowByType",
    "alert",
    "openDialogAsWindow",
    "ttMessageService",
    "MessageService",
    "addSubscriptionsRPCArgs",
    "getStaticURL",
    "putStringOnClipboard",
    "makeWrappableText",
    "getLanguageName",
    "DocumentWebProgressListener",
    "wrapMozillaException",
    "deviceSupportsCapabilitySet",
    "getLocalFilePathFromURL"
);
function makeCallback(obj, func)
{
    var oldDocument = null;
    if (func.__parent__ instanceof CI.nsIDOMWindow)
        oldDocument = func.__parent__.document;

    var prependParameters = Array.prototype.slice.call(arguments, 2);
    return function()
    {
        if (!oldDocument || func.__parent__.document == oldDocument)
        {
            Array.prototype.unshift.apply(arguments, prependParameters);
            return func.apply(obj, arguments);
        }
        else
        {
}
    }
}
function debug(message)
{
    dump(message + "\n");
    consoleService().logStringMessage(message);
}
function unimportantError(ex, context)
{
    var messageString = "";
    if (typeof context != "undefined")
        messageString += context + "\n";
    messageString += (typeof ex == "object" && "debugMessage" in ex ? ex.debugMessage : ex) + "\n" + _getStackText(ex);
    dump(messageString);

    var message = Cc["@mozilla.org/scripterror;1"].createInstance(CI.nsIScriptError);
    if (message instanceof CI.nsIException)
        message.init(messageString, ex.filename, "", ex.lineNumber, ex.columnNumber, CI.nsIScriptError.warningFlag, "home-unimportantError");
    else
        message.init(messageString, ex.fileName, "", ex.lineNumber, 0, CI.nsIScriptError.warningFlag, "home-unimportantError");
    consoleService().logMessage(message);
}
function errorToText(ex)
{
    if (ex instanceof CI.ttIError)
        return ex.wmessage;
    else if (typeof ex == "object" && "message" in ex)
        return ex.message;
    else
        return String(ex);
}
function error(ex)
{
    errorInternal(null, ex);
}
function errorInternal(parentWindow, ex)
{
    if (!ex)
    {
        try {
            (function () { throw new Exception("Unknown error."); })();
        } catch (e) { ex = e; }
    }

    _logError(ex, false);

    var dumptext = formatExceptionChain(ex);{}
try {
        if (ex.severity == Exception.SEVERITY_IGNORE)
            return;

        var message = errorToText(ex);
        if (ex instanceof ServerError &&
            (ex.code == Components.results.NS_ERROR_OFFLINE ||
             ex.code == Components.results.NS_ERROR_UNKNOWN_PROXY_HOST))
        {
            var stringBundle = getStringBundle("chrome://tthome/locale/logic/util.properties");
            if (ex.severity == Exception.SEVERITY_WARNING)
            {
                ttMessageService(parentWindow).alert(stringBundle.GetStringFromName("nonFatalNoInternet.title"),
                            stringBundle.GetStringFromName("nonFatalNoInternet.prompt"));
            }
            else
            {
                ttMessageService(parentWindow).alert(stringBundle.GetStringFromName("abortNoInternet.title"),
                            stringBundle.GetStringFromName("abortNoInternet.prompt"));
            }
        }
        else
        {
            openDialogAsWindow("chrome://tthome/content/ui/dialogs/error/errorPage.xul", {
                    message : message,
                    dumpText : dumptext,
                    exception : ex,
                    severity : ex.severity
                }, null, null, true, parentWindow);
        }
    } catch(e) {
        try {
unimportantError(e, "Failed to open error page/dialog for the above error because of ");
            alert(dumptext);
        } catch(e2) {
            dump("Failed to open error page/dialog for the above error because of " + e + ", but even alert()/unimportantError() failed because of " + e2 + "\n");
        }
    }
}
function formatExceptionChain(ex)
{
    var dumptext = "ERROR: ";

    do {
        var isEx = false;
        if (typeof(ex) == "object" && "name" in ex && "message" in ex)
        {
            isEx = true;
            try {
                ex = ex.QueryInterface(CI.ttIError);
            } catch (e) {
            }
        }

        dumptext += (typeof ex == "object" && "debugMessage" in ex ? ex.debugMessage : ex);
        if (isEx)
        {
            if ("filename" in ex && "lineNumber" in ex)
                dumptext += "\nin " + ex.filename + ":" + ex.lineNumber;
            else if ("fileName" in ex && "lineNumber" in ex)
                dumptext += "\nin " + ex.fileName + ":" + ex.lineNumber;
        }
        dumptext += "\n";
        dumptext += _getStackText(ex);

        if (ex.inner)
            dumptext += "\nCAUSE: ";

        ex = ex.inner;
    } while (ex);

    return dumptext;
}

function _getStackArrayFromStackFrame(location)
{
    var result = [];
    for (var frame = location; frame; frame = frame.caller)
    {
        var entry = {
            func : (frame.name == "anonymous" ? null : frame.name),
            filename : frame.filename,
            lineNumber : frame.lineNumber,
            sourceLine : frame.sourceLine
        }
        try {
            if (!entry.sourceLine && entry.filename && entry.lineNumber)
                    entry.sourceLine = _getSourceLine(entry.filename, entry.lineNumber);
        } catch (e) {
}
        result.push(entry);
    }
    return result;
}

function _getStackArrayFromStackString(stack)
{
    var result = [];
    var frames = stack.split("\n");
    for each (var line in frames)
    {
        if (!/^(.*?)?@(.*):(\d+)/.test(line))
            continue;
        var entry = {
            func : RegExp.$1 || null,
            filename : RegExp.$2 || null,
            lineNumber : parseInt(RegExp.$3) || null,
            sourceLine : null
        }
        try {
            if (entry.filename && entry.lineNumber)
                entry.sourceLine = _getSourceLine(entry.filename, entry.lineNumber);
        } catch (e) {
}
        result.push(entry);
    }
    return result;
}

function _getStackText(ex)
{
    var stackArray = null;
    try {
        if (ex.location)
        {
            stackArray = _getStackArrayFromStackFrame(ex.location);
        }
        else if (ex.stack)
        {
            stackArray = _getStackArrayFromStackString(ex.stack);
        }
        else
        {
            var errorFrame = Components.stack;
            while (errorFrame && errorFrame.name != "error" && errorFrame.name != "unimportantError")
                errorFrame = errorFrame.caller;

            var location = (errorFrame ? errorFrame.caller : Components.stack);
            stackArray = _getStackArrayFromStackFrame(location);
        }
    } catch (e) { dump("creating stack array failed: " + e + "\n"); }
    if (stackArray)
    {
        var stacktext = "Stack:\n";
        for (var i = 0; i < stackArray.length; i++)
        {
            var f = stackArray[i];
            stacktext += i + ".";
            if (f.func)
                stacktext += " function " + f.func + "  ";
            stacktext += " " + (f.filename ? f.filename : "unknown" );
            if (f.lineNumber)
                stacktext += ":" + f.lineNumber;
            stacktext += "\n";
            if (f.sourceLine)
                stacktext += "    " + f.sourceLine + "\n";
        }
        return stacktext;
    }
    else
        return "no stack\n";
}

function _getSourceLine(url, lineNumber)
{
    if (url.match(/ttwizard\.xml$/))
        lineNumber++;

    var uri = getIOService().newURI(url, null, null);
    var lines = readURL(uri);
    return lines[lineNumber - 1];
}

function _logError(ex, opt_warn)
{
    try {
        if (typeof(ex) == "object" && "lineNumber" in ex)
        {
            var consoleError = Cc["@mozilla.org/scripterror;1"]
                            .createInstance(CI.nsIScriptError);
            var flags = opt_warn ? 1 : 0;
            consoleError.init(errorToText(ex),
                            "filename" in ex ? ex.filename : null,
                            "sourceLine" in ex ? ex.sourceLine : null,
                            ex.lineNumber,
                            "columnNumber" in ex ? ex.columnNumber : null,
                            flags, null);
            consoleService().logMessage(consoleError);
        }
        else
        {
            Components.utils.reportError(ex);
        }
    } catch (ex2) {
        dump("Error '" + ex + "' could not be logged because of " + ex2 + "\n");
    }
}
function dumpObject(obj, name, maxDepth, curDepth)
{
    try {
        var result = "";

        if (curDepth == undefined)
          curDepth = 1;
        if (maxDepth != undefined && curDepth > maxDepth)
          return "";

        result += name + " = [" + typeof(obj) + "] ";
        if (obj == null)
        {
            result += " empty";
        }
        else if (typeof(obj) == "object" && typeof(obj.length) != "undefined")
        {
            result += "[probably array, length " + obj.length + "] ";
            for (var i = 0; i < obj.length; i++)
                 result += "\n" + dumpObject(obj[i], name + "." + i, maxDepth, curDepth+1);
            result += "\n";
        }
        else if (typeof(obj) == "object")
        {
            for (var prop in obj)
                result += "\n" + dumpObject(obj[prop], name + "." + prop, maxDepth, curDepth+1);
            result += "\n";
        }
        else if (typeof(obj) == "xml")
        {
            result += obj.toXMLString();
        }
        else
            result += obj;

        return result;
    }
    catch (e)
    {
        return "";
    }
}

var gRunTimerInternal = [];
function runAsync(func)
{
    runAsyncDelayed(func, 0);
}
function runAsyncDelayed(func, ms)
{
    var timer = Cc["@mozilla.org/timer;1"].createInstance(CI.nsITimer);
    timer.initWithCallback(function()
    {
        ArrayRemoveFirst(gRunTimerInternal, timer);
        func();
    }, ms, CI.nsITimer.TYPE_ONE_SHOT);
    gRunTimerInternal.push(timer);
}
function memoize(obj, prop, value)
{
    obj.__defineGetter__(prop, function() value);
    return value;
}
function readURL(uri)
{
    assert(uri instanceof CI.nsIURI, "url must be nsIURI");
    try {
        var sis = Cc["@mozilla.org/scriptableinputstream;1"]
            .createInstance(CI.nsIScriptableInputStream);
        var chan = getIOService().newChannelFromURI(uri);
        sis.init(chan.open());

        try {
            var content = "";
            var availBytes;
            while ((availBytes = sis.available()) > 0)
                content += sis.read(availBytes);
        } finally {
            sis.close();
        }

        content = content.replace("\r\n", "\n");
        content = content.replace("\r", "\n");

        return content.split("\n");
    } catch (e) {
        var stringBundle = getStringBundle("chrome://tthome/locale/logic/util.properties");
        throw new ExceptionWithCustomMessage(stringBundle.formatStringFromName("readURLException", [ uri.spec ], 1),
            wrapMozillaException(e));
    }
}
function readURLUTF8(uri)
{
    assert(uri instanceof CI.nsIURI, "uri must be nsIURI");
    try {
        var chan = getIOService().newChannelFromURI(uri);

        var charset = "UTF-8";
        const replacementChar = Components.interfaces.nsIConverterInputStream.DEFAULT_REPLACEMENT_CHARACTER;
        var is = Cc["@mozilla.org/intl/converter-input-stream;1"]
            .createInstance(CI.nsIConverterInputStream);
        is.init(chan.open(), charset, 1024, replacementChar);

        try {
            var str = {};
            var content = "";
            while (is.readString(4096, str) != 0)
            {
                content += str.value;
            }
        } finally {
            is.close();
        }

        content = content.replace("\r\n", "\n");
        content = content.replace("\r", "\n");

        return content.split("\n");
    } catch (e) {
        var stringBundle = getStringBundle("chrome://tthome/locale/logic/util.properties");
        throw new ExceptionWithCustomMessage(stringBundle.formatStringFromName("readURLUTF8Exception", [ uri.spec ], 1),
            wrapMozillaException(e));
    }
}
function readChromePropertiesIntoArray(chromeURL)
{
    var lines = null;

    var req = new XMLHttpRequest();
    req.overrideMimeType("text/plain");
    req.open("GET", chromeURL, false);
    req.send("");

    lines = req.responseText;
    lines = lines.replace("\r\n", "\n");
    lines = lines.replace("\r", "\n");

    return lines.split("\n");
}
function assert(test, errorStr, id)
{
    if (test)
        return;
    unimportantError(errorStr);
    debugger;
    throw new Bug(errorStr, "Internal/AssertFail" + (id ? "/" + id : ""));
}
function extend(child, supertype)
{
   child.prototype.__proto__ = supertype.prototype;
}
function extendMixIn(child, supertype)
{
    for (var property in supertype.prototype) {
        if (!child.prototype.__lookupGetter__(property) &&
            !child.prototype.__lookupSetter__(property) &&
            typeof child.prototype[property] == "undefined")
        {
            var getter = supertype.prototype.__lookupGetter__(property);
            var setter = supertype.prototype.__lookupSetter__(property);
            if (getter || setter)
            {
                if (getter)
                    child.prototype.__defineGetter__(property, getter);
                if (setter)
                    child.prototype.__defineSetter__(property, setter);
            }
            else
                child.prototype[property] = supertype.prototype[property];
        }
    }
    return child;
}
function ArrayContains(array, element)
{
    for (let i = 0; i < array.length; i++)
    {
        if (array[i] == element)
            return true;
    }
    return false;
}
function ArrayRemoveFirst(array, element)
{
    for (let i = 0; i < array.length; i++)
    {
        if (array[i] == element)
        {
            array.splice(i, 1);
            return true;
        }
    }
    return false;
}
function ArrayRemoveAllMatching(array, element)
{
    var found = 0;
    for (var i = 0; i < array.length; i++)
    {
        if (array[i] == element)
        {
            array.splice(i, 1);
            i--;
            found++;
        }
    }
    return found;
}
function IteratorToArray(iterator, iid)
{
    var result = new Array();
    while (iterator.hasMoreElements())
    {
        result.push(iterator.getNext().QueryInterface(iid));
    }
    return result;
}
function ArrayIterator(array, itemprocessor)
{
    this.array = array;
    this.i = 0;
    this.itemprocessor = itemprocessor;
}
ArrayIterator.prototype =
{
    hasMoreElements : function()
    {
        return this.i < this.array.length;
    },
    getNext : function()
    {
        if (this.i >= this.array.length)
            throw Components.results.NS_ERROR_UNEXPECTED;

        if (this.itemprocessor)
            return this.itemprocessor(this.array[this.i++]);
        else
            return this.array[this.i++];
    },
    QueryInterface : function(iid)
    {
        if (iid.equals(CI.nsISimpleEnumerator) ||
            iid.equals(CI.nsISupports))
            return this;
        throw Components.results.NS_NOINTERFACE;
    }
}
function versionOlderThan(ver1, ver2)
{
    return versionNewerThan(ver2, ver1);
}
function versionNewerThan(ver1, ver2)
{
    if (ver1 == ver2)
        return false;
    var a1 = ver1.split(".");
    var a2 = ver2.split(".");
    for (var i = 0; i < a1.length && i < a2.length; i++)
    {
        var c1 = makeSafe.integer(a1[i]);
        var c2 = makeSafe.integer(a2[i]);
        if (c1 == c2)
            continue;
        else
            return c1 > c2;
    }
    return a1.length > a2.length;
}
function buildNewerThan(bn1, bn2)
{
    bn1 = bn1.replace(/\..*/, '');
    bn2 = bn2.replace(/\..*/, '');
    return makeSafe.integer(bn1) > makeSafe.integer(bn2);
}
function Map()
{
    this._hashMap = {};
}
Map.prototype =
{
    _hashMap : null,
    addValue : function (key, value)
    {
        this._hashMap[this._getSafeKey(key)] = value;
    },
    getValue : function (key)
    {
        var safeKey = this._getSafeKey(key);
        if (this._hashMap.hasOwnProperty(safeKey))
            return this._hashMap[safeKey];
        else
            return null;
    },
    hasValue : function (key)
    {
        return this._hashMap.hasOwnProperty(this._getSafeKey(key));
    },
    values : function ()
    {
        var values = [];
        for each (var value in this._hashMap)
            values.push(value);
        return values;
    },
iterateOverAll : function (func)
    {
        for (var key in this._hashMap)
            func(this._getOriginalKey(key), this._hashMap[key]);
    },
    _getSafeKey : function (key)
    {
        return "___" + key;
    },
    _getOriginalKey : function (safeKey)
    {
        return safeKey.replace("___", "");
    }
};
function Exception(message, id, userMessageParams)
{
    if (id == null)
    {
this._message = message;
        this._id = "";
    }
    else
    {
        if (!Exception._userErrorStringBundle)
            Exception._userErrorStringBundle = getStringBundle("chrome://tthome/locale/usererror.properties");
        var messageId = id.toString().replace(/\/?#.*/, "");
        if (messageId != "")
        {
            if (!userMessageParams)
                userMessageParams = [];

            function messageForID(id, userMessageParams)
            {
                try {
                    return Exception._userErrorStringBundle.formatStringFromName(id,
                        userMessageParams, userMessageParams.length);
                } catch (e) {
return null;
                }
            }

            this._message = messageForID(messageId, userMessageParams);
            if (!this._message)
                this._message = messageForID(this._id, userMessageParams);
        }
        else
            this._message = "";

        this._id = id;
    }
    this.name = "Exception";
    this.location = Components.stack;
    var proto = this.__proto__;
    if (proto)
        proto = proto.__proto__;
    while (proto)
    {
        try {
            this.location = this.location.caller;
        } catch (e) {
            delete this.location;
            break;
        }
        proto = proto.__proto__;
    }
}
Exception.SEVERITY_BASE      = 1100;
Exception.SEVERITY_IGNORE     = Exception.SEVERITY_BASE + 1;
Exception.SEVERITY_INFO       = Exception.SEVERITY_BASE + 2;
Exception.SEVERITY_WARNING    = Exception.SEVERITY_BASE + 3;
Exception.SEVERITY_RESUMABLE  = Exception.SEVERITY_BASE + 4;
Exception.SEVERITY_FATAL      = Exception.SEVERITY_BASE + 5;

Exception.prototype =
{
severity: Exception.SEVERITY_FATAL,
allowReportToServer : true,
_inner: null,

    _debugInfo: null,

    set inner(ex)
    {
        if (this._inner)
            throw new Exception("Inner exception already set. (" + this._inner + ")");
        this._inner = ex;
    },

    get inner()
    {
        return this._inner;
    },
withCause: function(ex)
    {
        this.inner = ex;
        return this;
    },
get debugInfo()
    {
        return this._debugInfo;
    },
    set debugInfo(debugInfo)
    {
        this._debugInfo = debugInfo;
        return this.debugInfo;
    },
    toString : function()
    {
        return this.debugMessage;
    },
    get debugMessage()
    {
        var msg = (this.id ? this.id + ": " : "");
        msg += this.name + ": ";
        if (this._message)
            msg += this._message;
        if (this.debugInfo)
            msg += "\n" + this.debugInfo;
        msg += "\nSeverity: " + this.severity;
        if (this._inner)
        {
            msg += "\n-\n";
            if (this._inner.debugMessage)
                msg += this._inner.debugMessage;
            else
                msg += this._inner;
        }
        return msg;
    },
    get message()
    {
        return this._message;
    },
    set message(message)
    {
        this._message = message;
        return this.message;
    },
get id()
    {
        if (this._inner && this._inner.id)
        {
            return this._id + "/" + this._inner.id;
        }
        return this._id;
    },
    QueryInterface : function(iid)
    {
        if (iid.equals(CI.ttIError))
            return this;
        throw Components.results.NS_ERROR_NO_INTERFACE;
    }
}
Exception._userErrorStringBundle = null;
Exception.downgradeSeverity = function(current, requested)
{
    if (typeof(current) == "undefined" || current == null)
        return requested;
    if (current > requested)
        return requested;
    return current;
}
Exception.renameException = function (oldException, id, userMessageParams)
{
    var ex = new Exception(null, id, userMessageParams);
    ex.inner = oldException;
    return ex;
}
function ExceptionWithCustomMessage(message, e)
{
    Exception.call(this, message);
    this.name = "ExceptionWithCustomMessage";
    this._inner = e;
}
ExceptionWithCustomMessage.prototype =
{
    get message()
    {
        var innerMsg = errorToText(this._inner);
        var sb = getStringBundle("chrome://tthome/locale/logic/util.properties");
        return sb.formatStringFromName("ExceptionWithCustomMessageToString",
            [this._message, innerMsg], 2);
    },
    set message(message)
    {
        this._message = message;
        return this.message;
    }
};
extend(ExceptionWithCustomMessage, Exception);
function UserCancelledException()
{
    Exception.call(this, null, this._id);
    this.name = "User Cancelled";
    this.code = kErrorUserCancelled;
}
UserCancelledException.prototype =
{
    _id: "User/Cancelled",
    severity: Exception.SEVERITY_IGNORE,
    allowReportToServer: false
};
extend(UserCancelledException, Exception);
function DeviceAbsentException()
{
    Exception.call(this, null, this._id);
    this.name = "DeviceAbsentException";
}
DeviceAbsentException.prototype =
{
    _id: "User/DeviceAbsent",
    severity: Exception.SEVERITY_FATAL,
    allowReportToServer: false
};
extend(DeviceAbsentException, Exception);


function TaskPreconditionsUnmetException(message, id, userMessageParams)
{
    Exception.call(this, message, (id ? id : this._id), userMessageParams);
    this.name = "TaskPreconditionsUnmetException";
}
TaskPreconditionsUnmetException.prototype = {
    _id: "User/TaskPreconditionsUnmet",
    severity: Exception.SEVERITY_FATAL,
    allowReportToServer: false
};
extend(TaskPreconditionsUnmetException, Exception);


function UnsupportedCapabilityException(message, id, userMessageParams)
{
    TaskPreconditionsUnmetException.call(this, message, id, userMessageParams);
    this.name = "UnsupportedCapabilityException";
}
UnsupportedCapabilityException.prototype = {
    _id: "User/UnsupportedCapability",
    severity: Exception.SEVERITY_FATAL,
    allowReportToServer: false
};
extend(UnsupportedCapabilityException, TaskPreconditionsUnmetException);

function InformativeException(message, id, userMessageParams)
{
    Exception.call(this, message, id, userMessageParams);
    this.name = "InformativeException";
}
InformativeException.prototype =
{
    severity: Exception.SEVERITY_INFO,
    allowReportToServer: false
};
extend(InformativeException, Exception);
function BadValue(message, badValue, id, userMessageParams)
{
    Exception.call(this, null, (id ? id : this._id), userMessageParams);
    this.name = "BadValue";
    this.badValue = badValue;
    this.debugInfo = message + "\nBad value: \"" + badValue + "\" (type: " + typeof this.badValue + ")";
}
BadValue.prototype =
{
    _id: "Internal/BadValue",
    severity: Exception.SEVERITY_FATAL
};
extend(BadValue, Exception);
function Malformed(message, badValue, id, userMessageParams)
{
    BadValue.call(this, message, badValue, id, userMessageParams);
    this.name = "Malformed";
}
Malformed.prototype =
{
    _id: "Internal/Malformed",
    severity: Exception.SEVERITY_FATAL
};
extend(Malformed, BadValue);
function InvalidValue(message, badValue, id, userMessageParams)
{
    BadValue.call(this, message, badValue, id, userMessageParams);
    this.name = "Invalid value";
}
InvalidValue.prototype =
{
    _id: "Internal/InvalidValue",
    severity: Exception.SEVERITY_FATAL
};
extend(InvalidValue, BadValue);
function InvalidArgument(message, badValue, id, userMessageParams)
{
    BadValue.call(this, message, badValue, id, userMessageParams);
    this.name = "Invalid argument";
}
InvalidArgument.prototype =
{
    _id: "Internal/InvalidArgument",
    severity: Exception.SEVERITY_FATAL
};
extend(InvalidArgument, BadValue);
function NotImplemented(message, id, userMessageParams)
{
    Exception.call(this, null, message, (id ? id : this._id), userMessageParams);
    this.name = "Not implemented";
    this.debugInfo = message;
}
NotImplemented.prototype =
{
    _id: "Internal/NotImplemented",
    severity: Exception.SEVERITY_FATAL
};
extend(NotImplemented, Exception);
function Bug(message, id, userMessageParams)
{
    Exception.call(this, null, (id ? id : this._id), userMessageParams);
    this.name = "Code bug";
    this.debugInfo = message;
}
Bug.prototype =
{
    _id: "Internal/Bug",
    severity: Exception.SEVERITY_FATAL
};
extend(Bug, Exception);
function NotReached(message, id, userMessageParams)
{
    Exception.call(this, null, (id ? id : this._id), userMessageParams);
    this.name = "Not reached";
    this.debugInfo = message;
}
NotReached.prototype =
{
    _id: "Internal/NotReached",
    severity: Exception.SEVERITY_FATAL
};
extend(NotReached, Bug);
function ensureExceptionIsObject(e)
{
    if (e instanceof CI.nsIException)
        return { __proto__: e };
    else
        return e;
}

var gPrefServiceCache;
function prefService()
{
    if (gPrefServiceCache)
        return gPrefServiceCache;
    gPrefServiceCache = Cc["@mozilla.org/preferences-service;1"]
                        .getService(CI.nsIPrefService)
                        .getBranch(null);
    return gPrefServiceCache;
}

var gIOService;
function getIOService()
{
    if (gIOService)
        return gIOService;
    gIOService = Cc["@mozilla.org/network/io-service;1"]
                    .getService(CI.nsIIOService);
    return gIOService;
}

var gConsoleServiceCache;
function consoleService()
{
    if (gConsoleServiceCache)
        return gConsoleServiceCache;
    gConsoleServiceCache = Cc["@mozilla.org/consoleservice;1"]
                           .getService(CI.nsIConsoleService);
    return gConsoleServiceCache;
}

function promptService()
{
    return Cc["@mozilla.org/embedcomp/prompt-service;1"]
            .getService(CI.nsIPromptService);
}

var gStringBundleServiceCache;
function stringBundleService()
{
    if (gStringBundleServiceCache)
        return gStringBundleServiceCache;

    gStringBundleServiceCache = Cc["@mozilla.org/intl/stringbundle;1"]
                                    .getService(CI.nsIStringBundleService);

    return gStringBundleServiceCache;
}

function getStringBundle(aStringBundleURI)
{
    try {
        return stringBundleService().createBundle(aStringBundleURI);
    } catch (exception) {
        throw new Exception("Failed to get stringbundle URI <" + aStringBundleURI + ">. Error: " + exception);
    }
}
function getMainWindowByType()
{
    var windowManager = Cc['@mozilla.org/appshell/window-mediator;1'].
            getService(CI.nsIWindowMediator);
    var windows = windowManager.getEnumerator("tthome:main");
    var mainWindow = null;
    if (windows.hasMoreElements())
    {
        var window = windows.getNext().QueryInterface(CI.nsIDOMWindow);
        if (window && !window.closed)
            mainWindow = window;
    }
    return mainWindow;
}

function alert(text)
{
    var appname = "TomTom HOME";
    try {
        appname = getStringBundle("chrome://branding/locale/brand.properties")
                    .GetStringFromName("brandFullName");
    } catch (e) {}
    return promptService().alert(null, appname, text);
}
function openDialog(parentWindow, url, windowName, features)
{
    if (!gApplication || !gApplication.settings || gApplication.settings.isOSMacOSX())
        parentWindow = null;
    else  if (!parentWindow)
    {
        var mediator = Cc["@mozilla.org/appshell/window-mediator;1"].getService(CI.nsIWindowMediator);
        var enumerator = mediator.getZOrderDOMWindowEnumerator(null, true);

        if (enumerator.hasMoreElements())
            parentWindow = enumerator.getNext().QueryInterface(CI.nsIDOMWindow);

        if (!parentWindow)
            unimportantError(new Exception("Opening dialog " + url + " without a parent window"));
        else
        {
            try
            {
                parentWindow = parentWindow.top;
                parentWindow.focus();
            }
            catch (e) {
                unimportantError(e);
            }
        }
    }

    var array = Cc["@mozilla.org/array;1"].createInstance(CI.nsIMutableArray);

    for (var i = 4; i < arguments.length; i++)
    {
        var variant = Cc["@mozilla.org/variant;1"].createInstance(CI.nsIWritableVariant);
        variant.setFromVariant(arguments[i]);
        array.appendElement(variant, false);
    }

    var watcher = Cc["@mozilla.org/embedcomp/window-watcher;1"].getService(CI.nsIWindowWatcher);
    return watcher.openWindow(parentWindow, url, windowName, features, array);
}
var openDialogs = {};
openDialogs.__proto__ = null;
function openDialogAsWindow(url, params, loadCallback, unloadCallback, allowDupe, parentWindow)
{
    if (!allowDupe && url in openDialogs)
        return;

    openDialogs[url] = true;
    var handle = { _window : null };
    var result = { value : null };

    try {
		openDialog(parentWindow, url, "_blank", "modal,centerscreen,close=yes", params, result, handle, loadCallback);
    } finally {
        delete openDialogs[url];
    }
try {
        if (unloadCallback)
            unloadCallback(result.value);
    } catch (e) {
        error(e);
    }
    return result.value;
}
function ttMessageService(parentWindow)
{
    return new MessageService(parentWindow);
}
function MessageService(parentWindow)
{
    this._parentWindow = parentWindow;
}
MessageService.prototype =
{
    _parentWindow : null,

    _openDialog : function (params, confirmFunction, cancelFunction, middleFunction)
    {
        return openDialogAsWindow(
            "chrome://tthome/content/ui/dialogs/message/message.xul",
            params,
            null,
            function(result)
            {
                if (result == null || result == false)
                {
                    if (cancelFunction)
                        cancelFunction();
                }
                else if(result == 'middle')
                {
                    if (middleFunction)
                        middleFunction();
                }
                else
                {
                    if (confirmFunction)
                        confirmFunction(result);
                }
            },
            true,
            this._parentWindow);
    },
alert : function (title, msg, severity)
    {
        return this._openDialog(
        {
            pageType : "alert",
            title : title,
            msg : msg,
            showOK : true,
            showCancel : false,
            severity : severity
        },
        null, null);
    },
confirm : function (title, msg, cancelLabel, okLabel, confirmFunction, cancelFunction, severity, customID, urlid, urllabel)
    {
        return this._openDialog(
        {
            pageType : "confirm",
            title : title,
            msg : msg,
            urlid : urlid,
            urllabel : urllabel,
            showOK : true,
            showCancel : true,
            labelOK : okLabel,
            labelCancel : cancelLabel,
            severity : severity,
            customID : customID
        },
        confirmFunction,
        cancelFunction);
    },
confirmEx : function (title, msg, cancelLabel, okLabel, middleLabel, confirmFunction, cancelFunction, middleFunction, severity, customID, urlid, urllabel)
    {
        return this._openDialog(
        {
            pageType : "confirm",
            title : title,
            msg : msg,
            urlid : urlid,
            urllabel : urllabel,
            showOK : true,
            showCancel : true,
            showMiddle : true,
            labelMiddle : middleLabel,
            labelOK : okLabel,
            labelCancel : cancelLabel,
            severity : severity,
            customID : customID
        },
        confirmFunction,
        cancelFunction,
        middleFunction
        );
    },
prompt : function (title, msg, defaultValue, cancelLabel, okLabel, confirmFunction, cancelFunction, severity)
    {
        return this._openDialog(
        {
            pageType : "prompt",
            title : title,
            msg : msg,
            showOK : true,
            showCancel : true,
            labelOK : okLabel,
            labelCancel : cancelLabel,
            defaultValue : defaultValue,
            severity : severity
        },
        confirmFunction,
        cancelFunction);
    }
};

MessageService.SEVERITY_INFORMATIVE = 0;
MessageService.SEVERITY_WARNING = 1;
function dateToISO8601(aDate) {
    var sb = getStringBundle("data:text/plain,format=" + encodeURIComponent("%04S-%02S-%02ST%02S:%02S:%02SZ"));
    var data = [aDate.getUTCFullYear(), aDate.getUTCMonth() + 1, aDate.getUTCDate(),
                aDate.getUTCHours(), aDate.getUTCMinutes(), aDate.getUTCSeconds()];
    return sb.formatStringFromName("format", data, data.length);
}
function addSubscriptionsRPCArgs(args)
{
    var dev = gApplication.deviceManager.activeDevice;
    if ( ! dev)
        return;

    var gprsinfo = dev.gprsInfo;
    if (gprsinfo)
    {
        args.pragueSerial = gprsinfo.imei;
        args.imsiPrefix = gprsinfo.imsi;
        args.mnc = gprsinfo.mnc;
        args.gprsModel = gprsinfo.model;
        args.iccid = gprsinfo.iccid;
    }

    var rdstmc = dev.rdstmcSubscription;
    if (rdstmc)
    {
        args.rdstmcDeviceID = rdstmc.receiverID;
        args.rdstmcLastConnect = dateToISO8601(new Date(rdstmc.lastConnect));
        args.rdstmcExpires = dateToISO8601(new Date(rdstmc.expires));
    }
}
function getStaticURL(urlid, successCallback, errorCallback, additionalParameters)
{
    var locale = gApplication.settings.uiLocale;
    var country = gApplication.settings.country;
    var dev = gApplication.deviceManager ? gApplication.deviceManager.lastDevice : null;

    var paramString = "";
    if (additionalParameters)
    {
        for (var param in additionalParameters)
            paramString += "&" + encodeURIComponent(param) + "=" + encodeURIComponent(additionalParameters[param]);
    }

    rpc.init(function()
    {
        var redirectUrl = rpc.serviceBaseURL("urlRedirect");
        var url = redirectUrl.spec + "/" + urlid +
                  "?locale=" + encodeURIComponent(locale) +
                  "&country=" + encodeURIComponent(country) + paramString;
        if (dev)
            url += "&deviceModel=" + encodeURIComponent(dev.model.id);
        if (successCallback)
            successCallback(url);
    },
    function(e)
    {
        if (errorCallback)
            errorCallback(e);
    });
}
function putStringOnClipboard(aString)
{
    assert(aString, "aString must not be null or empty");

    var transferable = Cc["@mozilla.org/widget/transferable;1"].createInstance(CI.nsITransferable);
    var dataString = Cc["@mozilla.org/supports-string;1"].createInstance(CI.nsISupportsString);
    var clipboard = Cc["@mozilla.org/widget/clipboard;1"].getService(CI.nsIClipboard);

    dataString.data = aString;
    transferable.setTransferData("text/unicode", dataString, aString.length * 2);
    clipboard.setData(transferable, null, CI.nsIClipboard.kGlobalClipboard);
}
function makeWrappableText(aString)
{
    return aString.replace(/\S{30}(?=\S)/g, "$& ");
}

var gLanguageNameCache = new Map();
var gLanguageNameSB = null;
function getLanguageName(aLanguageTag)
{
    if (gLanguageNameCache.hasValue(aLanguageTag))
        return gLanguageNameCache.getValue(aLanguageTag);

    if (!gLanguageNameSB)
        gLanguageNameSB = getStringBundle("chrome://tthome/locale/ttlocalelist/languages.properties");

    try {
        gLanguageNameCache.addValue(aLanguageTag, gLanguageNameSB.GetStringFromName(aLanguageTag));
    }
    catch (e) {
        try {
            gLanguageNameCache.addValue(aLanguageTag, gLanguageNameSB.GetStringFromName(aLanguageTag.split("-")[0]));
        }
        catch (e) {
            unimportantError(e, "Could not find name for language " + aLanguageTag + ": ");
            gLanguageNameCache.addValue(aLanguageTag, aLanguageTag);
        }
    }

    return gLanguageNameCache.getValue(aLanguageTag);
}
function DocumentWebProgressListener(aFrame)
{
    assert(aFrame, "aFrame must not be null");
    this._frame = aFrame;
    this._frame.docShell
        .QueryInterface(CI.nsIInterfaceRequestor).getInterface(CI.nsIWebProgress)
        .addProgressListener(this, CI.nsIWebProgress.NOTIFY_ALL);
}
DocumentWebProgressListener.prototype =
{
    _frame : null,
    onProgressChange : function (wp, req, cur, max, curtotal, maxtotal) {},
onStateChange : function (wp, req, state, status)
    {
        if (!req)
            return;

        if ((state & CI.nsIWebProgressListener.STATE_IS_WINDOW) &&
            (state & CI.nsIWebProgressListener.STATE_STOP))
        {
this.loaded();
        }
    },
    onLocationChange : function (wp, req, loc) {},
    onStatusChange : function (wp, req, status, message) {},
    onSecurityChange : function (wp, req, state) {},
    QueryInterface : function(aIID)
    {
        if (aIID.equals(CI.nsIWebProgressListener) ||
            aIID.equals(CI.nsISupportsWeakReference) ||
            aIID.equals(CI.nsISupports)) {
            return this;
        }
        throw Components.results.NS_NOINTERFACE;
    },
destructor : function ()
    {
        this._frame.docShell
            .QueryInterface(CI.nsIInterfaceRequestor).getInterface(CI.nsIWebProgress)
            .removeProgressListener(this);
    },
loaded : function ()
    {
        throw new NotReached("override this method in the sub type");
    }
}

function deviceSupportsCapabilitySet(dev, capabilitySet)
{
    var supported = false;

    for each (var capabilitySet in capabilitySet.split(","))
    {
        supported = capabilitySet
            .split("+")
            .every(function(c) dev.hasCapability(c));

        if (supported)
            return true;
    }

    return false;
}
function getLocalFilePathFromURL(url)
{

    if (url instanceof CI.nsIURI && url.scheme == "file")
    {
        let fileHandler = Cc["@mozilla.org/network/protocol;1?name=file"].getService(CI.nsIFileProtocolHandler);
        url = fileHandler.getFileFromURLSpec(url.spec);
    }
    if (url instanceof CI.nsIFile)
    {
        return url.path;
    }
    return null;
}
