/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/



var gQuickAccessCategories = {};
var gMapShare = null;
var gMaps = null;
var gDeviceSize = 0;

var gRemCorrsDeviceObserver =
{
    changed : function(deviceChanged, locationChanged)
    {
        dialog.close();
    }
};
function NYPrefsDialog()
{
    ttDialog.call(this, "footer");
}
NYPrefsDialog.prototype =
{
    onLoad: function(params)
    {
        init();
    },

    saveAndClose : function ()
    {
        if (gApplication.deviceManager.activeDevice &&
            gApplication.deviceManager.activeDevice.mapShare)
        {
            gMapShare.savePOISubscriptionsToDevice();
        }
        updateMaps(gMaps);
    },

    close : function()
    {
        gApplication.deviceManager.removeObserver(gRemCorrsDeviceObserver);
        ttDialog.prototype.close.apply(this, arguments);
    },

    __end : null
}
extend(NYPrefsDialog, ttDialog);
var dialog = new NYPrefsDialog();

function init()
{
    try
    {
        gMapShare = gApplication.deviceManager.activeDevice.mapShare.QueryInterface(CI.ttIMapShare);
        gMaps = new Array();

        if (!gApplication.deviceManager.activeDevice || !gApplication.deviceManager.activeDevice.mapShare)
        {
            E("mapshare-not-supported").collapsed = false;
            return;
        }
        else
        {
            E("mapshare-supported").collapsed = false;
        }

        var activeMap = gApplication.deviceManager.activeDevice.activeMap;
        if (activeMap)
        {
            E("trustslider").addEventListener("select", function() {
                activeMap.mapShareLocalCache.currentTrustLevel = E("trustslider").trustLevel;
                updateTotalSize();
            }, false);
        }
        gApplication.homeServices.setMapShareLocalCacheFolder(
            gApplication.settings.mapShareCacheDir, new ScanListener(function() {}, function()
        {
            if (activeMap)
            {
                E("trustslider").setAttribute("disabled", "false");
                E("trustslider").trustLevel = activeMap.mapShareLocalCache.currentTrustLevel;
            }
            var devItems = gApplication.deviceManager.activeInstallLocation.deviceNavItems();
            devItems.populate(new ScanListener(function(){},function()
            {
                var iterator = devItems.iterator;
                while (iterator.hasMoreElements())
                {
                    var item = iterator.getNext().QueryInterface(CI.ttIItem);
                    if(item.itemType == item.kItemMap)
                    {
                        gMaps.push(item.QueryInterface(CI.ttIMap));
                    }
                }

                loadMapPatchInfoFromDLL();
                loadPOICategoriesFromDLL();
                updateTotalSize();
            },
            error
            ));
        },
        error
        ));
   } catch (e) { error(e); }
}

function selectPatch(item)
{
    item.countryInfo.installPatches = true;
    updateCategorySize(item.parentNode);
}

function unselectPatch(item)
{
    item.countryInfo.installPatches = false;
    updateCategorySize(item.parentNode);
}

function updateCategorySize(category)
{
    category.sizeNode.value = ttItem.sizeBytesToMB(category.map.mapShareLocalCache.getPatchFilesSizeInBytes());
    updateTotalSize ();
}

function updateTotalSize()
{
    var size = 0;
    for each (var map in gMaps)
    {
        size += map.mapShareLocalCache.getPatchFilesSizeInBytes();
    }

    E('trustafterupdate').value = E('afterupdate').value = ttItem.sizeBytesToMB(size);
    var sizeDelta = size - gDeviceSize;
    var doneButton = E("footer").getRightButton();
    doneButton.disabled = (sizeDelta > gApplication.deviceManager.activeInstallLocation.sizeInBytesFree);
}

function getRegionString(countryInfo)
{
    var countriesStringBundle = E("countriesStringBundle");
    var usStringBundle = E("usStringBundle");
    var caStringBundle = E("caStringBundle");

    var countryCode = countryInfo.countryCode;
    var regionString = "";
    if (countryCode == "ZZ")
    {
        countryCode = "XXX";
    }

    try {
        if (countryCode == "US")
        {
            regionString = usStringBundle.getString(countryInfo.subCountryCode);
        }
        else if (countryCode == "CA" && countryInfo.subCountryCode)
        {
            regionString = caStringBundle.getString(countryInfo.subCountryCode);
        }
        else
        {
            regionString = countriesStringBundle.getString(countryCode);
        }
    }
    catch (e)
    {
        regionString = countryInfo.subCountryCode || countryCode;
    }

    return regionString;
}

function loadMapPatchInfoFromDLL()
{
    var listbox = E("mappatch-list");
    var mapShareStringBundle = E("mapShareStringBundle");
    var deviceSizeLabel = E("ondevice");
    gDeviceSize = 0;

    listbox.addColumn("name", mapShareStringBundle.getString("column.name"), 6);
    listbox.addColumn("size", mapShareStringBundle.getString("column.size"), 1);
    listbox.addColumn("select", mapShareStringBundle.getString("select"), 2);

    for each (var map in gMaps)
    {
        var category = undefined;

        gDeviceSize += map.mapPatchSizeInBytes;
        var countryInfoList = IteratorToArray(map.mapShareLocalCache.mapPatchCountryInfoList, CI.ttILocalCountryMapPatchInfo);
        for each (var countryInfo in countryInfoList)
        {
            if (typeof category == "undefined")
            {
                category = document.createElementNS(kNS_XUL, "ttlistboxcategory");
                listbox.addItem(category);
                category.label = map.name;
                category.map = map;
                category.sizeNode = category.addTextCell(ttItem.sizeBytesToMB(map.mapShareLocalCache.getPatchFilesSizeInBytes()));
                category.addSelectionButton();
            }

            var item = document.createElementNS(kNS_XUL, "ttlistboxitem");
            item.countryInfo = countryInfo;
            category.addItem(item);

            item.addTextCell(getRegionString(countryInfo));
            item.addTextCell(ttItem.sizeBytesToMB(countryInfo.getEstimatedSizeInBytes(CI.ttIMapShareLocalCache.kTrustLevelAll)));
            item.addToggleButton("", "", selectPatch, unselectPatch);

            item.selected = countryInfo.installPatches;
        }
    }

    listbox.doneLoading = true;
    deviceSizeLabel.value = E("trustondevice").value = ttItem.sizeBytesToMB(gDeviceSize);
    updateTotalSize();
}

function loadPOICategoriesFromDLL()
{
    var catStringBundle = E("catStringBundle");

    gApplication.deviceManager.addObserver(gRemCorrsDeviceObserver);

    var subscriptionsArray = IteratorToArray(gMapShare.mapSharePOISubscriptions,
                                              CI.ttIMapSharePOISubscription);

    var columnOne = E("column-1");
    var categoryLength = subscriptionsArray.length;
    var colLength = Math.ceil(categoryLength / 3);

    var localSubscriptionsArray = new Array();
    for (var i = 0 ; i < subscriptionsArray.length ; i++)
    {
        localSubscriptionsArray[i] =
            {
                _poiSubsObject : subscriptionsArray[i],
                category : subscriptionsArray[i].category,
                name : catStringBundle.getString(subscriptionsArray[i].category),
                get subscribed()
                {
                    return this._poiSubsObject.subscribed;
                },
                set subscribed(aValue)
                {
                    this._poiSubsObject.subscribed = aValue;
                }
            };
    }
    localSubscriptionsArray = localSubscriptionsArray.sort(comparePOIStrings);

    for (var c = 1; c <= 3; c++)
    {
        var container = E("column-" + c);
        for (var i = ( c - 1 ) * colLength; i < subscriptionsArray.length && i < c*colLength; i++)
        {
            var cat = localSubscriptionsArray[i];
            var subscribed = localSubscriptionsArray[i].subscribed;
            gQuickAccessCategories[cat.category] = cat;
            var checkbox = document.createElement("checkbox");
            checkbox.setAttribute("checked", subscribed);
            checkbox.setAttribute("label", localSubscriptionsArray[i].name);
            checkbox.setAttribute("category", cat.category);

            checkbox.addEventListener("command", function()
            {
                setSubscription(this);
            },
            false);

            container.appendChild(checkbox);
        }
    }
}

function comparePOIStrings (poi1, poi2)
{
    var catStringBundle = E("catStringBundle");
    var collation = gApplication.settings.uiCollation;
    try
    {
        return collation.compareString(collation.kCollationStrengthDefault, poi1.name, poi2.name);
    }
    catch (e)
    {
        if (poi1.name < poi2.name)
            return -1;
        else if (poi1.name > poi2.name)
            return 1;
        else
            return 0;
    }
}

function setSubscription(checkbox)
{
    gQuickAccessCategories[checkbox.getAttribute("category")].subscribed = checkbox.checked;
}
function setBusy(busy)
{
    var backBtn = E("footer").getLeftButton();
    var nextBtn = E("footer").getRightButton();

    if (busy)
    {
        window.setCursor("wait");
        backBtn.disabled = true;
        nextBtn.setAttribute("busy", "true");
        nextBtn.disabled = true;
    }
    else
    {
        window.setCursor("auto");
        backBtn.disabled = false;
        nextBtn.removeAttribute("busy");
        updateTotalSize();
    }
}
function updateMaps(maps, index)
{
    if (typeof index == "undefined")
    {
        index = 0;
        setBusy(true);
    }

    if (index >= maps.length)
    {
        setBusy(false);
        dialog.close();
        return;
    }

    var map = maps[index];

    map.mapShareLocalCache.saveCountryInfoToDevice();
    var installer = map.createMapPatchesInstaller();
    installer.doInstall(new CopyListener(
        function(percent, text) {},
        function()
        {
            updateMaps(maps, index + 1);
        },
        function(e)
        {
            setBusy(false);
            error(e);
        }
    ));
}
