/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/


var gStringBundle = null;
var gDevice = null;

const XULNS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

var observer =
{
    added : function(item)
    {
        repopulate();
    },
    removed : function(item)
    {
        repopulate();
    },
    repopulated : function()
    {
        repopulate();
    },
    QueryInterface : function(iid)
    {
        if (iid.equals(CI.ttICollectionObserver) ||
            iid.equals(CI.nsISupportsWeakReference) ||
            iid.equals(CI.nsISupports))
            return this;
        throw Components.results.NS_ERROR_NO_INTERFACE;
    }
}
function DeviceSelectionDialog()
{
    ttDialog.call(this, "footer");
}
DeviceSelectionDialog.prototype =
{
    _resultCallback : null,
    _closing : false,

    onLoad: function(params)
    {
        if ("installLocOfDevice" in params)
        {
            gDevice = params.installLocOfDevice;
        }
        if ("resultCallback" in params)
        {
            this._resultCallback = params.resultCallback;
        }
        gStringBundle = E("devicesel-bundle");

        var listbox = E("item-listbox");
        listbox.addColumn("location", gStringBundle.getString("locationCol"), "50" );
        listbox.addColumn("path", gStringBundle.getString("pathCol"), "15" );
        listbox.addColumn("space", gStringBundle.getString("spaceCol"), "20" );
        listbox.addColumn("button", "", "25");

        repopulate();
        gApplication.homeServices.devices().addObserver(observer);
    },
    allowClose : function()
    {
        return gApplication.deviceManager.activeDevice ? true : false;
    },

    close : function(result)
    {
        if (this._closing)
            return;

        this._closing = true;

        if (this._resultCallback)
            this._resultCallback(result);
        gApplication.homeServices.devices().removeObserver(observer);
        ttDialog.prototype.close.apply(this, arguments);
    },

    __end : null
}
extend(DeviceSelectionDialog, ttDialog);
var dialog = new DeviceSelectionDialog();


function repopulate()
{
    try
    {
        E("footer").setAttribute("leftdisabled", ! dialog.allowClose());

        var listbox = E("item-listbox");
        listbox.clear();

        var devices = gApplication.deviceManager.getAllDevices();
        if (gDevice)
        {
            devices = ArrayContains(devices, gDevice) ? [gDevice] : [];
        }
        if (devices.length == 0)
        {
            dialog.cancel();
            return;
        }
        for (var i = 0; i < devices.length; i++)
        {
            var category = document.createElementNS(XULNS, "ttlistboxcategory");
            category.setAttribute("label", devices[i].model.modelName + " - " + devices[i].serial);
            listbox.addItem(category);

            var installLocations = gApplication.deviceManager.getInstallLocationsForDevice(devices[i]);
for (var j = 0; j < installLocations.length; j++)
            {
                devices[i].installLocations().addObserver(observer);

                var item = document.createElementNS(XULNS, "ttdevice");
                item.setAttribute("location", installLocations[j].name);

                switch (installLocations[j].physicalType)
                {
                    case CI.ttIInstallLocation.kLocationTypeInternal:
                        item.setAttribute("location-type", "internal");
                        break;
                    case CI.ttIInstallLocation.kLocationTypeHardDisk:
                        item.setAttribute("location-type", "harddisk");
                        break;
                    case CI.ttIInstallLocation.kLocationTypeMemoryCard:
                        item.setAttribute("location-type", "memorycard");
                        break;
                    case CI.ttIInstallLocation.kLocationTypeCardReader:
                        item.setAttribute("location-type", "cardreader");
                        break;
                    default:
                        item.setAttribute("location-type", "unknown");
                        break;
                }
                var path = installLocations[j].path.path;
                if (path.indexOf("\\\\{") == 0 && path.indexOf("}") > 0)
                    path = path.substring(path.indexOf("}") + 1);
                item.setAttribute("path", path);

                var mBytes = ttItem.sizeBytesToMB(installLocations[j].sizeInBytesFree);
                item.setAttribute("space", mBytes);

                category.addItem(item);

                if (gApplication.deviceManager.activeInstallLocation == installLocations[j])
                {
                    var wrapper = document.createElementNS(XULNS, "hbox");
                    var descr = document.createElementNS(XULNS, "description");
                    descr.textContent = gStringBundle.getString("location.selected.label")
                    wrapper.appendChild(descr);
                    item.addMixedCell(wrapper);
                }
                else
                {
                    item._device = devices[i];
                    item._installLocation = installLocations[j];
                    item.addButton(gStringBundle.getString("location.select.button"),
                        function(item)
                        {
                            dialog.close({ device : item._device, installLocation : item._installLocation });
                        });
                }
            }
        }
        listbox.doneLoading = true;
    }
    catch(e)
    {
        error(e);
    }
}
