/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/


function ErrorDialog()
{
    ttDialog.call(this, "error-footer");
}
ErrorDialog.prototype =
{
    _sb : null,
    _message : "unknown",
    _dumpText : null,
    _exception : null,
    _haveInternet : true,
    _allowReportToServer : true,
    _autoReport : false,

    onLoad : function(params)
    {
        try {
            this._sb = getStringBundle("chrome://tthome/locale/ui/dialogs/error/errorPage.properties");
            this._message = params.message;
            this._dumpText = params.dumpText;
            this._exception = params.exception;

            if (typeof this._exception == "object" && "allowReportToServer" in this._exception)
            {
                this._allowReportToServer = this._exception.allowReportToServer;
            }
            if (params.severity == Exception.SEVERITY_INFO)
            {
                this._allowReportToServer = false;
            }

            try {
                this._autoReport = prefService().getBoolPref("tomtom.debug.error.autoReport");
            } catch (e) {
                unimportantError(e);
            } finally {
                E("auto-report-checkbox").checked = this._autoReport;
            }

            if(params.severity == 500){
                this._message = this._message + " " +this._sb.GetStringFromName("firewall.label");
            }

            this.setMessage(this._message);
            this.setException(this._exception);
            this.setSeverity(params.severity);

            this.debugMessageInfo(params.exception);
            var showDebugInfo = prefService().getBoolPref("tomtom.debug.error.showDebugInfo");
            let showDebugMsg = false;
            let isShowDebug = false;
            
            try{
                showDebugMsg = prefService().prefHasUserValue("tomtom.debug.error.showDebugInformation");
                if(showDebugMsg)
                {
                    isShowDebug = prefService().getBoolPref("tomtom.debug.error.showDebugInformation");
                }
            }  catch(e){ unimportantError(e); }

            if(showDebugMsg && isShowDebug){
                this.setDumpText(this._dumpText);
                E("toggleDetails").hidden = false;
                if (this._allowReportToServer || showDebugInfo)
                {
                    this.showDetails(showDebugInfo);
                }
                else
                {
                    E("toggleDetails").hidden = true;
                }
            }

            window.sizeToContent();
            try {
                if (!this._allowReportToServer)
                {
                    E("report").disabled = true;
                    E("reportcontainer").hidden = true;
                    E("reportseparator").hidden = true;
                }
                else if (this._haveInternet)
                {
                    let isBeta = false;
                    try {
                        isBeta = prefService().getBoolPref("tomtom.app.beta");
                    } catch (e) { unimportantError(e); }
                    if (isBeta)
                        E("auto-report-checkbox").hidden = true;
                    if (this._autoReport || isBeta)
                        this.report();
                }
            } catch (e) { unimportantError(e); }
        } catch (e) { alert("Error in error page: " + e); }
    },

    debugMessageInfo : function(ex)
    {
        var msg = (ex.id ? ex.id + ": " : "");
        if (ex.debugInfo)
            msg += "\n" + ex.debugInfo+ex.debugInfo;
        
        if(msg != "")
        {
            E("debugMessage").textContent = unescape(decodeURIComponent(msg));
            E("debugInformation").hidden = false;
        }
    },

    setSeverity : function (aSeverity)
    {
        switch (aSeverity)
        {
            case Exception.SEVERITY_INFO:
                var title    = this._sb.GetStringFromName("window.warning.title");
                var severity = "info";

                window.title = title;
                var header = E("title");
                header.setAttribute("severity", severity);
                header.setAttribute("title", title);

                E("user-message").setAttribute("severity", severity);
                E("faq").hidden = true;
                break;
            case Exception.SEVERITY_WARNING:
                var title    = this._sb.GetStringFromName("window.warning.title");
                var subtitle = this._sb.GetStringFromName("window.warning.subtitle");
                var severity = "warning";

                E("information").textContent = this._sb.GetStringFromName("informationNonFatal");
                window.title = title;
                var header = E("title");

                header.setAttribute("severity", severity);
                header.setAttribute("title", title);
                E("subtitle").hidden = false;
                E("subtitle").subtitle = subtitle;
                E("user-message").setAttribute("severity", severity);
                E("faq").hidden = true;
                break;
            case 500:
                E("faq").hidden = true;
                E("downloadFaq").hidden = false;
                break;
            default:
        }
    },

    setMessage : function (message)
    {
        try {
            if (this._exception &&
                (this._exception.name == "ReferenceError" ||
                 this._exception.name == "TypeError" ||
                 this._exception.name == "SyntaxError"))
            {
                message = this._sb.GetStringFromName("genericReplacementMessage");
            }
        } catch (e) {
}

        E("message").textContent = unescape((decodeURIComponent(message)));
    },

    setException : function (ex)
    {
        if (ex &&
            ex instanceof ServerError &&
            ex.code == Components.results.NS_ERROR_OFFLINE)
        {
            E("report").hidden = true;
            this._haveInternet = false;
        }
    },

    setDumpText : function (text)
    {
        E("dump-text").value = text + "\nTime: " + (new Date).toUTCString();
    },

    showDetails : function(show)
    {
        if (show)
        {
            E("debuginfo").hidden = false;
            E("debugseparator").hidden = false;
            E("toggleDetails").label = this._sb.GetStringFromName("hideDetails");
        }
        else
        {
            E("debuginfo").hidden = true;
            E("debugseparator").hidden = true;
            E("toggleDetails").label = this._sb.GetStringFromName("showDetails");
        }
        this._resizeWindowVertically();
    },

    toggleDetails : function()
    {
        this.showDetails(E("debuginfo").hidden);
    },

    report : function ()
    {
        try {
            E("report").hidden = true;
            E("reportstatus").hidden = false;
            E("report-result").textContent = this._sb.GetStringFromName("reportInProgress");
            this._resizeWindowVertically();

            var errorInfo = <jserror/>;
            if (this._exception &&
                typeof this._exception == "object" && "debugMessage" in this._exception)
                errorInfo.message = this._exception.debugMessage;
            else
                errorInfo.message = this._message;

            if (this._exception &&
                typeof this._exception == "object" && "id" in this._exception)
                errorInfo.id = this._exception.id;

            if (this._dumpText)
            {
                errorInfo.stack = this._dumpText;
            }
            if (this._exception &&
                typeof this._exception == "object" &&
                "lineNumber" in this._exception &&
                ("filename" in this._exception || "fileName" in this._exception))
            {
                if ("filename" in this._exception) {
                    errorInfo.line.@file = this._exception.filename;
                } else {
                    errorInfo.line.@file = this._exception.fileName;
                }
                errorInfo.line.@linenumber = this._exception.lineNumber;
            }
            errorInfo.environment.appversion = gApplication.settings.version;
            errorInfo.environment.language = gApplication.settings.networkLocale;
            errorInfo.environment.redirectserver.@url = rpc.serviceBaseURL("redirect").spec;
            try {
                var devid = prefService().getCharPref("tomtom.lastConnectedDevice.id");
                var modelid = prefService().getCharPref("tomtom.device." + devid + ".modelID");
                var navcore = prefService().getCharPref("tomtom.device." + devid + ".navcoreVersion");
                errorInfo.environment.device.@model = modelid;
                errorInfo.environment.device.@navcoreversion = navcore;
            } catch (e) { unimportantError(e); }
            try {
                var os = Cc['@mozilla.org/xre/app-info;1'].getService(CI.nsIXULRuntime).OS;
                errorInfo.environment.os.@family = os;
                errorInfo.environment.os.@nameversion = window.navigator.oscpu;
            } catch (e) { unimportantError(e); }

            var me = this;
            dialog.networkCall(rpc.call("statistics", "logError", {}, errorInfo, RPC.METHOD_POST,
                function(result)
                {
                    var incidentID = makeSafe.integer(result.@incidentid);{}
E("report-result").textContent =
                        me._sb.formatStringFromName(me._autoReport ? "reportResult_auto" : "reportResult", [incidentID], 1);
                    E("auto-report-checkbox").disabled = false;
                    me._resizeWindowVertically();

                    try {
                        var currentIDStr = prefService().getCharPref("tomtom.debug.error.reportedIncidents");
                        var currentIDList = makeSafe.list(currentIDStr, makeCallback(makeSafe, makeSafe.integer));
                        currentIDList = currentIDList.slice(-4);
                        currentIDList.push(incidentID);
                        prefService().setCharPref("tomtom.debug.error.reportedIncidents", currentIDList.join(","));
                    } catch (e) { unimportantError(e); }
                },
                makeCallback(this, this.reportFunctionHadError)));
        } catch (e) { this.reportFunctionHadError(e); }
    },

    reportFunctionHadError : function (e)
    {
        if (!document)
            return;
        unimportantError(e);
        E("report-result").textContent = errorToText(e);
        E("report-result").setAttribute("class", "error");
        E("auto-report-checkbox").disabled = false;
        this._resizeWindowVertically();
    },

    _resizeWindowVertically : function()
    {
        window.resizeBy(0, E("error-panel").boxObject.height - window.innerHeight);
    },

    back : function()
    {
        try {
            var shouldAutoReport = E("auto-report-checkbox").checked;
            if (this._autoReport != shouldAutoReport)
                prefService().setBoolPref("tomtom.debug.error.autoReport", shouldAutoReport);
        } catch (e) { unimportantError(e); }

        this.close();
    },

    __end : null
};
try {
    extend(ErrorDialog, ttDialog);
    var dialog = new ErrorDialog();
} catch (e) {
alert(e);
    unimportantError(e);
    window.close();
};
