/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/


function MapActivationDialog()
{
    ttDialog.call(this, "activation-footer");
}
MapActivationDialog.prototype =
{
    _map: null,
    _stringbundle: null,

    onLoad: function(params)
    {
        this._allowContinue(false);
        this._stringbundle = E("stringbundle");
        this._map = params.map;

        if (this._map.drmScheme == "activation-code")
        {
            var header = E("activation-header");
            header.title = this._stringbundle.getString("productCodePrompt.title");
            header.subtitle = this._stringbundle.getString("productCodePrompt.subtitle");
        }
    },
_getCodeValue: function()
    {
        var code = E("code-field").value;
        code = code.toUpperCase().trim();
        return code;
    },
validateCode: function()
    {
        var code = this._getCodeValue();

        var expectedLength = (this._map.drmScheme == "durham" ? 11 : 15);
        if (code.length < expectedLength)
        {
            this._allowContinue(false);
            this._markInvalid(E("code-field"), null);
            return;
        }

        try
        {
            if (this._map.drmScheme == "durham")
                checkPromotionCodeSyntax(code);
            else
                checkProductcodeSyntax(code);
        }
        catch (e)
        {
            this._allowContinue(false);
            this._markInvalid(E("code-field"), e);
            return;
        }

        this._allowContinue(true);
        this._markValid(E("code-field"));
    },
_markValid: function(node)
    {
        node.parentNode.removeAttribute("invalid");
    },
_markInvalid: function(node, e)
    {
        e = errorToText(e);

        node.parentNode.setAttribute("invalid", "true");
        if (node.hasAttribute("errorText"))
            E(node.getAttribute("errorText")).setAttribute("value", e);
    },

    cancel : function()
    {
        this.close(null);
    },

    next: function()
    {
        this._disableUI();

        var code = this._getCodeValue();
        if (this._map.drmScheme == "durham")
            this._usePromotionCode(code);
        else
            this._useProductCode(code);
    },
_errorCallback: function(aError)
    {
        if (this.abortingCalls)
            return;

        this._enableUI();
        error(aError);
    },
_usePromotionCode: function(promotionCode)
    {
        var me = this;
        this.networkCall(rpc.call("shop", "activatePromotion",
                {
                    promotionCode : promotionCode,
                    deviceModel: gApplication.deviceManager.activeDevice.model.id,
                    deviceSerial : gApplication.deviceManager.activeDevice.serial
                },
                null, RPC.METHOD_GET,
                function(xml)
        {
            var items = gApplication.itemFactory.createItemListFromServer(xml);
            for each (var item in items)
            {
                if (item.idurl == me._map.idurl)
                {
                    for each (var location in item.locations)
                    {
                        if (location.content == "activation" || location.content == "durham")
                        {
                            me.networkCall(new Downloader().download(location.url, gApplication.settings.osTempDir,
                                function() {},
                                function(certificateFile)
                                {
                                    var task = gMainWindow.currentTask();
                                    if (task)
                                    {
                                        task.addCleanupHandler(function()
                                        {
                                            try {
                                                certificateFile.remove(false);
                                            } catch (e) { unimportantError(e); }
                                        });
                                    }

                                    me.close(me._map.createCopyWithDurhamCertificate(certificateFile));
                                }, makeCallback(me, me._errorCallback)));
                            return;
                        }
                    }
                }
            }

            error(me._stringbundle.getString("promotionCode.noResult"));
        }, makeCallback(this, this._errorCallback)));
    },
_useProductCode: function(productCode)
    {
        var me = this;
        this.networkCall(rpc.call("client", "useProductCode",
                     {
                        productCode : productCode,
                        deviceModel: gApplication.deviceManager.activeDevice.model.id,
                        deviceCode : gApplication.deviceManager.activeDevice.deviceCode
                     },
                     null, RPC.METHOD_GET,
                     function(xml)
        {
            var activationCode = makeSafe.alphanumspace(makeSafe.nonemptystring(xml.activation.@code));
            me.close(me._map.createCopyWithActivationCode(activationCode));
        }, makeCallback(this, this._errorCallback)));
    },
_allowContinue: function(aAllowed)
    {
        E("activation-footer").getRightButton().disabled = !aAllowed;
    },
_disableUI: function()
    {
        var nextBtn = E("activation-footer").getRightButton();
        nextBtn.disabled = true;
        nextBtn.setAttribute("busy", "true");
    },
_enableUI: function()
    {
        var nextBtn = E("activation-footer").getRightButton();
        nextBtn.removeAttribute("busy");
        this.validateCode();
    },

    __end: null
};
extend(MapActivationDialog, ttDialog);
var dialog = new MapActivationDialog();
