/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/


function AddonUpdate()
{
    ttSubTask.apply(this, arguments);
    this._sb = getStringBundle("chrome://tthome/locale/ui/subtask/addonUpdate/addonUpdate.properties");
}
AddonUpdate.prototype =
{
    name    : "addonUpdate",
    _sb     : null,
    startup : false,

    start : function(startup)
    {
        ttSubTask.prototype.start.apply(this, arguments);

        this.startup = startup;
        if (this.startup)
            prefService().setBoolPref("extensions.update.notifyUser", false);

        this.pageLoad();
    },

    pageLoad : function()
    {
        this._ttwizard.loadPage(
            this._constructPageURL("addonUpdatePage"),
            this.startup,
            makeCallback(this, this.pageLoadAsync),
            null,
            makeCallback(this, this.pageDone)
            );
    },

    pageLoadAsync : function(page)
    {
        var itemlist = new ttItemList();
        page.inputStart(itemlist);

        var me = this;
        var addItems = function(list)
        {
            itemlist.addItemListArray(list.filter(me.validateUpdate, me));
            if (me.startup && itemlist.contents.length == 0)
                me.subtaskDone(null);
            else
                page.inputDone();
        };
        if (this.startup)
            addItems(gApplication.addonManager.getKnownAddonUpdates());
        else
            this.networkCall(gApplication.addonManager.requestAddonUpdates(addItems, errorAbortTask));
    },
validateUpdate : function(item)
    {
        if (!(item instanceof ttAddon) || ttAddon.getOperationPending(item.idstr))
            return false;

        try {
            var extensionManager = Cc["@mozilla.org/extensions/manager;1"].getService(CI.nsIExtensionManager);
            var installedItem = extensionManager.getItemForID(item.idstr);
            if (!installedItem)
                throw new InvalidValue("Server returned an update for an add-on that isn't installed", item.idstr);
            return versionNewerThan(item.version, installedItem.version);
        } catch (e) {
            unimportantError(e);
            return false;
        }
    },

    pageDone : function(result)
    {
        for each (let item in result)
        {
            this._ttwizard.addCommit(
                makeCallback(this, this.installCommit),
                item,
                this._sb.formatStringFromName("commit.install.name", [item.name], 1),
                this.commitPriority);
        }
        this._ttwizard.addCommit(
            ttAddon.restartCommitFunction,
            this.startup,
            getStringBundle("chrome://tthome/locale/logic/model/ttAddon.properties").GetStringFromName("commit.restart.label"),
            this.commitPriority
        );
        this.subtaskDone(result);
    },

    installCommit : function(item, progressCallback, successCallback, errorCallback)
    {
        var me = this;
        var successCallbackWrapper = function(newItem)
        {
            successCallback(me._sb.GetStringFromName("installed.label"));
        };

        return item.download(progressCallback, successCallbackWrapper, errorCallback);
    },

    __end : null
}
extend(AddonUpdate, ttSubTask);
