/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/


function Addons()
{
    ttSubTask.apply(this, arguments);
    this._stringBundle = getStringBundle("chrome://tthome/locale/ui/subtask/addons/addons.properties");
}
Addons.prototype =
{
    name : "addons",
    _stringBundle : null,
    _contentType : null,
start : function(contentType)
    {
        assert(contentType == "extension" || contentType == "theme", "Only extensions and themes are supported so far");
        this._contentType = contentType,
        ttSubTask.prototype.start.apply(this, arguments);
        this.addonsLoad();
    },

    addonsLoad : function()
    {
        this._ttwizard.loadPage(
            this._constructPageURL("addonsPage"),
            this._contentType,
            makeCallback(this, this.pageLoadAsync),
            null,
            makeCallback(this, this.pageDone)
            );
    },

    pageLoadAsync : function(page)
    {
        var itemlist = new ttItemList();
        itemlist.addFilter(new ttDuplicateFilter());
        itemlist.addFilter(new ttDependencyTreeBuilder());
        page.inputStart(itemlist);

        var addons = [];
        switch (this._contentType)
        {
            case "extension":
                addons = ttExtension.prototype.importItemsFromExtensionManager();
                break;
            case "theme":
                addons = ttTheme.prototype.importItemsFromExtensionManager();
                break;
        }
        itemlist.addItemListArray(addons);

        var serverargs =
        {
            contentType : this._contentType,
            price : "all",
            operatingSystem : gApplication.settings.operatingSystemServerName,
            recommendationsOnly : "false",
            publicBeta : prefService().getBoolPref("tomtom.app.showBetaVersions")
        };

        var dev = gApplication.deviceManager.activeDevice;
        if (dev)
        {
            serverargs.deviceModel = dev.model.id;
            serverargs.deviceCapabilities = gApplication.deviceManager.activeDeviceCapabilitiesString;
            serverargs.deviceParameters = gApplication.deviceManager.activeDeviceParametersString;
            serverargs.deviceNavcoreVersion = dev.versionNavCore;
            serverargs.deviceNavcoreBuild = dev.buildNumberNavCore;
        }

        var me = this;
        this.networkCall(rpc.call("shop", "getShopContent", serverargs,
            null, RPC.METHOD_GET,
            function(result)
            {
                itemlist.addItemListFromServer(result);
                page.inputDone();
            },
            function(e)
            {
                errorNonFatal(e);
                page.inputDone();
            }));
    },

    pageDone : function(result)
    {
        for each (let [action, item] in result)
        {
            switch(action)
            {
                case "install":
                    this._ttwizard.addCommit(
                        makeCallback(this, this.installCommit),
                        item,
                        this._stringBundle.formatStringFromName("commit.install.name", [item.name], 1),
                        this.commitPriority);
                    break;
                case "uninstall":
                    this._ttwizard.addCommit(
                        makeCallback(this, this.uninstallCommit),
                        item,
                        this._stringBundle.formatStringFromName("commit.uninstall.name", [item.name], 1),
                        this.commitPriority);
                    break;
                case "activate":
                    this._ttwizard.addCommit(
                        makeCallback(this, this.activateCommit),
                        item,
                        this._stringBundle.formatStringFromName("commit.activate.name", [item.name], 1),
                        this.commitPriority);
                    break;
                case "enable":
                    this._ttwizard.addCommit(
                        makeCallback(this, this.enableDisableCommit),
                        [item, true],
                        this._stringBundle.formatStringFromName("commit.enable.name", [item.name], 1),
                        this.commitPriority);
                    break;
                case "disable":
                    this._ttwizard.addCommit(
                        makeCallback(this, this.enableDisableCommit),
                        [item, false],
                        this._stringBundle.formatStringFromName("commit.disable.name", [item.name], 1),
                        this.commitPriority);
                    break;
                case "update":
                    this.loadSubTask(new AddonUpdate(), null, makeCallback(this, this.subtaskDone));
                    return;
            }
        }
        this._ttwizard.addCommit(
            ttAddon.restartCommitFunction,
            null,
            getStringBundle("chrome://tthome/locale/logic/model/ttAddon.properties").GetStringFromName("commit.restart.label"),
            this.commitPriority
        );
        this.subtaskDone(result);
    },

    activateCommit : function(item, progressCallback, successCallback, errorCallback)
    {
        item.setEnabled(true);
        if (this._contentType == "theme")
            this.markUserChosenTheme(item);

        successCallback(this._stringBundle.GetStringFromName("activated.label"));

        return new NoOpCall();
    },

    installCommit : function(item, progressCallback, successCallback, errorCallback)
    {
        var me = this;
        var successCallbackWrapper = function(newItem)
        {
            if (me._contentType == "theme")
                me.markUserChosenTheme(item);
            successCallback(me._stringBundle.GetStringFromName("installed.label"));
        };

        return item.download(progressCallback, successCallbackWrapper, errorCallback);
    },

    uninstallCommit : function(item, progressCallback, successCallback, errorCallback)
    {
        var me = this;
        var successCallbackWrapper = function()
        {
            successCallback(me._stringBundle.GetStringFromName("uninstalled.label"));
        };

        return item.uninstall(progressCallback, successCallbackWrapper, errorCallback);
    },

    enableDisableCommit : function([item, requestedState], progressCallback, successCallback, errorCallback)
    {
        item.setEnabled(requestedState);

        successCallback(this._stringBundle.GetStringFromName(requestedState ? "enabled.label" : "disabled.label"));

        return new NoOpCall();
    },
markUserChosenTheme : function(item)
    {
        try
        {
            prefService().setBoolPref("tomtom.branding.userChoseTheme", true);

            if (item)
                prefService().setCharPref("tomtom.app.customSkin", item._idstr);
        }
        catch(e)
        {
            unimportantError(e);
        }
    },

    __end : null
}
extend(Addons, ttSubTask);
