/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/


function Backup()
{
    ttSubTask.apply(this, arguments);
}
Backup.prototype =
{
    name : "backup",
    _stringBundle : null,

    start : function()
    {
        ttSubTask.prototype.start.apply(this, arguments);
        this._stringBundle = getStringBundle("chrome://tthome/locale/ui/subtask/backup/backup.properties");
        this._task.needDevice(makeCallback(this, this.initiateBackupLoad));
    },

    initiateBackupLoad : function()
    {
        this._ttwizard.loadPage(
            this._constructPageURL("initiateBackup"),
            null,
            null,
            null,
            makeCallback(this, this.initiateBackupDone)
            );
    },

    createBackup : function(successCallback)
    {
        var backupsDir = gApplication.settings.backupsDir;
        var backup = getLatestBackup(true, backupsDir);

        if (backup.location.isDirectory())
        {
            var dir = backup.location.clone();
            dir.append("backup.xml");
            if(dir.exists())
            {
                removeFile(dir, false);
            }
            dir.create(dir.NORMAL_FILE_TYPE, 0774);
        }

        var xml = '';
        var me= this;
        var items = [];
        var noOfScannedLocations = 0;
        var installLocations = gApplication.deviceManager.getInstallLocationsForDevice();
        var devItems = new ttItemList();
        devItems.addFilter(new ttDependencyTreeBuilder());
        for (var i = 0; i < installLocations.length ; i++)
        {
            var installLocationName = installLocations[i];
            installLocationName.deviceNavItems().populate(
                new ScanListener(function() {},
                        function()
                        {
                            devItems.addItemListFromDLL(installLocationName.deviceNavItems());
                            items = devItems.contents;
                            noOfScannedLocations++;
                            if(noOfScannedLocations == (installLocations.length))
                            {
                                me.writeXml(items, dir, successCallback);
                            }
                        },
                        error
                        ));
        }
    },

    writeXml : function(items, dir, successCallback)
    {
        var gDLLUtilsStringBundle = getStringBundle("chrome://tthome/locale/logic/dllUtils.properties");
        var table = <TomTomXml version="1.0"/>;
        var sortedItemList = items.sort(function (a,b){ return b['typePriority'] - a['typePriority']});
        for each(let item in sortedItemList)
        {
            var xml = this.createXML(item);
            table.appendChild(xml);
        }
        writeStringToFile(dir, table.toXMLString());
        successCallback(gDLLUtilsStringBundle.GetStringFromName("copyDone"));
    },

    createXML : function(item)
    {
        var xml = <Item />;
        xml.name = encodeURIComponent(item['name']);
        xml.description = encodeURIComponent(item['description']);
        xml.version = item['version'];
        xml.size = item['size'];
        xml.typeID = item['typeID'];
        if(item['icon'])
        xml.icon = item['icon']['url'];
        return xml;
    },

    initiateBackupDone : function()
    {
        var backUpAlways = this._ttwizard._wizbrowser.contentDocument.getElementById("alwaysbackup");
        var backupfooter = this._ttwizard._wizbrowser.contentDocument.getElementById("backupfooter");
        if(backupfooter.hidden == false)
        {
            if(backUpAlways.checked)
            {
                prefService().setIntPref("tomtom.backup.automatic", 1);
            }
            else
            {
                prefService().setIntPref("tomtom.backup.automatic", 2);
            }
        }

        var deviceid = gApplication.deviceManager.activeDevice.id;
        try
        {
            var backupFolder = gApplication.settings.backupsDir;
            var backup = getLatestBackup(true, backupFolder);
            if (!backup)
            {
                var backups = gApplication.homeServices.backups();
                backups.setFolder(backupFolder);
                backup = backups.createEmptyBackup(deviceid);
            }

            this._ttwizard.addCommit(
                makeCallback(this, this.backupCommit),
                backup,
                this._stringBundle.GetStringFromName("creating.backup"),
                this.commitPriority);
        }
        catch(e)
        {
            errorNonFatal(e);
        }
        this.subtaskDone();
    },

    backupCommit : function(backup, progressCallback, successCallback, errorCallback)
    {
        var errorStrOut = { value : null };
        var severity    = { value : null };
        backup.checkPreconditionsBackupOk(severity, { value: null }, errorStrOut);
        if (severity.value != 0 && severity.value != CI.ttIError.kSeverityWarning)
        {
            errorCallback(errorStrOut.value);
            return new NoOpAbortable();
        }
        var me = this;
        return new Abortable(backup.doBackup(new BaseListener(function(percent)
                {
                    progressCallback(percent * 0.1, me._stringBundle.GetStringFromName("scanning"));
                },
                function(errorCode) {},
                errorCallback),
                new CopyListener(function (percent, text)
                {
                    progressCallback(10 + (percent * 0.9), text);
                },
                function()
                {
                     me.createBackup(successCallback);
                },
                errorCallback)
        ));
    },
    __end : null
}
extend(Backup, ttSubTask);
