/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/


function Checkout()
{
    ttSubTask.apply(this, arguments);

    this._stringBundle = getStringBundle("chrome://tthome/locale/ui/subtask/checkout/checkout.properties");
}
Checkout.prototype =
{
    name : "checkout",
    orderID : null,
    itemsPurchased : null,
    _ucUserAccountDetails : null,
    _stringBundle : null,
    _ignoreBrowserHistory : null,
    _devItems : null,

    start : function()
    {
        ttSubTask.prototype.start.apply(this, arguments);
        this._ignoreBrowserHistory = false;
        this.cartLoad();
    },

    cartLoad : function()
    {
        this._ttwizard.loadPage(
            this._constructPageURL("showCart"),
            {
                ignoreBrowserHistory: this._ignoreBrowserHistory
            },
            null,
            null,
            makeCallback(this, this.cartDone)
        );
    },
    cartDone : function()
    {
        this.orderOverviewLoad();
    },

    orderOverviewLoad : function()
    {
        this._ttwizard.loadPage(
            this._constructPageURL("orderOverview"),
            null,
            makeCallback(this, this.orderOverviewAsyncPrecond),
            makeCallback(this, this.orderOverviewCheck),
            makeCallback(this, this.orderOverviewDone)
        );
    },

    orderOverviewAsyncPrecond : function(page)
    {
        var me = this;
        this.networkCall(this._task.needLogin(function()
        {
            me.orderOverviewAsyncLoad(page);
        },
        function()
        {
            me._ignoreBrowserHistory = true;
            me.cartLoad();
        }));
    },

    orderOverviewAsyncLoad : function(page)
    {
        var me = this;
        this.networkCall(rpc.call("account", "getAccountDetails",
            null,
            null, RPC.METHOD_GET, function(result)
            {
                me._ucUserAccountDetails = result;
                var companyData = {
                                isForCompany : false,
                                server : { company : makeSafe.string(result.company),
                                           vatnumber : makeSafe.string(result.vatnumber)  },
                                local :  { company : "", vatnumber: "" }
                              };
                page.inputDone(companyData);
            },
            errorAbortTask));
    },

    orderOverviewCheck : function(data, successCallback, errorCallback)
    {
        var me = this;
        if (data.isForCompany &&
            (data.local.vatnumber == "" || data.local.company == ""))
        {
            var ms = new MessageService();
            ms.alert(this._stringBundle.GetStringFromName("emptyCompanyOrVatTitle"),
                     this._stringBundle.GetStringFromName("emptyCompanyOrVatMsg"));
            return;
        }
        successCallback(data);
    },

    orderOverviewDone : function(data)
    {
        var me = this;
        var companyDataChanged = data.server.vatnumber != data.local.vatnumber ||
                                 data.server.company != data.local.company;
        if (data.isForCompany && companyDataChanged)
        {
            this.networkCall(rpc.call("account", "updateAccount",
                        {
                            company : data.local.company,
                            vatnumber : data.local.vatnumber,
                            publicBeta : prefService().getBoolPref("tomtom.app.showBetaVersions")
                        },
                        null, RPC.METHOD_POST,
                        makeCallback(this, this.billingAddressLoad), errorNonFatal));
        }
        else
        {
            this.billingAddressLoad();
        }
    },

    billingAddressLoad : function()
    {
        var ucAddress = this._ucUserAccountDetails.address;
        var address = {
            firstname: makeSafe.string(ucAddress.firstname),
            lastname: makeSafe.string(ucAddress.lastname),
            gender: makeSafe.string(this._ucUserAccountDetails.gender),
            street : makeSafe.string(ucAddress.street),
            zipCode :makeSafe.string(ucAddress.zipcode),
            city : makeSafe.string(ucAddress.city),
            state : makeSafe.string(ucAddress.state),
            country : makeSafe.string(ucAddress.country),
            company : makeSafe.string(this._ucUserAccountDetails.company),
            vatnumber : makeSafe.string(this._ucUserAccountDetails.vatnumber)
        }
            this._ttwizard.loadPage(
                this._constructPageURL("billingAddress"),
                address,
                null,
                null,
                makeCallback(this, this.billingAddressDone)
            );

    },

    billingAddressDone : function(data)
    {
        data.publicBeta = prefService().getBoolPref("tomtom.app.showBetaVersions");
		var theResidenceCountry = data.vatcountryresidence;
		var theSelfCertifiedCountry = data.vatcountryselfcertificated;
		theUserBillingCountry = data.country;
		data.vatcountryresidence = null;
		data.vatcountryselfcertificated = null;
		delete data.vatcountryresidence;
		delete data.vatcountryselfcertificated;
        this.networkCall(rpc.call("account", "updateAccount",
                     data,
                     null, RPC.METHOD_POST,
                     makeCallback(this, this.paymentLoad, 
								{userbillingcountry: theUserBillingCountry, vatcountryresidence : theResidenceCountry, vatcountryselfcertificated: theSelfCertifiedCountry}), errorNonFatal));
    },

    paymentLoad : function(data)
    {
        var selectedItems = gApplication.cart.contentsFlat;
        var itemIDs = "";
        for (var i = 0; i < selectedItems.length; i++)
        {
            itemIDs += (itemIDs == "" ? "" : ",") + selectedItems[i].idnum;
        }

        var dev = gApplication.deviceManager.activeDevice;

        var xmlUploadBody = null;
        try
        {
            if (dev && dev.activeMap)
            {
                var activeMap = gApplication.itemFactory.createItemFromDLL(dev.activeMap);
                xmlUploadBody = <initiatePayment>
                                    <contentidlist>
                                        <content idurl={activeMap.idurlWithVersion} certificate={activeMap.sourceDLL.certificates}></content>
                                    </contentidlist>
                                </initiatePayment>;
            }
        } catch (e) { unimportantError(e); }

        var deviceSerial = "", pragueSerial = "", sdcardid = "";
        if (dev)
        {
            deviceSerial = dev.serial;
            if (dev.gprsInfo)
                pragueSerial = dev.gprsInfo.imei;
            sdcardid = dev.SDCardId;
        }

		var theInitiatePaymentData = new function()
		{
			this.deviceSerial = deviceSerial;
            this.pragueSerial = pragueSerial;
            this.SDCardId = sdcardid;
            this.deviceModel = dev.model.id;
            this.deviceCapabilities = gApplication.deviceManager.activeDeviceCapabilitiesString;
            this.deviceParameters = gApplication.deviceManager.activeDeviceParametersString;
			(data.vatcountryresidence != "") && (this.vatcountryresidence = data.vatcountryresidence);
			(data.vatcountryselfcertificated != "" && data.userbillingcountry != data.vatcountryresidence) &&  (this.vatcountryselfcertificated = data.vatcountryselfcertificated);
            this.items = itemIDs;
            this.productlist = itemIDs;
		};
	
        var me = this;
        this.networkCall(rpc.call("payment", "initiatePayment",
                    theInitiatePaymentData, 
                    xmlUploadBody, RPC.METHOD_POST, function(result)
        {
            me.orderID = makeSafe.integer(result.orderid);
            var startURL = makeSafe.url(result.starturl).spec;
            var endURL = makeSafe.url(result.endurl).spec;{}
{}
me._ttwizard.loadPage(
                "chrome://tthome/content/ui/remoteContentPage/remoteContentPage.xul",
                {
                    title: me._stringBundle.GetStringFromName("payment.title"),
                    subtitle: me._stringBundle.GetStringFromName("payment.subtitle")
                },
                function(page) {
                    me._ttwizard.hidecartbutton = true;
                    page.inputDone({
                        startURL: startURL,
                        endURL: endURL,
                        securityUICallback: function(shown) {
                            top.gMainWindow.statusbar.setLockVisible(shown);
                        },
                        securityExceptionResolver: function() {
                            return "Checkout/PaymentLoad/ExpectedSecureConnection";
                        }
                    });
                },
                null,
                makeCallback(me, me.paymentDone)
            );
        },
        function(e) {
            if (e.code == 409)
            {
                error(e);
                me._ttwizard.setBusy(false);
            }
            else
            {
                errorAbortTask(e);
            }
        }));
    },

    paymentDone : function(result)
    {
        this.orderConfirmationLoad();
    },

    orderConfirmationLoad : function()
    {
        this._ttwizard.loadPage(
            this._constructPageURL("orderConfirmation"),
            { orderID : this.orderID },
            makeCallback(this, this.orderConfirmationAsyncLoad),
            null,
            makeCallback(this, this.orderConfirmationDone)
        );
    },

    orderConfirmationAsyncLoad : function(page)
    {
page.inputStart();
        var me = this;
        var args =
        {
            orderID : me.orderID
        };
        addSubscriptionsRPCArgs(args);
        this.networkCall(rpc.call("payment", "getPaymentStatus",
                     args, null, RPC.METHOD_GET, function(result)
        {
            var totalPrice = gApplication.cart.totalPrice;

            gApplication.cart.clear();

            var dev = gApplication.deviceManager.activeDevice;
            var args =
            {
                orderID : me.orderID,
                deviceCode: dev.deviceCode,
                deviceSerial: dev.serial,
                deviceModel: dev.model.id,
                deviceCapabilities: gApplication.deviceManager.activeDeviceCapabilitiesString,
                deviceParameters: gApplication.deviceManager.activeDeviceParametersString,
                deviceNavcoreVersion : dev.versionNavCore,
                deviceNavcoreBuild : dev.buildNumberNavCore,
                publicBeta : prefService().getBoolPref("tomtom.app.showBetaVersions")
            };
            addSubscriptionsRPCArgs(args);
            me.doDeviceScan(function()
            {
                var devItems = me._devItems.contents;
                var accountLicensedContentBody = <getaccountalicensedcontent><contentidlist/></getaccountalicensedcontent>;
                var accountLicensedContentList = {5: null};

                me.networkCall(rpc.call("shop", "getAccountLicensedContent",
                             args, accountLicensedContentBody, RPC.METHOD_POST & ~RPC.USE_CACHE, function(result)
                {
                    var list = new ttItemList();
                    list.addFilter(new ttDependencyTreeBuilder());
                    me.itemsPurchased = list.addItemListFromServer(result);
                    page.inputDone({success: true,
                                    totalPrice : totalPrice,
                                    itemlist : list,
                                    error : null });
                },
                errorAbortTask));
            });
        },
        function(error) {
            if (error instanceof ServerError &&
                (error.code == 780 ||
                 error.code == 790))
                gApplication.cart.clear();
            errorAbortTask(error);
        }));
    },

    orderConfirmationDone : function()
    {
        this.subtaskDone(this.itemsPurchased);
    },

    doDeviceScan: function(successCallback)
    {
        this._devItems = new ttItemList();
        this._devItems.addFilter(new ttDependencyTreeBuilder());
        this._task.doDeviceScan(successCallback, errorAbortTask, this._devItems);
    },

    getDeviceContentIDList : function(devItems, body, list)
    {
        var me = this;
        for each (var item in devItems)
        {
            if (item.idurl in list)
            {
                let previousItem = list[item.idurl];
                if (previousItem.newerThan(item))
                {
                    body.contentidlist.content.(@idurl == previousItem.idurlWithVersion).@idurl = item.idurlWithVersion;
                    list[item.idurl] = item;
                }
            }
            else
            {
                var content = <content/>;
                content.@idurl = item.idurlWithVersion;
                if(item instanceof ttMap)
                    content.@regions = item.regions;
                body.contentidlist.appendChild(content);

                list[item.idurl] = item;
            }
        }
    },

    __end: null
}
extend(Checkout, ttSubTask);
