/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/


function DownloadPromotions()
{
    ttSubTask.apply(this, arguments);
    this._sb = getStringBundle("chrome://tthome/locale/ui/subtask/downloadPromotions/downloadPromotions.properties");
}
DownloadPromotions.prototype =
{
    name  : "downloadPromotions",
    _page : null,
    _sb   : null,
    _devItems : null,

    start : function()
    {
        ttSubTask.prototype.start.apply(this, arguments);
        this._task.needDevice(makeCallback(this, this.promotionsLoad));
    },

    promotionsLoad : function()
    {
        this._ttwizard.loadPage(
            "chrome://tthome/content/ui/remoteContentPage/remoteContentPage.xul",
            {
                title: this._sb.GetStringFromName("page.title"),
                subtitle: this._sb.GetStringFromName("page.subtitle"),
                continueLabel: this._sb.GetStringFromName("continue.label"),
                ignoreBrowserHistory: true
            },
            makeCallback(this, this.promotionsPrecond),
            null,
            makeCallback(this, this.promotionsDone)
        );
    },

    promotionsPrecond : function(aPage)
    {
        this.networkCall(this._task.needLogin(makeCallback(this, this.promotionAsyncLoad, aPage)));
    },
promotionAsyncLoad : function(aPage)
    {
        this._page = aPage;

        this._task.doDeviceScan(makeCallback(this, this.deviceScanDone), errorAbortTask);
    },
deviceScanDone : function()
    {
        var currentTheme = "";
        try {
            currentTheme = prefService().getCharPref("general.skins.selectedSkin")
        } catch (e) { unimportantError(e); }

        var dev = gApplication.deviceManager.activeDevice;
        var args = {
            deviceModel: dev.model.id,
            deviceSerial: dev.serial,
            deviceCode: dev.deviceCode,
            deviceCapabilities: gApplication.deviceManager.activeDeviceCapabilitiesString,
            deviceParameters: gApplication.deviceManager.activeDeviceParametersString,
            deviceNavcoreVersion: dev.versionNavCore,
            operatingSystem : gApplication.settings.operatingSystemServerName,
            theme : currentTheme
        };
        addSubscriptionsRPCArgs(args);

        var headers = {
            "Content-Type": "text/xml",
            "TT-Generic-ClientVersion": gApplication.settings.version,
            "TT-Generic-Language": gApplication.settings.networkLocale,
            "TT-Generic-Country": gApplication.settings.country
        };
        for (let [name, value] in Iterator(args))
            headers["TT-Param-" + name] = value;

        var uploadBody = <promotionParams><contentidlist/></promotionParams>;
        var lmg = dev.latestMapGuarantee;
        uploadBody.lmg = (lmg ? new XML(lmg) : <lmg/>);
        for each (let installLocation in gApplication.deviceManager.getInstallLocationsForDevice())
        {
            var devItems = gApplication.itemFactory.createItemListFromDLL(installLocation.deviceNavItems());
            for each (let item in devItems)
            {
                let content = <content idurl={item.idurlWithVersion}/>;
                if (item instanceof ttMap && item.sourceDLL.active == CI.ttINavItem.kActive)
                    content.@active = "true";

                uploadBody.contentidlist.appendChild(content);
            }
        }

        this._page.inputDone({
            startURL: rpc.serviceBaseURL("promotion").spec,
            headers: headers,
            body: uploadBody.toXMLString(),
            securityExceptionResolver: function() {
                return "DownloadPromotions/PromotionsLoad/ExpectedSecureConnection";
            }

        });
    },

    promotionsDone : function(promoID)
    {
        var dev = gApplication.deviceManager.activeDevice;
        var args =
        {
            promotionCode: promoID,
            deviceCode: dev.deviceCode,
            deviceSerial: dev.serial,
            deviceModel: dev.model.id,
            deviceCapabilities: gApplication.deviceManager.activeDeviceCapabilitiesString,
            deviceParameters: gApplication.deviceManager.activeDeviceParametersString,
            deviceNavcoreVersion : dev.versionNavCore,
            deviceNavcoreBuild : dev.buildNumberNavCore,
            publicBeta : prefService().getBoolPref("tomtom.app.showBetaVersions")
        };
        addSubscriptionsRPCArgs(args);

        var me = this;
        me.doDeviceScan(function()
        {
            var devItems = me._devItems.contents;
            var accountLicensedContentBody = <getaccountalicensedcontent><contentidlist/></getaccountalicensedcontent>;
            var accountLicensedContentList = {5: null};
            me.getDeviceContentIDList(devItems, accountLicensedContentBody, accountLicensedContentList);
            me.networkCall(rpc.call("shop", "getAccountLicensedContent",
                             args, accountLicensedContentBody, RPC.METHOD_POST & ~RPC.USE_CACHE, function(result)
                {

                    me.congratulationsScreen(gApplication.itemFactory.createItemListFromServer(result));
                },
                errorAbortTask));
        });
    },

    doDeviceScan: function(successCallback)
    {
        this._devItems = new ttItemList();
        this._devItems.addFilter(new ttDependencyTreeBuilder());
        this._task.doDeviceScan(successCallback, errorAbortTask, this._devItems);
    },

    getDeviceContentIDList : function(devItems, body, list)
    {
        var me = this;
        for each (var item in devItems)
        {
            if (item.idurl in list)
            {
                let previousItem = list[item.idurl];
                if (previousItem.newerThan(item))
                {
                    body.contentidlist.content.(@idurl == previousItem.idurlWithVersion).@idurl = item.idurlWithVersion;
                    list[item.idurl] = item;
                }
            }
            else
            {
                var content = <content/>;
                content.@idurl = item.idurlWithVersion;
                if(item instanceof ttMap)
                    content.@regions = item.regions;
                body.contentidlist.appendChild(content);

                list[item.idurl] = item;
            }
        }
    },

    congratulationsScreen : function(items)
    {
        var filteredItems = items.filter(function(item) item.locations.length > 0);
        var needInstall = filteredItems.length > 0;
        var itemlist = new ttItemList();
        itemlist.addItemListArray(items);

        this._ttwizard.loadPage(
            "chrome://tthome/content/ui/subtask/downloadPromotions/congratulationsPage.xul",
            {
                needInstall: needInstall,
                fullItemList: itemlist
            },
            null,
            null,
            makeCallback(this, this.congratulationsDone, filteredItems)
        );
    },

    congratulationsDone : function(items)
    {
        this.subtaskDone(items);
    },

    __end: null
}
extend(DownloadPromotions, ttSubTask);
