/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/



function LoginPage()
{
    ttPage.apply(this, arguments);
    this._stringBundle = getStringBundle("chrome://tthome/locale/ui/subtask/login/loginPage.properties");
}
LoginPage.prototype =
{
    _stringBundle: null,
    _inputData: null,
    _passwordBlurred: false,
    _disposition: "login",
    _items: null,
inputOnLoad: function(inputData)
    {
        var wizard = this._wizard();
        wizard._wizfooter.getLeftButton().hidden = true;
        wizard._wizfooter.getRightButton().hidden = true;
        
        this._inputData = inputData;
        this._passwordBlurred = false;
                
        var items = {
            emailField: E("login.email.field"),
            passwordField: E("login.password.field"),
            rememberCheckbox: E("login.remember.checkbox"),
            loggedInLabel: E("login.loggedin.label"),
            loggedInButton: E("logIn_but")
        };
            var firstFocusFlag = false;
            items.loggedInButton.disabled = true;
            this._items = items;

        function inputDataDefault(name, defaultValue)
        {
            return (name in inputData) ? inputData[name] : defaultValue;
        }

        var username = "";
        var password = "";
        var rememberPassword = true;
        try {
            username = gApplication.login.getDefaultUsername();
            password = gApplication.login.getPassword(username);
            rememberPassword = prefService().getBoolPref("tomtom.account.rememberPassword");
        } catch (e) {}

        username = inputDataDefault("username", username);
        rememberPassword = inputDataDefault("rememberPassword", rememberPassword);

        try {
            var emailField = items.emailField; 
            var passwordField = items.passwordField;
            if (rememberPassword)
            {
                emailField.value = username;
                passwordField.value = this._stringBundle.GetStringFromName("password.label");
            }
            else
            {
                this._setNormalTrue(passwordField);
                emailField.value = this._stringBundle.GetStringFromName("email.label");
                passwordField.value = this._stringBundle.GetStringFromName("password.label");
            }
            items.rememberCheckbox.checked = rememberPassword;
            emailField.focus();
        } catch (e) {
            error(e);
        }

        this.checkLogin();
    },
checkField: function(id)
    {
        if(id == "")
            return;
        var elem = E(id);
        var val = elem.value;
        this._setActive(elem);

        if(val.length >0 && (val == this._stringBundle.GetStringFromName("email.label") ||
            val == this._stringBundle.GetStringFromName("password.label")))
        {
            elem.value = "";
        }

        if(val == this._stringBundle.GetStringFromName("password.label"))
        {
            elem.type = "password";
        }
    },
result: function()
    {
        var items = this._items;
        return {
            disposition: this._disposition,
            username: items.emailField.value,
            password: items.passwordField.value,
            rememberPassword: items.rememberCheckbox.checked,
            loginBtn: items.loggedInButton
        };
    },
newAccount: function()
    {
        var items = this._items;

        this._disposition = "newAccount";
        if(!items.emailField.parentNode.hasAttribute("invalid"))
        {
            this._inputData.username = items.emailField.value.trim();
        }
        else
        {
            this._inputData.username = "";
        }
        this._inputData.rememberPassword = items.rememberCheckbox.checked;

        this.continue();
    },
checkLogin: function()
    {
        if (gApplication.login.isLoggedIn)
        {
            var username = gApplication.login.currentUsername;
            window.page.subtitle = this._stringBundle.formatStringFromName("alreadyLoggedin", [username], 1);

        }
    },
forgotPassword: function()
    {
        this.validateFields();
        var emailField = this._items.emailField; 
        var username = emailField.value.trim();

        if (emailField.parentNode.getAttribute("invalid") || username == this._stringBundle.GetStringFromName("email.label"))
        {
            var title = this._stringBundle.GetStringFromName("invalidUsernameTitle");
            var text = this._stringBundle.GetStringFromName("invalidUsername");
            ttMessageService().alert(title, text);
            return;
        }
        else
        {
            top.gMainWindow.dialogs.openDialog(
                "chrome://tthome/content/ui/dialogs/login/resetPassword.xul",
                { username: username },
                null, null, false, top);
        }
    },
_checkEmailValid: function()
    {
        var emailField = this._items.emailField;
        try
        {
            checkEmailAddressSyntax(emailField.value.trim());
        }
        catch (e)
        {
            this._items.loggedInButton.disabled=true;
            unimportantError(e);
            if(!(emailField.value.trim() == "" || (emailField.value.trim() == this._stringBundle.GetStringFromName("email.label"))))
            {
                this._markInvalid(emailField);
            }
            else
            {
                this._markValid(emailField);
            }
            this._handleTextColors(emailField, "email.label");
            return false;
        }
        this._markValid(emailField);
        this._handleTextColors(emailField, "email.label");
        return true;
    },
_checkPasswordValid: function()
    {
        var passwordField = this._items.passwordField;
        var ok = (passwordField.value.trim().length > 0 && (passwordField.value.trim() != this._stringBundle.GetStringFromName("password.label")));

        if (ok || !this._passwordBlurred)
        {
            this._markValid(passwordField);
            this._items.loggedInButton.disabled=true;
        }
        else
        {
            this._items.loggedInButton.disabled=true;
            this._markInvalid(passwordField);
        }

        return ok;
    },
validateFields: function()
    {
        var emailValid = this._checkEmailValid();
        var passwordValid = this._checkPasswordValid();
        if(emailValid && passwordValid)
        {
            this._items.loggedInButton.disabled=false;
        }
        this.allowContinue(emailValid && passwordValid);
    },
blurPassword: function(event)
    {
        var passwordField = this._items.passwordField;
        if (event.target == passwordField) return;

        this._passwordBlurred = true;
        this.validateFields();
    },
_handleTextColors: function(field, label)
    {
        if(field.value.trim() == "" || field.value.trim() == this._stringBundle.GetStringFromName(label))
        {
            field.type = "text";
            this._setNormalTrue(field);
            field.value = this._stringBundle.GetStringFromName(label);
        }
        else
        {
            this._setTypeOn(field);
        }
    },
_setTypeOn: function(node)
    {
        if(!node.parentNode.hasAttribute("invalid"))
        {
            this._removeOtherAttributes(node, 'typed');
            node.parentNode.setAttribute("typed", "true");
        }
    },
_setNormalTrue: function(node)
    {
        this._removeOtherAttributes(node, 'normal');
        node.parentNode.setAttribute("normal", "true");
    },
_setActive: function(node)
    {
        this._removeOtherAttributes(node, 'active');
        node.parentNode.setAttribute("active", "true");
    },
_removeOtherAttributes: function(node, attr)
    {
        if(attr != 'typed')
        {
            node.parentNode.removeAttribute("typed");
        }

        if(attr != 'normal')
        {
            node.parentNode.removeAttribute("normal");
        }

        if(attr != 'active')
        {
            node.parentNode.removeAttribute("active");
        }

        if(attr != 'invalid')
        {
            node.parentNode.removeAttribute("invalid");
        }
    },
_markValid: function(node)
    {
        node.parentNode.removeAttribute("invalid");
    },
_markInvalid: function(node)
    {
        this._removeOtherAttributes(node, 'invalid');
        node.parentNode.setAttribute("invalid", "true");
    },
passwordKey: function(event)
    {
        if (event.keyCode == CI.nsIDOMKeyEvent.DOM_VK_RETURN)
        {
            this._tryLogIn();
        }
    },
_tryLogIn: function()
    {
        if (!this._checkEmailValid()) {
            ttMessageService().alert(
                this._stringBundle.GetStringFromName("invalidEmail"),
                this._stringBundle.GetStringFromName("invalidEmailMessage"));
            return;
        }

        if (!this._checkPasswordValid()) {
            ttMessageService().alert(
                this._stringBundle.GetStringFromName("invalidPassword"),
                this._stringBundle.GetStringFromName("invalidPasswordMessage"));
            return;
        }
        this._items.loggedInButton.disabled=false;
        this._disposition = "login";
      
        this.continue();
    }
}
extend(LoginPage, ttPage);
window.page = new LoginPage();