/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/



function LoginSubTask(inputData)
{
    ttSubTask.apply(this, arguments);
    this._stringBundle = getStringBundle("chrome://tthome/locale/ui/subtask/login/loginSubTask.properties");
}
LoginSubTask.prototype =
{
    name: "login",
    _stringBundle: null,
    _inputData: null,
start: function(inputData)
    {
        ttSubTask.prototype.start.apply(this, arguments);

        this._inputData = inputData;

        if (inputData["createAccount"])
            this._newAccount();
        else
            this._loginSubTaskLoad();
    },
_loginSubTaskLoad: function()
    {
        this._ttwizard.loadPage(
            this._constructPageURL("loginPage"),
            this._inputData,
            makeCallback(this, this._loginPageAsyncLoad),
            makeCallback(this, this._loginPageCheck),
            makeCallback(this, this._loginPageDone));
    },
_loginPageAsyncLoad: function(pageHandle)
    {
        pageHandle.inputDone();
    },
_loginPageCheck: function(result, successCallback, errorCallback, returnCallback)
    {
        var disposition = result["disposition"]
        assert(disposition == "login" || disposition == "newAccount", "Unknown or undefined disposition.");
        if (disposition == "login")
        {
            this._tryLogin(result, successCallback, errorCallback, returnCallback);
        }
        else if (disposition == "newAccount")
        {
            successCallback();
        }
    },
_loginPageDone: function(result)
    {
        var disposition = result["disposition"];
        assert(disposition == "login" || disposition == "newAccount", "Unknown or undefined disposition.");
        if (disposition == "newAccount")
        {
            this._newAccount();
        }
        else
        {
            this.subtaskDone();
        }
    },
_tryLogin: function(result, successCallback, errorCallback, returnCallback)
    {
        var username = result["username"];
        var password = result["password"];
        var remember = result["rememberPassword"];
        var loginButton = result["loginBtn"];

        loginButton.disabled = true;
        loginButton.setAttribute('busy','true');
       
        gApplication.login.login(username, password,
        makeCallback(this, function()
            {
                try
                {
                    this._rememberPassword(remember, username, password);
                    successCallback();
                } catch (e) {
                    errorCallback(e);
                }
            }),
        makeCallback(this, function(e)
            {
                try {
                    if ((e.code == 401) || (e.code == 104))
                    {
                        var text = this._stringBundle.formatStringFromName("error401Login", [username], 1);
                        var title = this._stringBundle.GetStringFromName("error401LoginTitle");
                        ttMessageService().alert(title, text);
                    }
                    else if (e.code == 402)
                    {
                        loginButton.disabled = false;
                        var text = this._stringBundle.formatStringFromName("error402Login", [username], 1);
                        var title = this._stringBundle.GetStringFromName("error402LoginTitle");
                        ttMessageService().alert(title, text);
                    }
                    else
                    {
                        errorCallback(e);
                        return;
                    }
                } catch (e) {
                    errorCallback(e);
                    return;
                }
                loginButton.setAttribute('busy','false');
                returnCallback();
            }));
                
    },
_rememberPassword: function(remember, username, password)
    {
        var dev = gApplication.deviceManager.activeDevice;
        try {
            if (dev)
                prefService().setCharPref("tomtom.device." + dev.id + ".username", username);
            prefService().setCharPref("tomtom.account.username", username);
        } catch (e) { unimportantError(e); }

        try {
            prefService().setBoolPref("tomtom.account." + username + ".rememberPassword", remember);
            prefService().setBoolPref("tomtom.account.rememberPassword", remember);

            if (remember)
            {
                gApplication.login.savePassword(username, password);
            }
            else
            {
                gApplication.login.deletePassword(username);
            }
        } catch (e) {
            error (e);
        }
    },
_newAccount: function()
    {
        this._ttwizard.loadPage(
            this._constructPageURL("newAccountPage"),
            this._inputData,
            makeCallback(this, this._newAccountPageAsyncLoad),
            makeCallback(this, this._newAccountPageCheck),
            makeCallback(this, this._newAccountPageDone));
    },
_newAccountPageAsyncLoad: function(pageHandle)
    {
        pageHandle.inputDone();
    },
_newAccountPageCheck: function(result, successCallback, errorCallback, returnCallback)
    {
        var username = result['username'];
        var password = result['password'];
        var remember = result['rememberPassword'];
        var tomtomNews = result['tomtomNews'];
        var tomtomWork = result['tomtomWork'];
        var firstName = result['firstName'];
        var lastName = result['surName'];
        var country = result['country'];
        var createAccountBtn = result["createAccountBtn"];
        var gender = result["gender"];
        
        createAccountBtn.disabled = true;
        createAccountBtn.setAttribute('busy','true');
        
        var publicBeta = prefService().getBoolPref("tomtom.app.showBetaVersions");

        this.networkCall(rpc.call("account", "checkUsernameExists", {
            username: username
        }, null, RPC.METHOD_GET & ~RPC.USE_CACHE,
        makeCallback(this, function(result)
        {
            if (result.exists[0] == "true")
            {
                ttMessageService().alert(this._stringBundle.GetStringFromName("username"),
                                         this._stringBundle.GetStringFromName("usernameExists"));
                returnCallback();
                createAccountBtn.setAttribute('busy','false');
                createAccountBtn.disabled = false;
                return;
            }

            this.networkCall(rpc.call("account", "createAccount", {
                username: username,
                password: password,
                emailAddress: username,
                publicBeta: publicBeta,
                firstName: firstName,
                lastName: lastName,
                country: country
            }, null, RPC.METHOD_POST,
            makeCallback(this, function()
            {
                prefService().setCharPref("tomtom.server.country", country);
                this.networkCall(rpc.login(username, password,
                makeCallback(this, function()
                {
                    gApplication.login._setTo(username, password);
                    gMainWindow._updateLogInUser(this._inputData["username"]);
                    gMainWindow._accountObserver = true; 
                    this._rememberPassword(remember, username, password);

                    this.networkCall(rpc.call("account", "updateAccount", {
                        gender: gender,
                        newsletter: tomtomNews,
                        worknewsletter: tomtomWork,
                        publicBeta: publicBeta
                    }, null, RPC.METHOD_POST,
                    makeCallback(this, function()
                    {
                        this._inputData["password"] = password;
                        this._inputData["firstName"] = firstName;
                        this._newAccountSuccess();
                    }),
                    makeCallback(this, function(e)
                    {
                        errorCallback(e);
                    })));
                }),
                makeCallback(this, function(e)
                {
errorCallback(e);
                })));
                createAccountBtn.setAttribute('busy','false');
            }),
            makeCallback(this, function (e)
            {
                createAccountBtn.setAttribute('busy','false');
                createAccountBtn.disabled = false;
                errorCallback(e);
            })));

        }),
        makeCallback(this, function(e)
        {
            errorCallback(e);
        })));
    },
_newAccountSuccess: function()
    {
        this._ttwizard.loadPage(
            this._constructPageURL("thankYouPage"),
            this._inputData,
            makeCallback(this, this._thankYouPageAsyncLoad),
            makeCallback(this, this._thankYouPageCheck),
            makeCallback(this, this._thankYouPageDone));
    },
    
    _thankYouPageAsyncLoad: function(pageHandle)
    {
        pageHandle.inputDone();
    },
_thankYouPageCheck: function(result, successCallback, errorCallback, returnCallback)
    {
        var username = this._inputData.username;
        var password = this._inputData.password;

        gApplication.login.login(username, password,
                        makeCallback(this, function() 
                        { 
                            successCallback();
                        }),
                        makeCallback(this, function(e) 
                        {
                            errorCallback(e);
                        }));
    },
    
    _thankYouPageDone: function()
    {
        this.subtaskDone();
    },
    
    _newAccountPageDone: function(result)
    {   
        this.subtaskDone();
    }
    
}
extend(LoginSubTask, ttSubTask);
