/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/


function ManageItems()
{
    ttSubTask.apply(this, arguments);
    this._stringBundle = getStringBundle("chrome://tthome/locale/ui/subtask/manageItems/selectItems.properties");
}
ManageItems.prototype =
{
    name : "manageItems",
    _stringBundle : null,
    _page : null,
    _localItems : null,
    _action : null,
_observedDevice : null,

    start : function()
    {
        ttSubTask.prototype.start.apply(this, arguments);
        window.addEventListener("unload", makeCallback(this, function()
        {
            gApplication.deviceManager.removeObserver(this);
            this.observeDevice(null);
        }), false);

        this.selectItemsLoad();

    },

    selectItemsLoad : function()
    {
        this._ttwizard.loadPage(
            this._constructPageURL("selectItems"),
            null,
            makeCallback(this, this.selectItemsLoadAsync),
            makeCallback(this, this.selectItemsCheck),
            makeCallback(this, this.selectItemsDone)
            );
    },

    selectItemsLoadAsync : function(page)
    {
        this._page = page;
        gApplication.deviceManager.addObserver(this);
        this.observeDevice(gApplication.deviceManager.activeDevice);

        var deviceItems = null;
        var device = gApplication.deviceManager.activeDevice;
        if (device)
        {
            deviceItems = new ttItemList();
            deviceItems.addFilter(new ManageItemsFilter());
            deviceItems.addFilter(new ttDependencyTreeBuilder());
        }

        var localItems = this._localItems;
        if (!this._localItems)
        {
            localItems = new ttItemList();
            localItems.addFilter(new ManageItemsFilter());
            localItems.addFilter(new ttDependencyTreeBuilder());
            localItems.addItemListFromDLL(gApplication.homeServices.localNavItems());

            this._localItems = localItems;
        }

        page.inputStart([deviceItems, localItems]);
        if (deviceItems)
            this._task.doDeviceScan(makeCallback(page, page.inputDone), errorAbortTask, deviceItems);
        else
            page.inputDone();
    },

    observeDevice : function(device)
    {
        if (this._observedDevice)
        {
            try {
                this._observedDevice.installLocations().removeObserver(this);
            } catch (e) {
}
        }
        this._observedDevice = device;
        if (this._observedDevice)
            this._observedDevice.installLocations().addObserver(this);
    },

    changed : function(deviceChanged, locationChanged)
    {
        if (!deviceChanged)
            return;

        this.observeDevice(gApplication.deviceManager.activeDevice);
        this.selectItemsLoadAsync(this._page);
    },

    selectItemsCheck : function(result, successCallback, errorCallback, returnCallback)
    {
        let [action, items, criticals] = result;

        if (action == "removedevice" || action == "removelocal")
        {
            let criticalMessages = [];
            if (action == "removedevice")
            {
                if (criticals.map)
                    criticalMessages.push(this._stringBundle.GetStringFromName("reallyDeleteWarningTextAllMaps"));
                if (criticals.voice)
                    criticalMessages.push(this._stringBundle.GetStringFromName("reallyDeleteWarningTextAllVoices"));
                if (criticals.program)
                    criticalMessages.push(this._stringBundle.GetStringFromName("reallyDeleteWarningTextApplications"));
            }

            let warningText;
            if (criticalMessages.length)
            {
                warningText = this._stringBundle.formatStringFromName("reallyDeleteWarningTextCriticals",
                                      [criticalMessages.join("\n")], 1);
            }
            else
            {
                warningText = this._stringBundle.GetStringFromName("reallyDeleteWarningText" + (action == "removedevice" ? "Device" : "Computer"));
            }

            ttMessageService().confirm(
                this._stringBundle.GetStringFromName("reallyDeleteWarningTitle"),
                warningText,
                null, null,
                successCallback,
                returnCallback);
        }
        else
            successCallback();
    },

    selectItemsDone : function(result)
    {
        let [action, items] = result;
        this._action = action;
        gApplication.deviceManager.removeObserver(this);

        switch (action)
        {
            case "removedevice":
                this._ttwizard.addCommit(
                    makeCallback(this, this.removeDeviceCommit),
                    items,
                    this._stringBundle.GetStringFromName("device.commit.name"),
                    this.commitPriority,
                    'removedevice');
                this.selectCopyLoad(items);
                break;
            case "removelocal":
                this._ttwizard.addCommit(
                    makeCallback(this, this.removeLocalCommit),
                    items,
                    this._stringBundle.GetStringFromName("local.commit.name"),
                    this.commitPriority,
                    'removelocal');
                this.subtaskDone();
                break;
            case "copytopc":
                this._ttwizard.addCommit(
                    makeCallback(this, this.copyToPCCommit),
                    items,
                    this._stringBundle.GetStringFromName("copyItems.commit.name"),
                    this.commitPriority + 0.1,
                    'copytopc');
                this.subtaskDone();
                break;
        }
    },

    selectCopyLoad : function(removeItems)
    {
        var localItems = gApplication.itemFactory.createItemListFromDLL(gApplication.homeServices.localNavItems());
        var localItemsMap = new Map();
        for each (var item in localItems)
            localItemsMap.addValue(item.idurl, item);

        var askCopyItems = removeItems.filter(function(element)
        {
            if (!element.sourceDLL.canCopyToComputer || (element instanceof ttNavItem && element.isIncompleteInstall))
                return false;

            return (!localItemsMap.hasValue(element.idurl) ||
                    element.newerThan(localItemsMap.getValue(element.idurl)));
        });

        if (askCopyItems.length == 0)
        {
            this.selectCopyDone([]);
            return;
        }

        var itemlist = new ttItemList();
        itemlist.addFilter(new ttDependencyTreeBuilder());
        itemlist.addItemListArray(askCopyItems);

        this._ttwizard.loadPage(
            this._constructPageURL("selectCopy"),
            itemlist,
            null,
            makeCallback(this, ttNavItem.copyToPCCheck),
            makeCallback(this, this.selectCopyDone)
            );
    },
selectCopyDone : function(items)
    {
        if (items.length > 0)
        {
            this._ttwizard.addCommit(
                makeCallback(this, this.copyToPCCommit),
                items,
                this._stringBundle.GetStringFromName("backupItems.commit.name"),
                this.commitPriority + 0.1);
        }
        this.subtaskDone();
    },
    added : function(installLocation)
    {
        this.selectItemsLoadAsync(this._page);
    },

    removed : function(installLocation)
    {
        this.selectItemsLoadAsync(this._page);
        if(this._ttwizard)
        {
            let commitPage = this._ttwizard._wizbrowser.contentWindow.page;
            if (commitPage && this._action != 'removelocal')
            {
                if(!commitPage._done && commitPage._disconnect)
                {
                    commitPage._disconnect = false;
                    commitPage.disturbedDownload(true);
                    commitPage.loadDisconnectDeviceErrorMessage();
                }
            }
        }
    },
    repopulated : function()
    {
        this.selectItemsLoadAsync(this._page);
    },
    QueryInterface : function(iid)
    {
        if (iid.equals(CI.ttICollectionObserver) ||
            iid.equals(CI.nsISupportsWeakReference) ||
            iid.equals(CI.nsISupports))
            return this;
        throw Components.results.NS_ERROR_NO_INTERFACE;
    },

    copyToPCCommit : function(items, progressCallback, successCallback, errorCallback)
    {
        var me = this;
        return ttNavItem.copyToPC(items, progressCallback, function()
        {
            successCallback(me._stringBundle.GetStringFromName("copyItems.commit.done"));
        },
        errorCallback);
    },
removeDeviceCommit : function(result, progressCallback, successCallback, errorCallback)
    {
        var devItems = gApplication.deviceManager.activeInstallLocation.deviceNavItems();
        var iterator = new ArrayIterator(result, function(item) { return item.sourceDLL; });
        var deleter = devItems.createItemDeleter(iterator);
        var dl = new DeleteListener(progressCallback, successCallback, errorCallback);
        dl.successText = this._stringBundle.formatStringFromName("commit.done",
                         [result.length], 1);
        return new Abortable(deleter.doDelete(dl));
    },
removeLocalCommit : function(result, progressCallback, successCallback, errorCallback)
    {
        var localItems = gApplication.homeServices.localNavItems();
        var iterator = new ArrayIterator(result, function(item) { return item.sourceDLL; });
        var deleter = localItems.createItemDeleter(iterator);
        var dl = new DeleteListener(progressCallback, successCallback, errorCallback);
        dl.successText = this._stringBundle.formatStringFromName("commit.done",
                         [result.length], 1);
        return new Abortable(deleter.doDelete(dl));
    },

    __end : null
}
extend(ManageItems, ttSubTask);
function ManageItemsFilter()
{
    ttItemListFilter.apply(this);
}
ManageItemsFilter.prototype =
{
    checkItem : function(item)
    {
        if ((item.locationType != 1 && item.locationType != 2) ||
            (item.locationType == 1 && item instanceof ttPartialUpdate))
        {
            return [];
        }

        return [item];
    }
}
extend(ManageItemsFilter, ttItemListFilter);
