/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/


function SelectItems()
{
    ttPage.apply(this, arguments);
}

SelectItems.prototype =
{
    _result : null,
    _deviceItems : null,
    _localItems : null,

    inputOnLoad : function(args)
    {
        this._sb = E("stringbundle");
        E("device-listbox").addObserver(this);
        E("local-listbox").addObserver(this);

        E("tabbox").addEventListener("select", makeCallback(this, this.tabSelectionChanged), false);
        this.tabSelectionChanged();
        var dev = gApplication.deviceManager.activeDevice;
        if(dev && dev.isUsbDevice)
        {
			E("device-tab-icon").hidden=true;
            E("device-tab-icon-usb").hidden=false;
        }        
    },

    inputStart : function(lists)
    {
        let [deviceItems, localItems] = lists;

        E("device-tab").setAttribute("disabled", !deviceItems);
        if (deviceItems != this._deviceItems)
        {
            let listbox = E("device-listbox");
            listbox.clear();
            if (gApplication.deviceManager.getInstallLocationsForDevice().length <= 1)
                listbox.setAttribute("hiddencolumns", listbox.getAttribute("hiddencolumns") + " location");
            else
                listbox.setAttribute("hiddencolumns", listbox.getAttribute("hiddencolumns").replace(/\blocation\b/g, ""));

            this._deviceItems = deviceItems;

            if (deviceItems)
                listbox.addItemList(deviceItems);

            E("spacebar").clean();
        }

        E("local-tab").setAttribute("disabled", !localItems);
        if (localItems != this._localItems)
        {
            let listbox = E("local-listbox");
            listbox.clear();
            this._localItems = localItems;

            if (localItems)
            {
                listbox.addItemList(localItems);

                let localBytes = 0;
                for each (let item in localItems.contents)
                    localBytes += item.size;
                E("local-legend").value = this._sb.stringBundle.formatStringFromName("legend.pc", [ttItem.sizeBytesToMB(localBytes)], 1);
            }
        }

        this.updateSelected();

        if (!localItems)
            E("tabbox").selectedTab = E("device-tab");
        else if (!deviceItems)
            E("tabbox").selectedTab = E("local-tab");
    },

    addedListItem : function(listitem)
    {
        let canCopy = listitem.modelitem.sourceDLL.canCopyToComputer;
        let canRemove = listitem.modelitem.sourceDLL.canDeleteFromDevice;
        if (!canCopy && !canRemove)
            listitem.errorMessage = this._sb.getString("noRemoveOrCopy.message");
        else if (!canCopy)
            listitem.errorMessage = this._sb.getString("noCopy.message");
        else if (listitem.modelitem instanceof ttNavItem && listitem.modelitem.isIncompleteInstall)
            listitem.errorMessage = this._sb.getString("incompleteWarning.map");
        else if (!canRemove)
            listitem.errorMessage = this._sb.getString("noRemove.message");
        listitem.disabled = false;

        listitem.addToggleButton(
            null, null,
            makeCallback(this, this.updateSelected),
            makeCallback(this, this.updateSelected));
        if (listitem.modelitem instanceof ttMap && listitem.modelitem.locationType !== 2 && listitem.modelitem.locationType !== 3)
        {
            let maptype = listitem.modelitem.mapType;
            if (maptype == 2)
                listitem.addMapZoneLink(makeCallback(this, this.changeMapZone));
        }
    },

    changeMapZone : function(listitem)
    {
        top.gMainWindow.loadTaskByName("mapZones", { map : listitem.modelitem, devItems : this._deviceItems });
    },

    tabSelectionChanged : function()
    {
	    var dev = gApplication.deviceManager.activeDevice;
        let footer = this._wizard()._wizfooter;
        if (E("tabbox").selectedTab == E("device-tab"))
        {
            footer.getMiddleButton().notvisible = false;
            footer.getMiddleButton().label = this._sb.getString("removeFromDevice.button.label");
            footer.getRightButton().label = this._sb.getString("copyToComputer.button.label");
            if(dev && dev.isUsbDevice)
			{
                footer.getMiddleButton().notvisible = true;
				footer.getRightButton().notvisible = true;
			}
        }
        else if (E("tabbox").selectedTab == E("install-tab"))
        {
            top.gMainWindow.loadTaskByName("showLocalItems");
        }
        else
        {
		    footer.getRightButton().notvisible = false;
            footer.getMiddleButton().notvisible = true;
            footer.getRightButton().label = this._sb.getString("removeFromComputer.button.label");
        }

        this.disableButtons();
    },

    disableButtons : function()
    {
        let selectedTab = (E("tabbox").selectedTab == E("device-tab") ? "device" : "local");
        let selectedCount = E(selectedTab + "-listbox").selectedListitems.length;
        this.allowContinue(selectedCount > 0);
        this.allowMiddle(selectedCount > 0);
    },

    updateSelected : function()
    {
        E("spacebar").hidden = !this._deviceItems;
        if (this._updateTimeout)
            clearTimeout(this._updateTimeout);
        this._updateTimeout = setTimeout(makeCallback(this, this._updateSelectedInternal), 0);
    },

    _updateTimeout : null,
    _updateSelectedInternal : function()
    {
        this._updateTimeout = null;

        if (this._deviceItems)
        {
            let installLocations = {__proto__: null};
            for each (let installLocation in gApplication.deviceManager.getInstallLocationsForDevice())
                installLocations[installLocation.path.path] = installLocation;

            let data = ttNavItem.getInstalledSizeDelta([], E("device-listbox").selectedModelitems, this._deviceItems.contents);
            let spacebarData = [];
            for (let installLocationPath in data)
            {
                spacebarData.push({
                    installLocation: installLocations[installLocationPath],
                    selected: Math.max(0, -data[installLocationPath])
                });
            }
            
            var dev = gApplication.deviceManager.activeDevice;
            if(dev && dev.isUsbDevice)
            {
                spacebarData.push({
                    installLocation: "InDashDevice",
                    selected: null
                });
            }
            E("spacebar").setInstallLocationData(spacebarData);
        }

        var localBytes = 0;
        var localItems = 0;
        for each (let listitem in E("local-listbox").selectedListitems)
        {
            localBytes += listitem.modelitem.size;
            localItems++;
        }
        var none = this._sb.getString("none.selected");
        E("local-selected-label").value = this._sb.getFormattedString("local.selected.items", [localItems || none]);
        E("local-selected").value = ttItem.sizeBytesToMB(localBytes);
        this.disableButtons();
    },

    middleButtonClicked : function()
    {
        let noRemove = E("device-listbox").selectedListitems.filter(function(item) !item.modelitem.sourceDLL.canDeleteFromDevice);
        if (noRemove.length)
        {
            ttMessageService().alert(this._sb.getString("removeNotAllowed.title"),
                    this._sb.getFormattedString("removeNotAllowed.message", [noRemove.map(function(item) item.modelitem.name).join("\n")]),
                    MessageService.SEVERITY_WARNING);
            for each (let item in noRemove)
                item.selected = false;
        }
        else
        {
            let allMapsSelected = null;
            let allVoicesSelected = null;
            let programsSelected = false;
            for each (let item in E("device-listbox").listitems)
            {
                switch (item.modelitem.displayCategory)
                {
                    case "map":
                        allMapsSelected = (item.selected && allMapsSelected !== false);
                        break;
                    case "voice":
                        allVoicesSelected = (item.selected && allVoicesSelected !== false);
                        break;
                    case "program":
                        programsSelected = programsSelected || item.selected;
                        break;
                }
            }

            this._result = ["removedevice", E("device-listbox").selectedModelitems,
                            {map: allMapsSelected === true, voice: allVoicesSelected === true, program: programsSelected}];
            this.continue();
        }
    },

    rightButtonClicked : function()
    {
        if (E("tabbox").selectedTab == E("device-tab"))
        {
            let noCopy = E("device-listbox").selectedListitems.filter(function(item) !item.modelitem.sourceDLL.canCopyToComputer || item.modelitem.isIncompleteInstall);
            if (noCopy.length)
            {
                ttMessageService().alert(this._sb.getString("copyNotAllowed.title"),
                        this._sb.getFormattedString("copyNotAllowed.message", [noCopy.map(function(item) item.modelitem.name).join("\n")]),
                        MessageService.SEVERITY_WARNING);
                for each (let item in noCopy)
                    item.selected = false;
            }
            else
            {
                this._result = ["copytopc", E("device-listbox").selectedModelitems, null];
                this.continue();
            }
        }
        else
        {
            this._result = ["removelocal", E("local-listbox").selectedModelitems, null];
            this.continue();
        }
    },

    result : function()
    {
        return this._result;
    }
}
extend(SelectItems, ttPage);
window.page = new SelectItems();