/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/


function MapZones()
{
    ttSubTask.apply(this, arguments);
    this._stringBundle = getStringBundle("chrome://tthome/locale/ui/subtask/mapFlavours/mapFlavours.properties");
}
MapZones.prototype =
{
    _stringBundle : null,
    name : "mapZones",
_zones : null,
_originalmap : null,
_installedItems : null,
_devItems : null,
_removableItems : null,
_freeSpaceBuffer : null,
_additionalSizeRequirements : null,
_params : null,
_originalmapServerModel : null,
    _devItems : null,

    start : function(params)
    {
        this.networkCall(this._task.needLogin(makeCallback(this, this.getZones, params)));
    },

    getZones : function(params)
    {
        assert(params && params.item && params.item instanceof ttMap && params.item.basemap,
               "The item passed to the mapZones subtask must be a zoned/zoneable ttMap.");
        assert(typeof params.insideInstallFlow != "undefined", "The context for the map zones task should be specified in the parameters");
        this._params = params;

        if (params.insideInstallFlow)
            this.showZonesIntroScreen();
        else
            this.showZonesChoice();
    },
showZonesIntroScreen : function()
    {
        this._ttwizard.loadPage(
            this._constructPageURL("mapZoneIntroScreen"),
            null,
            makeCallback(this, this.zoneIntroScreenDone),
            null,
            makeCallback(this, this.showZonesChoice)
            );
    },
zoneIntroScreenDone : function(page)
    {
        page.inputDone();
    },
showZonesChoice : function()
    {
        this._ttwizard.loadPage(
            this._constructPageURL("mapZoneChoice"),
            { },
            makeCallback(this, this.showZonesChoiceLoadAsync),
            null,
            makeCallback(this, this.showZonesChoiceDone)
            );
    },
showZonesChoiceLoadAsync : function(page)
    {
        let params = this._params;
        this._originalmap = params.item;
        this._installedItems = params.installedItems;
        this._devItems = params.devItems;
        this._removableItems = params.removableItems;
        this._freeSpaceBuffer = params.freeSpaceBuffer;
        this._additionalSizeRequirements = params.additionalSizeRequirements;
        var dev = gApplication.deviceManager.activeDevice;

        var args =
        {
            deviceCode: dev.deviceCode,
            deviceSerial: dev.serial,
            deviceModel: dev.model.id,
            deviceCapabilities: gApplication.deviceManager.activeDeviceCapabilitiesString,
            deviceParameters: gApplication.deviceManager.activeDeviceParametersString,
            deviceNavcoreVersion : dev.versionNavCore,
            deviceNavcoreBuild : dev.buildNumberNavCore,
            publicBeta : prefService().getBoolPref("tomtom.app.showBetaVersions")
        };

        var me = this;
        addSubscriptionsRPCArgs(args);
        me.doDeviceScan(function()
        {
            var devItems = me._devItems.contents;
            var accountLicensedContentBody = <getaccountalicensedcontent><contentidlist/></getaccountalicensedcontent>;
            var accountLicensedContentList = {5: null};
            me.getDeviceContentIDList(devItems, accountLicensedContentBody, accountLicensedContentList);
            me.networkCall(rpc.call("shop", "getAccountLicensedContent",
            args, accountLicensedContentBody, RPC.METHOD_POST & ~RPC.USE_CACHE,
            function(result)
            {
                let items = gApplication.itemFactory.createItemListFromServer(result);
                me._zones = [];
                if (items.length > 0)
                {
                    for each (let item in items)
                    {
                        if (item.basemap && item.basemap == me._originalmap.basemap)
                        {
                            if (item.idstr != me._originalmap.idstr)
                                me._zones.push(item);
                            else
                                me._originalmapServerModel = item;
                        }
                    }
                }
                me._zones.unshift(me._originalmap);

                if (me._zones.length <= 1)
                    throw new InformativeException(me._stringBundle.GetStringFromName("zones.notfound"));

                page.inputDone({ maps : me._zones,
                     originalmapServerModel : me._originalmapServerModel,
                     originalmap : me._originalmap,
                     installedItems : me._installedItems,
                     devItems : me._devItems,
                     freeSpaceBuffer : me._freeSpaceBuffer,
                     additionalSizeRequirements :me._additionalSizeRequirements });
            },
            makeCallback(this, function(e)
            {
                if (params.insideInstallFlow)
                {
                    unimportantError(e);
                    this.subtaskDone(this._originalmap);
                }
                else
                    errorAbortTask(e);
            })));
            });
    },
showZonesChoiceDone : function(map)
    {
        this.subtaskDone(map);
    },
_willFullmapPotentiallyFit : function(fullmap)
    {
        let installedItems = this._installedItems.concat(fullmap);

        if (!this._removableItems)
        {
            this._removableItems = new ttItemList();
            this._task.addCleanupHandler(makeCallback(this, function() {
                                                          this._removableItems.destructor();
                                                      }));
            this._removableItems.addFilter(new ttNonRemovableFilter(installedItems));
            this._removableItems.addItemList(this._devItems);
        }
        let fullmapPotentiallyFits = true;

        let itemsPotentiallyFitByLocation = ttNavItem.willItemsFitByTargetLocation(
            installedItems,
            this._removableItems.contents,
            this._devItems.contents,
            this._additionalSizeRequirements,
            this._freeSpaceBuffer);

        let installTargets = ttNavItem.getItemsByTargetLocation([fullmap], this._devItems.contents);
        for (let target in installTargets)
        {
            fullmapPotentiallyFits = typeof itemsPotentiallyFitByLocation[target] == "undefined" ? true : itemsPotentiallyFitByLocation[target];

            if (!fullmapPotentiallyFits)
                break;
        }

        return fullmapPotentiallyFits;
    },

    doDeviceScan: function(successCallback)
    {
        this._devItems = new ttItemList();
        this._devItems.addFilter(new ttDependencyTreeBuilder());
        this._task.doDeviceScan(successCallback, errorAbortTask, this._devItems);
    },

    getDeviceContentIDList : function(devItems, body, list)
    {
        var me = this;
        for each (var item in devItems)
        {
            if (item.idurl in list)
            {
                let previousItem = list[item.idurl];
                if (previousItem.newerThan(item))
                {
                    body.contentidlist.content.(@idurl == previousItem.idurlWithVersion).@idurl = item.idurlWithVersion;
                    list[item.idurl] = item;
                }
            }
            else
            {
                var content = <content/>;
                content.@idurl = item.idurlWithVersion;
                if(item instanceof ttMap)
                    content.@regions = item.regions;
                body.contentidlist.appendChild(content);

                list[item.idurl] = item;
            }
        }
    },

    __end : null
}
extend(MapZones, ttSubTask);
