/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/


function URLHandlerInstall()
{
    ttSubTask.apply(this, arguments);
    this._stringBundle = getStringBundle("chrome://tthome/locale/ui/subtask/urlHandlerInstall/urlHandlerInstall.properties");

}
URLHandlerInstall.prototype =
{
    name : "urlHandlerInstall",
    uri : null,
    one : null,
    _stringBundle : null,

    start : function(args)
    {
        ttSubTask.prototype.start.apply(this, arguments);

        this.uri = args.uri;
        this.one = args.one;

        this.selectInstallLoad();
    },

    selectInstallLoad : function(items)
    {
        this._ttwizard.loadPage(
            this._constructPageURL("selectInstall"),
            null,
            makeCallback(this, this.selectInstallAsyncLoad),
            makeCallback(this, this.selectInstallCheck),
            makeCallback(this, this.selectInstallDone)
            );
    },

    selectInstallAsyncLoad : function(page)
    {
        var list = new ttItemList();
        list.addFilter(new ttDuplicateFilter());
        list.addFilter(new ttDependencyTreeBuilder());
        page.inputStart(list);
        try {
            var me = this;
            var dev = gApplication.deviceManager.lastDevice;
            if (dev)
            {
                this.uri += this.uri.indexOf("?") == -1 ? "?" : "&";
                this.uri += "deviceModel=" + dev.model.id +
                            "&deviceNavcoreVersion=" + dev.versionNavCore;
            }

            var headerArgs = {
                "TT-Param-DeviceParameters": gApplication.deviceManager.activeDeviceParametersString,
                "TT-Param-DeviceCapabilities": gApplication.deviceManager.activeDeviceCapabilitiesString,
                "TT-Param-publicBeta": prefService().getBoolPref("tomtom.app.showBetaVersions")
            };
            var call = this.networkCall(fetchHTTP.fetch(this.uri, headerArgs, false, null, function(xml)
            {
                if (typeof(xml) != "xml")
                    throw "Item server gave malformed response, cannot process item";
                if (me.one)
                {
                    if (xml.localName() != "item")
                        throw "Item server response invalid, <item> expected";
                    var item = gApplication.itemFactory.createItemFromServer(xml.elements()[0]);
                    list.addItem(item);
                }
                else
                {
                    if (xml.localName() != "items" || !xml.items.contentlist)
                        throw "Item server response invalid, <items><contentlist> expected";
                    list.addItemListFromServer(xml);
                }
                me.checkItems(me.uri, list);
                page.inputDone();
            },
            errorAbortTask));
        } catch(e) 
        { 
            unimportantError(e);
            throw "Item ID " + this.uri + " not supported";
        }
    },
checkItems : function(url, list)
    {
        var allowedTypes = {
            satimage: true, voice: true, poi: true, itinerary: true, htmlpack: true, colorscheme: true,
            carsymbol: true, startupscreen: true, suspendscreen: true, ringtone: true,
            messagereceived: true, warningsound: true, arrivalsound: true, buddysymbol: true,
            homesymbol: true, shutdownsound: true, startupsound: true, favouritesymbol: true };
        var items = list.contents;
        for (var i = 0; i < items.length; i++)
        {
            var item = items[i];
            if (item instanceof ttNavItem &&
                item.typeID in allowedTypes)
                continue;
            var host = makeSafe.url(url).hostPort;
            var tomtomIDDomain = ".id.tomtom.com";
            try {
tomtomIDDomain = prefService().getCharPref("tomtom.server.eiffelIDTrustedDomain");
            } catch (e) {}
            var isTomTom = host.substr(0 - tomtomIDDomain.length) == tomtomIDDomain;
            if ( ! isTomTom)
            {
                list.clear();{}
throw "This item cannot be installed for security or policy reasons";
            }
        }
    },

    selectInstallCheck : function(result, successCallback, errorCallback)
    {
        if (result.length == 0)
        {
            errorCallback("Either select an item to install, or abort");
            return;
        }
        successCallback();
    },

    selectInstallDone : function(result)
    {
        this.subtaskDone(result);
    },

    __end : null
}
extend(URLHandlerInstall, ttSubTask);
