/*
  @Copyright (C) 2006-2012, TomTom International B.V. All rights reserved.
*/


const ApplicationUpdater =
{
    _updateRunner: null,
    _running: null,
initiateUpdate: function (aAutomatic)
    {
if (this._running)
        {
            if (!aAutomatic)
            {
                var stringBundle = getStringBundle("chrome://tthome/locale/ui/updater/appUpdater.properties");
                ttMessageService(window).alert(stringBundle.GetStringFromName("updaterAlreadyRunning.title"),
                                               stringBundle.GetStringFromName("updaterAlreadyRunning.prompt"));
            }
            return;
        }

        this._running = true;
        this._updateRunner = new ApplicationUpdateRunner(aAutomatic,
                                    makeCallback(this, this.updateFinished));
        this._updateRunner.start();
    },

    updateFinished: function ()
    {
        this._running = false;
        this._updateRunner = null;
    }
};
function ApplicationUpdateRunner(aAutomatic, aFinishedCallback)
{
    this._mAutomatic = aAutomatic;
    this._mFinishedCallback = aFinishedCallback;
    const kStringBundleURI = "chrome://tthome/locale/ui/updater/appUpdater.properties";
    this._mStringbundle = getStringBundle(kStringBundleURI);
}

ApplicationUpdateRunner.prototype =
{
    _kUpdaterDeniedPref: "tomtom.updater.denied",

    _mStringbundle     : null,
    _mAutomatic        : false,
    _mUpdateSpec       : null,
    _mDownloadCancelled: false,
    _mProgressMeter    : null,
    _mFinishedCallback : null,

    start: function ()
    {
        this._queryForUpdate(makeCallback(this, this._queryUpdateSuccessCallback),
                             makeCallback(this, this._queryUpdateErrorCallback));
    },

    downloadCancel: function ()
    {
        this._mDownloadCancelled = true;
    },
_queryForUpdate: function (aSuccessCallback, aErrorCallback)
    {
        var operatingSystem = gApplication.settings.operatingSystemServerName;

        var callParams =
        {
            clientVersion     : gApplication.settings.version,
            operatingSystem   : operatingSystem,
            OSVersion         : window.navigator.oscpu,
            autoUpdate        : this._mAutomatic,
            extendedVersioning: "true"
        };

        try {
            callParams.publicBeta = prefService().getBoolPref("tomtom.app.showBetaVersions");
        } catch(e) {
            unimportantError(e);
        }
        try {
            var deviceID = prefService().getCharPref("tomtom.lastConnectedDevice.id");
            callParams.deviceModel = prefService().getCharPref("tomtom.device." +
                                        deviceID + ".modelID");
        } catch(e) {}

        var updateQuery = rpc.call("client", "getLatestClientVersion", callParams,
                                   null, RPC.METHOD_GET, aSuccessCallback, aErrorCallback);
    },

    _queryUpdateSuccessCallback: function (aUpdateSpec)
    {
        try
        {
            var updateSpec = new AppUpdateSpec(aUpdateSpec);
        }
        catch (exception)
        {
            unimportantError(exception);
            this._alertUserUpdateFailedDialog(null, exception.toString());

            this._finished();
            return;
        }

        try
        {
            if (!versionNewerThan(updateSpec.version, gApplication.settings.version))
            {
                if (!this._mAutomatic)
                    ttMessageService(window).alert(this._mStringbundle.GetStringFromName("noNewVersionAvailable.title"),
                                                   this._mStringbundle.formatStringFromName("noNewVersionAvailable.prompt.label", [gApplication.settings.version], 1));

                this._cleanPermanentUpdaterDir();

                this._finished();
                return;
            }
            try
            {
                if (this._mAutomatic &&
                    prefService().getCharPref(this._kUpdaterDeniedPref) == updateSpec.version)
                {
this._finished();
                    return;
                }
            }
            catch (exception)
            {
            }
            var updateFile = this._getExistingUpdateFile(updateSpec.url.fileName);

            if (updateFile)
            {
if (this._verifyDownload(updateFile, updateSpec))
                {
                    if (this._askIfShouldInstall(this._mAutomatic && !updateSpec.required, updateSpec.required, updateSpec.beta, gApplication.settings.version, updateSpec.version) &&
                        this._askIfCanRestart())
                        this._installUpdate(updateFile);

                    this._finished();
                    return;
                }
                else
                {
try
                    {
                        updateFile.remove(false);
                    }
                    catch (exception)
                    {
                        unimportantError(exception);
                    }
                }
            }

            this._mUpdateSpec = updateSpec;
if (!this._mAutomatic || updateSpec.beta)
            {
                var promptID = "updatePermission";
                promptID += (this._mAutomatic ? "Automatic" : "Manual");
                promptID += (updateSpec.required ? "Critical" : "Optional");
                promptID += (updateSpec.beta ? "Beta" : "Release");

                var buttonID = "updatePermission";
                if (updateSpec.required)
                    buttonID += "Critical";
                else
                    buttonID += (this._mAutomatic ? "Automatic" : "Manual") + "Optional";

                var titleID = ((updateSpec.required) ? "updatePermission.title" : "shouldInstall.title");

                if (!ttMessageService(window).confirm(this._mStringbundle.GetStringFromName(titleID),
                                                      this._mStringbundle.formatStringFromName(promptID + ".prompt.label", [updateSpec.version, gApplication.settings.version], 2),
                                                      this._mStringbundle.GetStringFromName(buttonID + ".leftbutton.label"),
                                                      this._mStringbundle.GetStringFromName(buttonID + ".rightbutton.label")))
                {
                    this._finished();
                    if (updateSpec.required)
                        gApplication.quitApplication(true, false);

                    return;
                }
            }
            this._downloadUpdate(updateSpec.url,
                                 makeCallback(this, this._downloadUpdateSuccessCallback),
                                 makeCallback(this, this._downloadUpdateErrorCallback));
            if (updateSpec.required)
                window.open("chrome://tthome/content/ui/dialogs/appUpdate/appUpdate.xul","title","centerscreen ,modal,chrome=yes,titlebar=no"); 

        }
        catch (exception)
        {
            unimportantError(exception);
            this._alertUserUpdateFailedDialog(updateSpec.url.spec, exception.toString());

            this._finished();
        }
    },

    _queryUpdateErrorCallback: function (aError)
    {
        if (this._mAutomatic)
            unimportantError(aError);
        else
            error(aError);

        this._finished();
    },
_downloadUpdate: function (aURL, aSuccessCallback, aErrorCallback)
    {
        if (!BackgroundProcessManager.isEnabled())
            prefService().setBoolPref("tomtom.app.autostart", false);
            
        var downloader = new Downloader();

        downloader.resumeDownloadEnabled = true;

        this._mProgressMeter = gMainWindow.statusbar.showProgress(this._mStringbundle.GetStringFromName("downloadingUpdate.label"));

        var request = downloader.download(aURL, gApplication.settings.osTempDir,
                                          makeCallback(this._mProgressMeter, this._mProgressMeter.progress),
                                          aSuccessCallback,
                                          aErrorCallback);
    },

    _downloadUpdateSuccessCallback: function (aTargetFile)
    {
        this._mProgressMeter.done();

        if (!aTargetFile || (aTargetFile && !this._verifyDownload(aTargetFile, this._mUpdateSpec)))
        {
this._alertUserUpdateFailedDialog(this._mUpdateSpec.url.spec, null);

            this._finished();
            return;
        }
var updateFile = this._moveUpdateToPermanentDir(aTargetFile, this._mUpdateSpec.url.fileName);

        if (!this._mUpdateSpec.required)
        {
            if (!this._askIfShouldInstall(this._mAutomatic && !this._mUpdateSpec.required, this._mUpdateSpec.required, this._mUpdateSpec.beta, gApplication.settings.version, this._mUpdateSpec.version) ||
            !this._askIfCanRestart())
            {
                this._finished();
                return;
            }
        }

        this._installUpdate(updateFile);
    },

    _downloadUpdateErrorCallback: function (aError)
    {
        this._mProgressMeter.done();
        if (!this._mDownloadCancelled)
        {
            unimportantError(aError);
            this._alertUserUpdateFailedDialog(this._mUpdateSpec.url.spec, aError.toString());
        }

        this._finished();
    },
_verifyDownload: function (aFile, aSpec)
    {
        return true;
    },
_cleanPermanentUpdaterDir: function ()
    {
        for (let child in getFilesInDir(this._getUpdaterDir()))
        {
            try
            {
                removeFile(child);
            }
            catch (e)
            {
unimportantError(e);
            }
        }
    },
_getExistingUpdateFile: function (aFileLeafName)
    {
        var updaterDir = this._getUpdaterDir();
        updaterDir.append(aFileLeafName);

        return (updaterDir.exists() ? updaterDir : null);
    },
_moveUpdateToPermanentDir: function (aUpdateFile, aFileName)
    {
        assert(aFileName);

        if (!aUpdateFile || !aUpdateFile.isFile())
            throw new InvalidValue("Update file is not a file.", aUpdateFile);

        moveFile(aUpdateFile, this._getUpdaterDir(), aFileName);
        var updateFile = this._getUpdaterDir().clone();
        updateFile.append(aFileName);

        if (!updateFile.exists())
            throw new Exception("Moving update file to permanent directory failed. File \"" + updateFile.path + "\" does not exist");

        return updateFile;
    },
_getUpdaterDir: function ()
    {
        var updaterDir = gApplication.settings.profileDir;{}
updaterDir.append("Updates");
        ensureDirExists(updaterDir);

        if (!updaterDir.isDirectory())
            throw new InvalidValue("Updates directory is invalid.", updaterDir);

        return updaterDir;
    },
_askIfCanRestart: function ()
    {
        var wnd = gApplication.getSomeMainWindow();
        return wnd.interrupt(
                function() {},
                this._mStringbundle.GetStringFromName("interrupt.reason"),
                this._mStringbundle.GetStringFromName("interrupt.target")
        );
    },
_askIfShouldInstall: function (aDenyUpdateOption, aCritical, aBeta, aCurrentVersion, aNewVersion)
    {
        assert(aCurrentVersion);
        assert(aNewVersion);

        var promptServ = promptService();

        var flags = promptServ.BUTTON_POS_0 * promptServ.BUTTON_TITLE_IS_STRING;
        if (!aCritical)
            flags += promptServ.BUTTON_POS_1 * promptServ.BUTTON_TITLE_IS_STRING;

        var denyButtonLabel = null;

        if (aDenyUpdateOption) {
            flags += promptServ.BUTTON_POS_2 * promptServ.BUTTON_TITLE_IS_STRING;
            denyButtonLabel = this._mStringbundle.GetStringFromName("shouldInstallOptional.middlebutton.label");
        }

        var promptID = "shouldInstall";
        promptID += (aCritical ? "Critical" : "Optional");
        promptID += (aBeta ? "Beta" : "Release");

        var titleID = ((aCritical) ? "updatePermission.title" : "shouldInstall.title");

        var buttonNo = promptServ.confirmEx(
                window,
                this._mStringbundle.GetStringFromName(titleID),
                this._mStringbundle.formatStringFromName(promptID + ".prompt", [aNewVersion, aCurrentVersion], 2),
                flags,
                this._mStringbundle.GetStringFromName(aCritical ? "shouldInstallCritical.rightbutton.label" : "shouldInstallOptional.rightbutton.label"),
                this._mStringbundle.GetStringFromName("shouldInstallOptional.leftbutton.label"),
                denyButtonLabel,
                null,
                { value: false }
        );

        switch (buttonNo)
        {
            case 0:
                return true;
            case 1:
return aCritical;
            case 2:{}
prefService().setCharPref(this._kUpdaterDeniedPref, aNewVersion);

                return false;
            default:
                return false;
        }
    },
_alertUserUpdateFailedDialog: function (aDownloadLocation, aMessage)
    {
        window.openDialog("chrome://tthome/content/ui/updater/updateFailedDialog.xul",
                          "updateFailedDialog",
                          "chrome,centerscreen",
                          aDownloadLocation,
                          aMessage);
    },
_installUpdate: function (aUpdateFile)
    {
assert(aUpdateFile);

        if (gApplication.settings.isOSWindows())
        {
            aUpdateFile.QueryInterface(CI.nsILocalFile).launch();
            gApplication.quitApplication(true, false);
        }
        else if (gApplication.settings.isOSMacOSX())
        {
            var unzipDir = this._getUpdaterDir();
            unzipDir.append("extract");
            unzipDir = createUniqueDir(unzipDir);

            var unzip = getFile("/usr/bin/unzip");
            var unzipProc = Cc["@mozilla.org/process/util;1"].createInstance(CI.nsIProcess);
            unzipProc.init(unzip);
            unzipProc.run(true, [aUpdateFile.path, "-d", unzipDir.path], 3);

            if (unzipProc.exitValue != 0)
                throw new Exception(this._mStringbundle.GetStringFromName("updateExtractionFailed.label"));
            var installerFilePath=this._getUpdaterDir(); 
            installerFilePath.append("extract");
            installerFilePath.append("MacLatest.pkg");

            if(installerFilePath.exists())
            {
                var installerFile= Cc["@mozilla.org/file/local;1"].createInstance(CI.nsILocalFile);
                installerFile.initWithPath(installerFilePath.path);
                installerFile.launch();
                gApplication.quitApplication(true, false);
            }
            else
            {
                unzipDir.append("TomTom HOME.app");
                var installDir = gApplication.settings.installDir.parent;
                var installParentDir = installDir.parent;
                var tempDir = gApplication.settings.osTempDir;
                tempDir.append("previous_TomTom_HOME");
                tempDir = createUniqueDir(tempDir);
                moveFile(installDir, tempDir, null);
                moveFile(unzipDir, installParentDir, installDir.leafName);
                gApplication.quitApplication(true, true);
            }
        }
        else
        {
            throw new Exception(this._mStringbundle.GetStringFromName("unsupportedOSUpdateError"));
        }
    },

    _finished: function ()
    {
        this._mFinishedCallback();
    }
};
function AppUpdateSpec(aUpdateSpec)
{
    this._version = makeSafe.version(aUpdateSpec.version);
    this._url = makeSafe.url(aUpdateSpec.url);

    if (!(this._url instanceof CI.nsIURL))
        throw new InvalidValue("URL must be of type nsIURL", this._url);

    this._required = makeSafe.bool(aUpdateSpec.required);
    if ("beta" in aUpdateSpec)
      this._beta = makeSafe.bool(aUpdateSpec.beta);
    if ("forcedloginenabled" in aUpdateSpec)
    prefService().setBoolPref("tomtom.server.forcedLogin", makeSafe.bool(aUpdateSpec.forcedloginenabled)); 
}

AppUpdateSpec.prototype =
{
    _version : null,
    _url     : null,
    _required: null,
    _beta    : false,

    get version()
    {
        return this._version;
    },

    get url()
    {
        return this._url;
    },

    get required()
    {
        return this._required;
    },

    get beta()
    {
        return this._beta;
    }
};
